/*----------------------------------------------------------------------------/
/  FatFs - Generic FAT Filesystem Module  R0.16                               /
/-----------------------------------------------------------------------------/
/
/ Copyright (C) 2025, ChaN, all right reserved.
/
/ FatFs module is an open source software. Redistribution and use of FatFs in
/ source and binary forms, with or without modification, are permitted provided
/ that the following condition is met:
/
/ 1. Redistributions of source code must retain the above copyright notice,
/    this condition and the following disclaimer.
/
/ This software is provided by the copyright holder and contributors "AS IS"
/ and any warranties related to this software are DISCLAIMED.
/ The copyright owner or contributors be NOT LIABLE for any damages caused
/ by use of this software.
/
/----------------------------------------------------------------------------*/

/**
*
* MODIFICATION HISTORY:
*
* Ver	Who	Date		Changes
* ----- ---- -------- -------------------------------------------------------
* 3.9   mn   04/18/18 Resolve build warnings for xilffs library
* 4.2   aru  07/10/19 Fix Coverity warnings
*       mn   08/16/19 Initialize Status variables with failure values
* 4.3   mn   02/05/20 Add support for Multi Partitions
*       mn   04/23/20 Add partition 0 for supporting default partition
* 4.7   sk   11/11/21 Add DCache invalidate for last unaligned byte count
*                     (< 512 bytes) in f_read().
* 5.1   ro   06/12/23 Added support for system device-tree flow.
* 5.2   sk   07/11/24 Add f_ioctl interface to perform UFS specific configs.
*       sk   07/11/24 Update drive number calculation logic to support multiple
*                     digit drive numbers.
* 5.6   an   11/12/25 Update the FatFS version to R0.16
* 5.6   vlt  01/30/26 Fixed codespell errors
******************************************************************************/
#include "xparameters.h"
#include "xstatus.h"

#ifdef SDT
#include "xilffs_config.h"
#endif

#if (defined FILE_SYSTEM_INTERFACE_SD) || (defined FILE_SYSTEM_INTERFACE_RAM)
#include "ff.h"			/* Declarations of FatFs API */
#include "diskio.h"		/* Declarations of device I/O functions */
#include "xil_printf.h"
#include "xil_cache.h"
#ifdef XPAR_XUFSPSXC_NUM_INSTANCES
#include <stdlib.h>
#endif
#include <string.h>


/*--------------------------------------------------------------------------

   Module Private Definitions

---------------------------------------------------------------------------*/

#if FF_DEFINED != 80386	/* Revision ID */
#error Wrong include file (ff.h).
#endif


/* Limits and boundaries */
#define MAX_DIR		0x200000		/* Max size of FAT directory (byte) */
#define MAX_DIR_EX	0x10000000		/* Max size of exFAT directory (byte) */
#define MAX_FAT12	0xFF5			/* Max FAT12 clusters (differs from specs, but right for real DOS/Windows behavior) */
#define MAX_FAT16	0xFFF5			/* Max FAT16 clusters (differs from specs, but right for real DOS/Windows behavior) */
#define MAX_FAT32	0x0FFFFFF5		/* Max FAT32 clusters (not defined in specs, practical limit) */
#define MAX_EXFAT	0x7FFFFFFD		/* Max exFAT clusters (differs from specs, implementation limit) */


/* Character code support macros */
#define IsUpper(c)		((c) >= 'A' && (c) <= 'Z')
#define IsLower(c)		((c) >= 'a' && (c) <= 'z')
#define IsDigit(c)		((c) >= '0' && (c) <= '9')
#define IsSeparator(c)	((c) == '/' || (c) == '\\')
#define IsTerminator(c)	((UINT)(c) < (FF_USE_LFN ? ' ' : '!'))
#define IsSurrogate(c)	((c) >= 0xD800 && (c) <= 0xDFFF)
#define IsSurrogateH(c)	((c) >= 0xD800 && (c) <= 0xDBFF)
#define IsSurrogateL(c)	((c) >= 0xDC00 && (c) <= 0xDFFF)


/* Additional file access control and file status flags for internal use */
#define FA_SEEKEND	0x20	/* Seek to end of the file on file open */
#define FA_MODIFIED	0x40	/* File has been modified */
#define FA_DIRTY	0x80	/* FIL.buf[] needs to be written-back */


/* Additional file attribute bits for internal use */
#define AM_VOL		0x08	/* Volume label */
#define AM_LFN		0x0F	/* LFN entry */
#define AM_MASK		0x3F	/* Mask of defined bits in FAT */
#define AM_MASKX	0x37	/* Mask of defined bits in exFAT */


/* Name status flags in fn[11] */
#define NSFLAG		11		/* Index of the name status byte */
#define NS_LOSS		0x01	/* Out of 8.3 format */
#define NS_LFN		0x02	/* Force to create LFN entry */
#define NS_LAST		0x04	/* Last segment */
#define NS_BODY		0x08	/* Lower case flag (body) */
#define NS_EXT		0x10	/* Lower case flag (ext) */
#define NS_DOT		0x20	/* Dot entry */
#define NS_NOLFN	0x40	/* Do not find LFN */
#define NS_NONAME	0x80	/* Not followed */


/* exFAT directory entry types */
#define	ET_BITMAP	0x81	/* Allocation bitmap */
#define	ET_UPCASE	0x82	/* Up-case table */
#define	ET_VLABEL	0x83	/* Volume label */
#define	ET_FILEDIR	0x85	/* File and directory */
#define	ET_STREAM	0xC0	/* Stream extension */
#define	ET_FILENAME	0xC1	/* Name extension */


/* FatFs refers the FAT structures as simple byte array instead of structure member
/ because the C structure is not binary compatible between different platforms */

#define BS_JmpBoot			0		/* x86 jump instruction (3-byte) */
#define BS_OEMName			3		/* OEM name (8-byte) */
#define BPB_BytsPerSec		11		/* Sector size [byte] (WORD) */
#define BPB_SecPerClus		13		/* Cluster size [sector] (BYTE) */
#define BPB_RsvdSecCnt		14		/* Size of reserved area [sector] (WORD) */
#define BPB_NumFATs			16		/* Number of FATs (BYTE) */
#define BPB_RootEntCnt		17		/* Size of root directory area for FAT [entry] (WORD) */
#define BPB_TotSec16		19		/* Volume size (16-bit) [sector] (WORD) */
#define BPB_Media			21		/* Media descriptor byte (BYTE) */
#define BPB_FATSz16			22		/* FAT size (16-bit) [sector] (WORD) */
#define BPB_SecPerTrk		24		/* Number of sectors per track for int13h [sector] (WORD) */
#define BPB_NumHeads		26		/* Number of heads for int13h (WORD) */
#define BPB_HiddSec			28		/* Volume offset from top of the drive (DWORD) */
#define BPB_TotSec32		32		/* Volume size (32-bit) [sector] (DWORD) */
#define BS_DrvNum			36		/* Physical drive number for int13h (BYTE) */
#define BS_NTres			37		/* WindowsNT error flag (BYTE) */
#define BS_BootSig			38		/* Extended boot signature (BYTE) */
#define BS_VolID			39		/* Volume serial number (DWORD) */
#define BS_VolLab			43		/* Volume label string (8-byte) */
#define BS_FilSysType		54		/* Filesystem type string (8-byte) */
#define BS_BootCode			62		/* Boot code (448-byte) */
#define BS_55AA				510		/* Boot signature (WORD, for VBR and MBR) */

#define BPB_FATSz32			36		/* FAT32: FAT size [sector] (DWORD) */
#define BPB_ExtFlags32		40		/* FAT32: Extended flags (WORD) */
#define BPB_FSVer32			42		/* FAT32: Filesystem version (WORD) */
#define BPB_RootClus32		44		/* FAT32: Root directory cluster (DWORD) */
#define BPB_FSInfo32		48		/* FAT32: Offset of FSINFO sector (WORD) */
#define BPB_BkBootSec32		50		/* FAT32: Offset of backup boot sector (WORD) */
#define BS_DrvNum32			64		/* FAT32: Physical drive number for int13h (BYTE) */
#define BS_NTres32			65		/* FAT32: Error flag (BYTE) */
#define BS_BootSig32		66		/* FAT32: Extended boot signature (BYTE) */
#define BS_VolID32			67		/* FAT32: Volume serial number (DWORD) */
#define BS_VolLab32			71		/* FAT32: Volume label string (8-byte) */
#define BS_FilSysType32		82		/* FAT32: Filesystem type string (8-byte) */
#define BS_BootCode32		90		/* FAT32: Boot code (420-byte) */

#define BPB_ZeroedEx		11		/* exFAT: MBZ field (53-byte) */
#define BPB_VolOfsEx		64		/* exFAT: Volume offset from top of the drive [sector] (QWORD) */
#define BPB_TotSecEx		72		/* exFAT: Volume size [sector] (QWORD) */
#define BPB_FatOfsEx		80		/* exFAT: FAT offset from top of the volume [sector] (DWORD) */
#define BPB_FatSzEx			84		/* exFAT: FAT size [sector] (DWORD) */
#define BPB_DataOfsEx		88		/* exFAT: Data offset from top of the volume [sector] (DWORD) */
#define BPB_NumClusEx		92		/* exFAT: Number of clusters (DWORD) */
#define BPB_RootClusEx		96		/* exFAT: Root directory start cluster (DWORD) */
#define BPB_VolIDEx			100		/* exFAT: Volume serial number (DWORD) */
#define BPB_FSVerEx			104		/* exFAT: Filesystem version (WORD) */
#define BPB_VolFlagEx		106		/* exFAT: Volume flags (WORD, out of check sum calculation) */
#define BPB_BytsPerSecEx	108		/* exFAT: Log2 of sector size in unit of byte (BYTE) */
#define BPB_SecPerClusEx	109		/* exFAT: Log2 of cluster size in unit of sector (BYTE) */
#define BPB_NumFATsEx		110		/* exFAT: Number of FATs (BYTE) */
#define BPB_DrvNumEx		111		/* exFAT: Physical drive number for int13h (BYTE) */
#define BPB_PercInUseEx		112		/* exFAT: Percent in use (BYTE, out of check sum calculation) */
#define BPB_RsvdEx			113		/* exFAT: Reserved (7-byte) */
#define BS_BootCodeEx		120		/* exFAT: Boot code (390-byte) */

#define DIR_Name			0		/* Short file name (11-byte) */
#define DIR_Attr			11		/* Attribute (BYTE) */
#define DIR_NTres			12		/* Low case flags of SFN (BYTE) */
#define DIR_CrtTime10		13		/* Created time sub-second (BYTE) */
#define DIR_CrtTime			14		/* Created time (DWORD) */
#define DIR_LstAccDate		18		/* Last accessed date (WORD) */
#define DIR_FstClusHI		20		/* Higher 16-bit of first cluster (WORD) */
#define DIR_ModTime			22		/* Modified time (DWORD) */
#define DIR_FstClusLO		26		/* Lower 16-bit of first cluster (WORD) */
#define DIR_FileSize		28		/* File size (DWORD) */
#define LDIR_Ord			0		/* LFN: LFN order and LLE flag (BYTE) */
#define LDIR_Attr			11		/* LFN: LFN attribute (BYTE) */
#define LDIR_Type			12		/* LFN: Entry type (BYTE) */
#define LDIR_Chksum			13		/* LFN: Checksum of the SFN (BYTE) */
#define LDIR_FstClusLO		26		/* LFN: MBZ field (WORD) */
#define XDIR_Type			0		/* exFAT: Type of exFAT directory entry (BYTE) */
#define XDIR_NumLabel		1		/* exFAT: Number of volume label characters (BYTE) */
#define XDIR_Label			2		/* exFAT: Volume label (11-WORD) */
#define XDIR_CaseSum		4		/* exFAT: Sum of case conversion table (DWORD) */
#define XDIR_NumSec			1		/* exFAT: Number of secondary entries (BYTE) */
#define XDIR_SetSum			2		/* exFAT: Sum of the set of directory entries (WORD) */
#define XDIR_Attr			4		/* exFAT: File attribute (WORD) */
#define XDIR_CrtTime		8		/* exFAT: Created time (DWORD) */
#define XDIR_ModTime		12		/* exFAT: Modified time (DWORD) */
#define XDIR_AccTime		16		/* exFAT: Last accessed time (DWORD) */
#define XDIR_CrtTime10		20		/* exFAT: Created time subsecond (BYTE) */
#define XDIR_ModTime10		21		/* exFAT: Modified time subsecond (BYTE) */
#define XDIR_CrtTZ			22		/* exFAT: Created timezone (BYTE) */
#define XDIR_ModTZ			23		/* exFAT: Modified timezone (BYTE) */
#define XDIR_AccTZ			24		/* exFAT: Last accessed timezone (BYTE) */
#define XDIR_GenFlags		33		/* exFAT: General secondary flags (BYTE) */
#define XDIR_NumName		35		/* exFAT: Number of file name characters (BYTE) */
#define XDIR_NameHash		36		/* exFAT: Hash of file name (WORD) */
#define XDIR_ValidFileSize	40		/* exFAT: Valid file size (QWORD) */
#define XDIR_FstClus		52		/* exFAT: First cluster of the file data (DWORD) */
#define XDIR_FileSize		56		/* exFAT: File/Directory size (QWORD) */

#define SZDIRE				32		/* Size of a directory entry */
#define DDEM				0xE5	/* Deleted directory entry mark set to DIR_Name[0] */
#define RDDEM				0x05	/* Replacement of the character collides with DDEM */
#define LLEF				0x40	/* Last long entry flag in LDIR_Ord */

#define FSI_LeadSig			0		/* FAT32 FSI: Leading signature (DWORD) */
#define FSI_StrucSig		484		/* FAT32 FSI: Structure signature (DWORD) */
#define FSI_Free_Count		488		/* FAT32 FSI: Number of free clusters (DWORD) */
#define FSI_Nxt_Free		492		/* FAT32 FSI: Last allocated cluster (DWORD) */
#define FSI_TrailSig		508		/* FAT32 FSI: Trailing signature (DWORD) */

#define MBR_Table			446		/* MBR: Offset of partition table in the MBR */
#define SZ_PTE				16		/* MBR: Size of a partition table entry */
#define PTE_Boot			0		/* MBR PTE: Boot indicator */
#define PTE_StHead			1		/* MBR PTE: Start head in CHS */
#define PTE_StSec			2		/* MBR PTE: Start sector in CHS */
#define PTE_StCyl			3		/* MBR PTE: Start cylinder in CHS */
#define PTE_System			4		/* MBR PTE: System ID */
#define PTE_EdHead			5		/* MBR PTE: End head in CHS */
#define PTE_EdSec			6		/* MBR PTE: End sector in CHS */
#define PTE_EdCyl			7		/* MBR PTE: End cylinder in CHS */
#define PTE_StLba			8		/* MBR PTE: Start in LBA */
#define PTE_SizLba			12		/* MBR PTE: Size in LBA */

#define GPTH_Sign			0		/* GPT HDR: Signature (8-byte) */
#define GPTH_Rev			8		/* GPT HDR: Revision (DWORD) */
#define GPTH_Size			12		/* GPT HDR: Header size (DWORD) */
#define GPTH_Bcc			16		/* GPT HDR: Header BCC (DWORD) */
#define GPTH_CurLba			24		/* GPT HDR: This header LBA (QWORD) */
#define GPTH_BakLba			32		/* GPT HDR: Another header LBA (QWORD) */
#define GPTH_FstLba			40		/* GPT HDR: First LBA for partition data (QWORD) */
#define GPTH_LstLba			48		/* GPT HDR: Last LBA for partition data (QWORD) */
#define GPTH_DskGuid		56		/* GPT HDR: Disk GUID (16-byte) */
#define GPTH_PtOfs			72		/* GPT HDR: Partition table LBA (QWORD) */
#define GPTH_PtNum			80		/* GPT HDR: Number of table entries (DWORD) */
#define GPTH_PteSize		84		/* GPT HDR: Size of table entry (DWORD) */
#define GPTH_PtBcc			88		/* GPT HDR: Partition table BCC (DWORD) */
#define SZ_GPTE				128		/* GPT PTE: Size of a GPT partition table entry */
#define GPTE_PtGuid			0		/* GPT PTE: Partition type GUID (16-byte) */
#define GPTE_UpGuid			16		/* GPT PTE: Partition unique GUID (16-byte) */
#define GPTE_FstLba			32		/* GPT PTE: First LBA of partition (QWORD) */
#define GPTE_LstLba			40		/* GPT PTE: Last LBA of partition (QWORD) */
#define GPTE_Flags			48		/* GPT PTE: Partition flags (QWORD) */
#define GPTE_Name			56		/* GPT PTE: Partition name */


/* Post process on fatal error in the file operations */
#define ABORT(fs, res)		{ fp->err = (BYTE)(res); LEAVE_FF(fs, res); }


/* Re-entrancy related */
#if FF_FS_REENTRANT
#if FF_USE_LFN == 1
#error Static LFN work area cannot be used in thread-safe configuration
#endif
#define LEAVE_FF(fs, res)	{ unlock_volume(fs, res); return res; }
#else
#define LEAVE_FF(fs, res)	return res
#endif


/* Definitions of logical drive to physical location conversion */
#if FF_MULTI_PARTITION
#define LD2PD(vol) VolToPart[vol].pd	/* Get physical drive number from the mapping table */
#define LD2PT(vol) VolToPart[vol].pt	/* Get partition number from the mapping table (0:auto search, 1-:forced partition number) */
#else
#define LD2PD(vol) (BYTE)(vol)	/* Each logical drive is associated with the same physical drive number */
#define LD2PT(vol) 0			/* Auto partition search */
#endif


/* Definitions of sector size */
#if (FF_MAX_SS < FF_MIN_SS) || (FF_MAX_SS != 512 && FF_MAX_SS != 1024 && FF_MAX_SS != 2048 && FF_MAX_SS != 4096) || (FF_MIN_SS != 512 && FF_MIN_SS != 1024 && FF_MIN_SS != 2048 && FF_MIN_SS != 4096)
#error Wrong sector size configuration
#endif
#if FF_MAX_SS == FF_MIN_SS
#define SS(fs)	((UINT)FF_MAX_SS)	/* Fixed sector size */
#else
#define SS(fs)	((fs)->ssize)	/* Variable sector size */
#endif


/* Timestamp */
#if FF_FS_NORTC == 1
#if FF_NORTC_YEAR < 1980 || FF_NORTC_YEAR > 2107 || FF_NORTC_MON < 1 || FF_NORTC_MON > 12 || FF_NORTC_MDAY < 1 || FF_NORTC_MDAY > 31
#error Invalid FF_FS_NORTC settings
#endif
#define GET_FATTIME()	((DWORD)(FF_NORTC_YEAR - 1980) << 25 | (DWORD)FF_NORTC_MON << 21 | (DWORD)FF_NORTC_MDAY << 16)
#else
#define GET_FATTIME()	get_fattime()
#endif


/* File lock controls */
#if FF_FS_LOCK
#if FF_FS_READONLY
#error FF_FS_LOCK must be 0 at read-only configuration
#endif
typedef struct {	/* Open object identifier with status */
	FATFS *fs;		/*  Object ID 1, volume (NULL:blank entry) */
	DWORD clu;		/*  Object ID 2, containing directory (0:root) */
	DWORD ofs;		/*  Object ID 3, offset in the directory */
	UINT ctr;		/*  Object open status, 0:none, 0x01..0xFF:read mode open count, 0x100:write mode */
} FILESEM;
#endif


/* SBCS up-case tables (\x80-\xFF) */
#define TBL_CT437  {0x80,0x9A,0x45,0x41,0x8E,0x41,0x8F,0x80,0x45,0x45,0x45,0x49,0x49,0x49,0x8E,0x8F, \
		0x90,0x92,0x92,0x4F,0x99,0x4F,0x55,0x55,0x59,0x99,0x9A,0x9B,0x9C,0x9D,0x9E,0x9F, \
		0x41,0x49,0x4F,0x55,0xA5,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF, \
		0xE0,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,0xEE,0xEF, \
		0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF}
#define TBL_CT720  {0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x8E,0x8F, \
		0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9A,0x9B,0x9C,0x9D,0x9E,0x9F, \
		0xA0,0xA1,0xA2,0xA3,0xA4,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF, \
		0xE0,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,0xEE,0xEF, \
		0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF}
#define TBL_CT737  {0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x8E,0x8F, \
		0x90,0x92,0x92,0x93,0x94,0x95,0x96,0x97,0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87, \
		0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x8E,0x8F,0x90,0x91,0xAA,0x92,0x93,0x94,0x95,0x96, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF, \
		0x97,0xEA,0xEB,0xEC,0xE4,0xED,0xEE,0xEF,0xF5,0xF0,0xEA,0xEB,0xEC,0xED,0xEE,0xEF, \
		0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF}
#define TBL_CT771  {0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x8E,0x8F, \
		0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9A,0x9B,0x9C,0x9D,0x9E,0x9F, \
		0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x8E,0x8F, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,0xDA,0xDB,0xDC,0xDC,0xDE,0xDE, \
		0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9A,0x9B,0x9C,0x9D,0x9E,0x9F, \
		0xF0,0xF0,0xF2,0xF2,0xF4,0xF4,0xF6,0xF6,0xF8,0xF8,0xFA,0xFA,0xFC,0xFC,0xFE,0xFF}
#define TBL_CT775  {0x80,0x9A,0x91,0xA0,0x8E,0x95,0x8F,0x80,0xAD,0xED,0x8A,0x8A,0xA1,0x8D,0x8E,0x8F, \
		0x90,0x92,0x92,0xE2,0x99,0x95,0x96,0x97,0x97,0x99,0x9A,0x9D,0x9C,0x9D,0x9E,0x9F, \
		0xA0,0xA1,0xE0,0xA3,0xA3,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xB5,0xB6,0xB7,0xB8,0xBD,0xBE,0xC6,0xC7,0xA5,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF, \
		0xE0,0xE1,0xE2,0xE3,0xE5,0xE5,0xE6,0xE3,0xE8,0xE8,0xEA,0xEA,0xEE,0xED,0xEE,0xEF, \
		0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF}
#define TBL_CT850  {0x43,0x55,0x45,0x41,0x41,0x41,0x41,0x43,0x45,0x45,0x45,0x49,0x49,0x49,0x41,0x41, \
		0x45,0x92,0x92,0x4F,0x4F,0x4F,0x55,0x55,0x59,0x4F,0x55,0x4F,0x9C,0x4F,0x9E,0x9F, \
		0x41,0x49,0x4F,0x55,0xA5,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0x41,0x41,0x41,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0x41,0x41,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xD1,0xD1,0x45,0x45,0x45,0x49,0x49,0x49,0x49,0xD9,0xDA,0xDB,0xDC,0xDD,0x49,0xDF, \
		0x4F,0xE1,0x4F,0x4F,0x4F,0x4F,0xE6,0xE8,0xE8,0x55,0x55,0x55,0x59,0x59,0xEE,0xEF, \
		0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF}
#define TBL_CT852  {0x80,0x9A,0x90,0xB6,0x8E,0xDE,0x8F,0x80,0x9D,0xD3,0x8A,0x8A,0xD7,0x8D,0x8E,0x8F, \
		0x90,0x91,0x91,0xE2,0x99,0x95,0x95,0x97,0x97,0x99,0x9A,0x9B,0x9B,0x9D,0x9E,0xAC, \
		0xB5,0xD6,0xE0,0xE9,0xA4,0xA4,0xA6,0xA6,0xA8,0xA8,0xAA,0x8D,0xAC,0xB8,0xAE,0xAF, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBD,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC6,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xD1,0xD1,0xD2,0xD3,0xD2,0xD5,0xD6,0xD7,0xB7,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF, \
		0xE0,0xE1,0xE2,0xE3,0xE3,0xD5,0xE6,0xE6,0xE8,0xE9,0xE8,0xEB,0xED,0xED,0xDD,0xEF, \
		0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xEB,0xFC,0xFC,0xFE,0xFF}
#define TBL_CT855  {0x81,0x81,0x83,0x83,0x85,0x85,0x87,0x87,0x89,0x89,0x8B,0x8B,0x8D,0x8D,0x8F,0x8F, \
		0x91,0x91,0x93,0x93,0x95,0x95,0x97,0x97,0x99,0x99,0x9B,0x9B,0x9D,0x9D,0x9F,0x9F, \
		0xA1,0xA1,0xA3,0xA3,0xA5,0xA5,0xA7,0xA7,0xA9,0xA9,0xAB,0xAB,0xAD,0xAD,0xAE,0xAF, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0xB6,0xB6,0xB8,0xB8,0xB9,0xBA,0xBB,0xBC,0xBE,0xBE,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC7,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xD1,0xD1,0xD3,0xD3,0xD5,0xD5,0xD7,0xD7,0xDD,0xD9,0xDA,0xDB,0xDC,0xDD,0xE0,0xDF, \
		0xE0,0xE2,0xE2,0xE4,0xE4,0xE6,0xE6,0xE8,0xE8,0xEA,0xEA,0xEC,0xEC,0xEE,0xEE,0xEF, \
		0xF0,0xF2,0xF2,0xF4,0xF4,0xF6,0xF6,0xF8,0xF8,0xFA,0xFA,0xFC,0xFC,0xFD,0xFE,0xFF}
#define TBL_CT857  {0x80,0x9A,0x90,0xB6,0x8E,0xB7,0x8F,0x80,0xD2,0xD3,0xD4,0xD8,0xD7,0x49,0x8E,0x8F, \
		0x90,0x92,0x92,0xE2,0x99,0xE3,0xEA,0xEB,0x98,0x99,0x9A,0x9D,0x9C,0x9D,0x9E,0x9E, \
		0xB5,0xD6,0xE0,0xE9,0xA5,0xA5,0xA6,0xA6,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC7,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xD0,0xD1,0xD2,0xD3,0xD4,0x49,0xD6,0xD7,0xD8,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF, \
		0xE0,0xE1,0xE2,0xE3,0xE5,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xEB,0xDE,0xED,0xEE,0xEF, \
		0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF}
#define TBL_CT860  {0x80,0x9A,0x90,0x8F,0x8E,0x91,0x86,0x80,0x89,0x89,0x92,0x8B,0x8C,0x98,0x8E,0x8F, \
		0x90,0x91,0x92,0x8C,0x99,0xA9,0x96,0x9D,0x98,0x99,0x9A,0x9B,0x9C,0x9D,0x9E,0x9F, \
		0x86,0x8B,0x9F,0x96,0xA5,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF, \
		0xE0,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,0xEE,0xEF, \
		0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF}
#define TBL_CT861  {0x80,0x9A,0x90,0x41,0x8E,0x41,0x8F,0x80,0x45,0x45,0x45,0x8B,0x8B,0x8D,0x8E,0x8F, \
		0x90,0x92,0x92,0x4F,0x99,0x8D,0x55,0x97,0x97,0x99,0x9A,0x9D,0x9C,0x9D,0x9E,0x9F, \
		0xA4,0xA5,0xA6,0xA7,0xA4,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF, \
		0xE0,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,0xEE,0xEF, \
		0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF}
#define TBL_CT862  {0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x8E,0x8F, \
		0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9A,0x9B,0x9C,0x9D,0x9E,0x9F, \
		0x41,0x49,0x4F,0x55,0xA5,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF, \
		0xE0,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,0xEE,0xEF, \
		0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF}
#define TBL_CT863  {0x43,0x55,0x45,0x41,0x41,0x41,0x86,0x43,0x45,0x45,0x45,0x49,0x49,0x8D,0x41,0x8F, \
		0x45,0x45,0x45,0x4F,0x45,0x49,0x55,0x55,0x98,0x4F,0x55,0x9B,0x9C,0x55,0x55,0x9F, \
		0xA0,0xA1,0x4F,0x55,0xA4,0xA5,0xA6,0xA7,0x49,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF, \
		0xE0,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,0xEE,0xEF, \
		0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF}
#define TBL_CT864  {0x80,0x9A,0x45,0x41,0x8E,0x41,0x8F,0x80,0x45,0x45,0x45,0x49,0x49,0x49,0x8E,0x8F, \
		0x90,0x92,0x92,0x4F,0x99,0x4F,0x55,0x55,0x59,0x99,0x9A,0x9B,0x9C,0x9D,0x9E,0x9F, \
		0x41,0x49,0x4F,0x55,0xA5,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF, \
		0xE0,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,0xEE,0xEF, \
		0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF}
#define TBL_CT865  {0x80,0x9A,0x90,0x41,0x8E,0x41,0x8F,0x80,0x45,0x45,0x45,0x49,0x49,0x49,0x8E,0x8F, \
		0x90,0x92,0x92,0x4F,0x99,0x4F,0x55,0x55,0x59,0x99,0x9A,0x9B,0x9C,0x9D,0x9E,0x9F, \
		0x41,0x49,0x4F,0x55,0xA5,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF, \
		0xE0,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,0xEE,0xEF, \
		0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF}
#define TBL_CT866  {0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x8E,0x8F, \
		0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9A,0x9B,0x9C,0x9D,0x9E,0x9F, \
		0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x8E,0x8F, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF, \
		0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9A,0x9B,0x9C,0x9D,0x9E,0x9F, \
		0xF0,0xF0,0xF2,0xF2,0xF4,0xF4,0xF6,0xF6,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF}
#define TBL_CT869  {0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x8E,0x8F, \
		0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9A,0x86,0x9C,0x8D,0x8F,0x90, \
		0x91,0x90,0x92,0x95,0xA4,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF, \
		0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF, \
		0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF, \
		0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xA4,0xA5,0xA6,0xD9,0xDA,0xDB,0xDC,0xA7,0xA8,0xDF, \
		0xA9,0xAA,0xAC,0xAD,0xB5,0xB6,0xB7,0xB8,0xBD,0xBE,0xC6,0xC7,0xCF,0xCF,0xD0,0xEF, \
		0xF0,0xF1,0xD1,0xD2,0xD3,0xF5,0xD4,0xF7,0xF8,0xF9,0xD5,0x96,0x95,0x98,0xFE,0xFF}


/* DBCS code range |----- 1st byte -----|  |----------- 2nd byte -----------| */
/*                  <------>    <------>    <------>    <------>    <------>  */
#define TBL_DC932 {0x81, 0x9F, 0xE0, 0xFC, 0x40, 0x7E, 0x80, 0xFC, 0x00, 0x00}
#define TBL_DC936 {0x81, 0xFE, 0x00, 0x00, 0x40, 0x7E, 0x80, 0xFE, 0x00, 0x00}
#define TBL_DC949 {0x81, 0xFE, 0x00, 0x00, 0x41, 0x5A, 0x61, 0x7A, 0x81, 0xFE}
#define TBL_DC950 {0x81, 0xFE, 0x00, 0x00, 0x40, 0x7E, 0xA1, 0xFE, 0x00, 0x00}


/* Macros for table definitions */
#define MERGE_2STR(a, b) a ## b
#define MKCVTBL(hd, cp) MERGE_2STR(hd, cp)




/*--------------------------------------------------------------------------

   Module Private Work Area

---------------------------------------------------------------------------*/
/* Remark: Variables defined here without initial value shall be guaranteed
/  zero/null at start-up. If not, the linker option or start-up routine is
/  not compliance with C standard. */

/*--------------------------------*/
/* File/Volume controls           */
/*--------------------------------*/

#if FF_VOLUMES < 1 || FF_VOLUMES > 175
#error Wrong FF_VOLUMES setting
#endif
static FATFS *FatFs[FF_VOLUMES];	/* Pointer to the filesystem objects (logical drives) */
static WORD Fsid;					/* Filesystem mount ID */

#if FF_FS_RPATH
static BYTE CurrVol;				/* Current drive number set by f_chdrive() */
#endif

#if FF_FS_LOCK
static FILESEM Files[FF_FS_LOCK];	/* Open object lock semaphores */
#if FF_FS_REENTRANT
static volatile BYTE SysLock;		/* System lock flag to protect Files[] (0:no mutex, 1:unlocked, 2:locked) */
static volatile BYTE SysLockVolume;	/* Volume id who is locking Files[] */
#endif
#endif

#if FF_STR_VOLUME_ID
#ifdef FF_VOLUME_STRS
static const char *const VolumeStr[FF_VOLUMES] = {FF_VOLUME_STRS};	/* Pre-defined volume ID */
#endif
#endif

#if FF_LBA64
#if FF_MIN_GPT > 0x100000000
#error Wrong FF_MIN_GPT setting
#endif
static const BYTE GUID_MS_Basic[16] = {0xA2, 0xA0, 0xD0, 0xEB, 0xE5, 0xB9, 0x33, 0x44, 0x87, 0xC0, 0x68, 0xB6, 0xB7, 0x26, 0x99, 0xC7};
#endif



/*--------------------------------*/
/* LFN/Directory working buffer   */
/*--------------------------------*/

#if FF_USE_LFN == 0		/* Non-LFN configuration */
#if FF_FS_EXFAT
#error LFN must be enabled when enable exFAT
#endif
#define DEF_NAMEBUFF
#define INIT_NAMEBUFF(fs)
#define FREE_NAMEBUFF()
#define LEAVE_MKFS(res)	return res

#else					/* LFN configurations */
#if FF_MAX_LFN < 12 || FF_MAX_LFN > 255
#error Wrong setting of FF_MAX_LFN
#endif
#if FF_LFN_BUF < FF_SFN_BUF || FF_SFN_BUF < 12
#error Wrong setting of FF_LFN_BUF or FF_SFN_BUF
#endif
#if FF_LFN_UNICODE < 0 || FF_LFN_UNICODE > 3
#error Wrong setting of FF_LFN_UNICODE
#endif
static const BYTE LfnOfs[] = {1, 3, 5, 7, 9, 14, 16, 18, 20, 22, 24, 28, 30};	/* FAT: Offset of LFN characters in the directory entry */
#define MAXDIRB(nc)	((nc + 44U) / 15 * SZDIRE)	/* exFAT: Size of directory entry block scratchpad buffer needed for the name length */

#if FF_USE_LFN == 1		/* LFN enabled with static working buffer */
#if FF_FS_EXFAT
static BYTE	DirBuf[MAXDIRB(FF_MAX_LFN)];	/* Directory entry block scratchpad buffer */
#endif
static WCHAR LfnBuf[FF_MAX_LFN + 1];		/* LFN working buffer */
#define DEF_NAMEBUFF
#define INIT_NAMEBUFF(fs)
#define FREE_NAMEBUFF()
#define LEAVE_MKFS(res)	return res

#elif FF_USE_LFN == 2 	/* LFN enabled with dynamic working buffer on the stack */
#if FF_FS_EXFAT
#define DEF_NAMEBUFF		WCHAR lbuf[FF_MAX_LFN+1]; BYTE dbuf[MAXDIRB(FF_MAX_LFN)];	/* LFN working buffer and directory entry block scratchpad buffer */
#define INIT_NAMEBUFF(fs)	{ (fs)->lfnbuf = lbuf; (fs)->dirbuf = dbuf; }
#define FREE_NAMEBUFF()
#else
#define DEF_NAMEBUFF		WCHAR lbuf[FF_MAX_LFN+1];	/* LFN working buffer */
#define INIT_NAMEBUFF(fs)	{ (fs)->lfnbuf = lbuf; }
#define FREE_NAMEBUFF()
#endif
#define LEAVE_MKFS(res)	return res

#elif FF_USE_LFN == 3 	/* LFN enabled with dynamic working buffer on the heap */
#if FF_FS_EXFAT
#define DEF_NAMEBUFF		WCHAR *lfn;	/* Pointer to LFN working buffer and directory entry block scratchpad buffer */
#define INIT_NAMEBUFF(fs)	{ lfn = ff_memalloc((FF_MAX_LFN+1)*2 + MAXDIRB(FF_MAX_LFN)); if (!lfn) LEAVE_FF(fs, FR_NOT_ENOUGH_CORE); (fs)->lfnbuf = lfn; (fs)->dirbuf = (BYTE*)(lfn+FF_MAX_LFN+1); }
#define FREE_NAMEBUFF()	ff_memfree(lfn)
#else
#define DEF_NAMEBUFF		WCHAR *lfn;	/* Pointer to LFN working buffer */
#define INIT_NAMEBUFF(fs)	{ lfn = ff_memalloc((FF_MAX_LFN+1)*2); if (!lfn) LEAVE_FF(fs, FR_NOT_ENOUGH_CORE); (fs)->lfnbuf = lfn; }
#define FREE_NAMEBUFF()	ff_memfree(lfn)
#endif
#define LEAVE_MKFS(res)	{ if (!work) ff_memfree(buf); return res; }
#define MAX_MALLOC	0x8000	/* Must be >=FF_MAX_SS */

#else
#error Wrong setting of FF_USE_LFN

#endif	/* FF_USE_LFN == 1 */
#endif	/* FF_USE_LFN == 0 */



/*--------------------------------*/
/* Code conversion tables         */
/*--------------------------------*/

#if FF_CODE_PAGE == 0	/* Run-time code page configuration */
#define CODEPAGE CodePage
static WORD CodePage;	/* Current code page */
static const BYTE *ExCvt;	/* Pointer to SBCS up-case table Ct???[] (null:disabled) */
static const BYTE *DbcTbl;	/* Pointer to DBCS code range table Dc???[] (null:disabled) */

static const BYTE Ct437[] = TBL_CT437;
static const BYTE Ct720[] = TBL_CT720;
static const BYTE Ct737[] = TBL_CT737;
static const BYTE Ct771[] = TBL_CT771;
static const BYTE Ct775[] = TBL_CT775;
static const BYTE Ct850[] = TBL_CT850;
static const BYTE Ct852[] = TBL_CT852;
static const BYTE Ct855[] = TBL_CT855;
static const BYTE Ct857[] = TBL_CT857;
static const BYTE Ct860[] = TBL_CT860;
static const BYTE Ct861[] = TBL_CT861;
static const BYTE Ct862[] = TBL_CT862;
static const BYTE Ct863[] = TBL_CT863;
static const BYTE Ct864[] = TBL_CT864;
static const BYTE Ct865[] = TBL_CT865;
static const BYTE Ct866[] = TBL_CT866;
static const BYTE Ct869[] = TBL_CT869;
static const BYTE Dc932[] = TBL_DC932;
static const BYTE Dc936[] = TBL_DC936;
static const BYTE Dc949[] = TBL_DC949;
static const BYTE Dc950[] = TBL_DC950;

#elif FF_CODE_PAGE < 900	/* Static code page configuration (SBCS) */
#define CODEPAGE FF_CODE_PAGE
static const BYTE ExCvt[] = MKCVTBL(TBL_CT, FF_CODE_PAGE);

#else					/* Static code page configuration (DBCS) */
#define CODEPAGE FF_CODE_PAGE
static const BYTE DbcTbl[] = MKCVTBL(TBL_DC, FF_CODE_PAGE);

#endif

#if FF_MULTI_PARTITION
PARTITION VolToPart[] = {
	/* SD0/SD1 */
	{0, 0},     /* Logical drive 0 ==> Physical drive 0, 0th partition */
	{0, 1},     /* Logical drive 1 ==> Physical drive 0, 1st partition */
	{0, 2},     /* Logical drive 2 ==> Physical drive 0, 2nd partition */
	{0, 3},     /* Logical drive 3 ==> Physical drive 0, 3rd partition */
	{0, 4},     /* Logical drive 4 ==> Physical drive 0, 4th partition */
	/* SD1 */
	{1, 0},     /* Logical drive 5 ==> Physical drive 1, 0th partition */
	{1, 1},     /* Logical drive 6 ==> Physical drive 1, 1st partition */
	{1, 2},     /* Logical drive 7 ==> Physical drive 1, 2nd partition */
	{1, 3},     /* Logical drive 8 ==> Physical drive 1, 3rd partition */
	{1, 4},      /* Logical drive 9 ==> Physical drive 1, 4th partition */
#ifdef XPAR_XUFSPSXC_NUM_INSTANCES
	/* UFS Boot-Lun */
	{2, 0},      /* Logical drive 10 ==> Physical drive 2, 0th partition */
	{2, 1},      /* Logical drive 11 ==> Physical drive 2, 1st partition */
	{2, 2},      /* Logical drive 12 ==> Physical drive 2, 2nd partition */
	{2, 3},      /* Logical drive 13 ==> Physical drive 2, 3rd partition */
	{2, 4},      /* Logical drive 14 ==> Physical drive 2, 4th partition */
	/* UFS */
	{3, 0},      /* Logical drive 15 ==> Physical drive 3, 0th partition */
	{3, 1},      /* Logical drive 16 ==> Physical drive 3, 1st partition */
	{3, 2},      /* Logical drive 17 ==> Physical drive 3, 2nd partition */
	{3, 3},      /* Logical drive 18 ==> Physical drive 3, 3rd partition */
	{3, 4},      /* Logical drive 19 ==> Physical drive 3, 4th partition */
	{4, 0},      /* Logical drive 20 ==> Physical drive 4, 0th partition */
	{4, 1},      /* Logical drive 21 ==> Physical drive 4, 1st partition */
	{4, 2},      /* Logical drive 22 ==> Physical drive 4, 2nd partition */
	{4, 3},      /* Logical drive 23 ==> Physical drive 4, 3rd partition */
	{4, 4},      /* Logical drive 24 ==> Physical drive 4, 4th partition */
	{5, 0},      /* Logical drive 25 ==> Physical drive 5, 0th partition */
	{5, 1},      /* Logical drive 26 ==> Physical drive 5, 1st partition */
	{5, 2},      /* Logical drive 27 ==> Physical drive 5, 2nd partition */
	{5, 3},      /* Logical drive 28 ==> Physical drive 5, 3rd partition */
	{5, 4},      /* Logical drive 29 ==> Physical drive 5, 4th partition */
	{6, 0},      /* Logical drive 30 ==> Physical drive 6, 0th partition */
	{6, 1},      /* Logical drive 31 ==> Physical drive 6, 1st partition */
	{6, 2},      /* Logical drive 32 ==> Physical drive 6, 2nd partition */
	{6, 3},      /* Logical drive 33 ==> Physical drive 6, 3rd partition */
	{6, 4},      /* Logical drive 34 ==> Physical drive 6, 4th partition */
	{7, 0},      /* Logical drive 35 ==> Physical drive 7, 0th partition */
	{7, 1},      /* Logical drive 36 ==> Physical drive 7, 1st partition */
	{7, 2},      /* Logical drive 37 ==> Physical drive 7, 2nd partition */
	{7, 3},      /* Logical drive 38 ==> Physical drive 7, 3rd partition */
	{7, 4},      /* Logical drive 39 ==> Physical drive 7, 4th partition */
	{8, 0},      /* Logical drive 40 ==> Physical drive 8, 0th partition */
	{8, 1},      /* Logical drive 41 ==> Physical drive 8, 1st partition */
	{8, 2},      /* Logical drive 42 ==> Physical drive 8, 2nd partition */
	{8, 3},      /* Logical drive 43 ==> Physical drive 8, 3rd partition */
	{8, 4},      /* Logical drive 44 ==> Physical drive 8, 4th partition */
	{9, 0},      /* Logical drive 45 ==> Physical drive 9, 0th partition */
	{9, 1},      /* Logical drive 46 ==> Physical drive 9, 1st partition */
	{9, 2},      /* Logical drive 47 ==> Physical drive 9, 2nd partition */
	{9, 3},      /* Logical drive 48 ==> Physical drive 9, 3rd partition */
	{9, 4},      /* Logical drive 49 ==> Physical drive 9, 4th partition */
	{10, 0},      /* Logical drive 50 ==> Physical drive 10, 0th partition */
	{10, 1},      /* Logical drive 51 ==> Physical drive 10, 1st partition */
	{10, 2},      /* Logical drive 52 ==> Physical drive 10, 2nd partition */
	{10, 3},      /* Logical drive 53 ==> Physical drive 10, 3rd partition */
	{10, 4},      /* Logical drive 54 ==> Physical drive 10, 4th partition */
	{11, 0},      /* Logical drive 55 ==> Physical drive 11, 0th partition */
	{11, 1},      /* Logical drive 56 ==> Physical drive 11, 1st partition */
	{11, 2},      /* Logical drive 57 ==> Physical drive 11, 2nd partition */
	{11, 3},      /* Logical drive 58 ==> Physical drive 11, 3rd partition */
	{11, 4},      /* Logical drive 59 ==> Physical drive 11, 4th partition */
	{12, 0},      /* Logical drive 60 ==> Physical drive 12, 0th partition */
	{12, 1},      /* Logical drive 61 ==> Physical drive 12, 1st partition */
	{12, 2},      /* Logical drive 62 ==> Physical drive 12, 2nd partition */
	{12, 3},      /* Logical drive 63 ==> Physical drive 12, 3rd partition */
	{12, 4},      /* Logical drive 64 ==> Physical drive 12, 4th partition */
	{13, 0},      /* Logical drive 65 ==> Physical drive 13, 0th partition */
	{13, 1},      /* Logical drive 66 ==> Physical drive 13, 1st partition */
	{13, 2},      /* Logical drive 67 ==> Physical drive 13, 2nd partition */
	{13, 3},      /* Logical drive 68 ==> Physical drive 13, 3rd partition */
	{13, 4},      /* Logical drive 69 ==> Physical drive 13, 4th partition */
	{14, 0},      /* Logical drive 70 ==> Physical drive 14, 0th partition */
	{14, 1},      /* Logical drive 71 ==> Physical drive 14, 1st partition */
	{14, 2},      /* Logical drive 72 ==> Physical drive 14, 2nd partition */
	{14, 3},     /* Logical drive 73 ==> Physical drive 14, 3rd partition */
	{14, 4},      /* Logical drive 74 ==> Physical drive 14, 4th partition */
	{15, 0},      /* Logical drive 75 ==> Physical drive 15, 0th partition */
	{15, 1},     /* Logical drive 76 ==> Physical drive 15, 1st partition */
	{15, 2},      /* Logical drive 77 ==> Physical drive 15, 2nd partition */
	{15, 3},      /* Logical drive 78 ==> Physical drive 15, 3rd partition */
	{15, 4},      /* Logical drive 79 ==> Physical drive 15, 4th partition */
	{16, 0},      /* Logical drive 80 ==> Physical drive 16, 0th partition */
	{16, 1},      /* Logical drive 81 ==> Physical drive 16, 1st partition */
	{16, 2},      /* Logical drive 82 ==> Physical drive 16, 2nd partition */
	{16, 3},      /* Logical drive 83 ==> Physical drive 16, 3rd partition */
	{16, 4},      /* Logical drive 84 ==> Physical drive 16, 4th partition */
	{17, 0},      /* Logical drive 85 ==> Physical drive 17, 0th partition */
	{17, 1},      /* Logical drive 86 ==> Physical drive 17, 1st partition */
	{17, 2},      /* Logical drive 87 ==> Physical drive 17, 2nd partition */
	{17, 3},      /* Logical drive 88 ==> Physical drive 17, 3rd partition */
	{17, 4},      /* Logical drive 89 ==> Physical drive 17, 4th partition */
	{18, 0},      /* Logical drive 90 ==> Physical drive 18, 0th partition */
	{18, 1},      /* Logical drive 91 ==> Physical drive 18, 1st partition */
	{18, 2},      /* Logical drive 92 ==> Physical drive 18, 2nd partition */
	{18, 3},      /* Logical drive 93 ==> Physical drive 18, 3rd partition */
	{18, 4},      /* Logical drive 94 ==> Physical drive 18, 4th partition */
	{19, 0},      /* Logical drive 95 ==> Physical drive 19, 0th partition */
	{19, 1},      /* Logical drive 96 ==> Physical drive 19, 1st partition */
	{19, 2},      /* Logical drive 97 ==> Physical drive 19, 2nd partition */
	{19, 3},      /* Logical drive 98 ==> Physical drive 19, 3rd partition */
	{19, 4},      /* Logical drive 99 ==> Physical drive 19, 4th partition */
	{20, 0},      /* Logical drive 100 ==> Physical drive 20, 0th partition */
	{20, 1},      /* Logical drive 101 ==> Physical drive 20, 1st partition */
	{20, 2},      /* Logical drive 102 ==> Physical drive 20, 2nd partition */
	{20, 3},      /* Logical drive 103 ==> Physical drive 20, 3rd partition */
	{20, 4},      /* Logical drive 104 ==> Physical drive 20, 4th partition */
	{21, 0},      /* Logical drive 105 ==> Physical drive 21, 0th partition */
	{21, 1},      /* Logical drive 106 ==> Physical drive 21, 1st partition */
	{21, 2},      /* Logical drive 107 ==> Physical drive 21, 2nd partition */
	{21, 3},      /* Logical drive 108 ==> Physical drive 21, 3rd partition */
	{21, 4},      /* Logical drive 109 ==> Physical drive 21, 4th partition */
	{22, 0},      /* Logical drive 110 ==> Physical drive 22, 0th partition */
	{22, 1},      /* Logical drive 111 ==> Physical drive 22, 1st partition */
	{22, 2},      /* Logical drive 112 ==> Physical drive 22, 2nd partition */
	{22, 3},      /* Logical drive 113 ==> Physical drive 22, 3rd partition */
	{22, 4},      /* Logical drive 114 ==> Physical drive 22, 4th partition */
	{23, 0},      /* Logical drive 115 ==> Physical drive 23, 0th partition */
	{23, 1},      /* Logical drive 116 ==> Physical drive 23, 1st partition */
	{23, 2},      /* Logical drive 117 ==> Physical drive 23, 2nd partition */
	{23, 3},      /* Logical drive 118 ==> Physical drive 23, 3rd partition */
	{23, 4},      /* Logical drive 119 ==> Physical drive 23, 4th partition */
	{24, 0},      /* Logical drive 120 ==> Physical drive 24, 0th partition */
	{24, 1},      /* Logical drive 121 ==> Physical drive 24, 1st partition */
	{24, 2},      /* Logical drive 122 ==> Physical drive 24, 2nd partition */
	{24, 3},      /* Logical drive 123 ==> Physical drive 24, 3rd partition */
	{24, 4},      /* Logical drive 124 ==> Physical drive 24, 4th partition */
	{25, 0},      /* Logical drive 125 ==> Physical drive 25, 0th partition */
	{25, 1},      /* Logical drive 126 ==> Physical drive 25, 1st partition */
	{25, 2},      /* Logical drive 127 ==> Physical drive 25, 2nd partition */
	{25, 3},      /* Logical drive 128 ==> Physical drive 25, 3rd partition */
	{25, 4},      /* Logical drive 129 ==> Physical drive 25, 4th partition */
	{26, 0},      /* Logical drive 130 ==> Physical drive 26, 0th partition */
	{26, 1},      /* Logical drive 131 ==> Physical drive 26, 1st partition */
	{26, 2},      /* Logical drive 132 ==> Physical drive 26, 2nd partition */
	{26, 3},      /* Logical drive 133 ==> Physical drive 26, 3rd partition */
	{26, 4},      /* Logical drive 134 ==> Physical drive 26, 4th partition */
	{27, 0},      /* Logical drive 135 ==> Physical drive 27, 0th partition */
	{27, 1},      /* Logical drive 136 ==> Physical drive 27, 1st partition */
	{27, 2},      /* Logical drive 137 ==> Physical drive 27, 2nd partition */
	{27, 3},      /* Logical drive 138 ==> Physical drive 27, 3rd partition */
	{27, 4},      /* Logical drive 139 ==> Physical drive 27, 4th partition */
	{28, 0},      /* Logical drive 140 ==> Physical drive 28, 0th partition */
	{28, 1},      /* Logical drive 141 ==> Physical drive 28, 1st partition */
	{28, 2},      /* Logical drive 142 ==> Physical drive 28, 2nd partition */
	{28, 3},      /* Logical drive 143 ==> Physical drive 28, 3rd partition */
	{28, 4},      /* Logical drive 144 ==> Physical drive 28, 4th partition */
	{29, 0},      /* Logical drive 145 ==> Physical drive 29, 0th partition */
	{29, 1},      /* Logical drive 146 ==> Physical drive 29, 1st partition */
	{29, 2},      /* Logical drive 147 ==> Physical drive 29, 2nd partition */
	{29, 3},      /* Logical drive 148 ==> Physical drive 29, 3rd partition */
	{29, 4},      /* Logical drive 149 ==> Physical drive 29, 4th partition */
	{30, 0},      /* Logical drive 150 ==> Physical drive 30, 0th partition */
	{30, 1},      /* Logical drive 151 ==> Physical drive 30, 1st partition */
	{30, 2},      /* Logical drive 152 ==> Physical drive 30, 2nd partition */
	{30, 3},      /* Logical drive 153 ==> Physical drive 30, 3rd partition */
	{30, 4},      /* Logical drive 154 ==> Physical drive 30, 4th partition */
	{31, 0},      /* Logical drive 155 ==> Physical drive 31, 0th partition */
	{31, 1},      /* Logical drive 156 ==> Physical drive 31, 1st partition */
	{31, 2},      /* Logical drive 157 ==> Physical drive 31, 2nd partition */
	{31, 3},      /* Logical drive 158 ==> Physical drive 31, 3rd partition */
	{31, 4},      /* Logical drive 159 ==> Physical drive 31, 4th partition */
	{32, 0},      /* Logical drive 160 ==> Physical drive 32, 0th partition */
	{32, 1},      /* Logical drive 161 ==> Physical drive 32, 1st partition */
	{32, 2},      /* Logical drive 162 ==> Physical drive 32, 2nd partition */
	{32, 3},      /* Logical drive 163 ==> Physical drive 32, 3rd partition */
	{32, 4},      /* Logical drive 164 ==> Physical drive 32, 4th partition */
	{33, 0},      /* Logical drive 165 ==> Physical drive 33, 0th partition */
	{33, 1},      /* Logical drive 166 ==> Physical drive 33, 1st partition */
	{33, 2},      /* Logical drive 167 ==> Physical drive 33, 2nd partition */
	{33, 3},      /* Logical drive 168 ==> Physical drive 33, 3rd partition */
	{33, 4},      /* Logical drive 169 ==> Physical drive 33, 4th partition */
	{34, 0},      /* Logical drive 170 ==> Physical drive 34, 0th partition */
	{34, 1},      /* Logical drive 171 ==> Physical drive 34, 1st partition */
	{34, 2},      /* Logical drive 172 ==> Physical drive 34, 2nd partition */
	{34, 3},      /* Logical drive 173 ==> Physical drive 34, 3rd partition */
	{34, 4},      /* Logical drive 174 ==> Physical drive 34, 4th partition */
#endif
};
#endif



/*--------------------------------------------------------------------------

   Module Private Functions

---------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------*/
/* Load/Store multi-byte word in the FAT structure                       */
/*-----------------------------------------------------------------------*/

static WORD ld_16 (const BYTE *ptr)	/*	 Load a 2-byte little-endian word */
{
	WORD rv;

	rv = ptr[1];
	rv = rv << 8 | ptr[0];
	return rv;
}

static DWORD ld_32 (const BYTE *ptr)	/* Load a 4-byte little-endian word */
{
	DWORD rv;

	rv = ptr[3];
	rv = rv << 8 | ptr[2];
	rv = rv << 8 | ptr[1];
	rv = rv << 8 | ptr[0];
	return rv;
}

#if FF_FS_EXFAT
static QWORD ld_64 (const BYTE *ptr)	/* Load an 8-byte little-endian word */
{
	QWORD rv;

	rv = ptr[7];
	rv = rv << 8 | ptr[6];
	rv = rv << 8 | ptr[5];
	rv = rv << 8 | ptr[4];
	rv = rv << 8 | ptr[3];
	rv = rv << 8 | ptr[2];
	rv = rv << 8 | ptr[1];
	rv = rv << 8 | ptr[0];
	return rv;
}
#endif

#if !FF_FS_READONLY
static void st_16 (BYTE *ptr, WORD val)	/* Store a 2-byte word in little-endian */
{
	*ptr++ = (BYTE)val;
	val >>= 8;
	*ptr++ = (BYTE)val;
}

static void st_32 (BYTE *ptr, DWORD val)	/* Store a 4-byte word in little-endian */
{
	*ptr++ = (BYTE)val;
	val >>= 8;
	*ptr++ = (BYTE)val;
	val >>= 8;
	*ptr++ = (BYTE)val;
	val >>= 8;
	*ptr++ = (BYTE)val;
}

#if FF_FS_EXFAT
static void st_64 (BYTE *ptr, QWORD val)	/* Store an 8-byte word in little-endian */
{
	*ptr++ = (BYTE)val;
	val >>= 8;
	*ptr++ = (BYTE)val;
	val >>= 8;
	*ptr++ = (BYTE)val;
	val >>= 8;
	*ptr++ = (BYTE)val;
	val >>= 8;
	*ptr++ = (BYTE)val;
	val >>= 8;
	*ptr++ = (BYTE)val;
	val >>= 8;
	*ptr++ = (BYTE)val;
	val >>= 8;
	*ptr++ = (BYTE)val;
}
#endif
#endif	/* !FF_FS_READONLY */



/*-----------------------------------------------------------------------*/
/* String functions                                                      */
/*-----------------------------------------------------------------------*/

/* Copy memory to memory */
static void mem_cpy (void *dst, const void *src, UINT cnt)
{
	BYTE *d = (BYTE *)dst;
	const BYTE *s = (const BYTE *)src;
#if FF_WORD_ACCESS == 1
	while (cnt >= sizeof (int)) {
		*(int *)d = *(int *)s;
		d += sizeof (int);
		s += sizeof (int);
		cnt -= sizeof (int);
	}
#endif
	if (cnt != 0) {
		do {
			*d++ = *s++;
		}
		while (--cnt);
	}
}
/* Test if the byte is DBC 1st byte */
static int dbc_1st (BYTE c)
{
#if FF_CODE_PAGE == 0		/* Variable code page */
	if (DbcTbl && c >= DbcTbl[0]) {
		if (c <= DbcTbl[1]) {
			return 1;        /* 1st byte range 1 */
		}
		if (c >= DbcTbl[2] && c <= DbcTbl[3]) {
			return 1;        /* 1st byte range 2 */
		}
	}
#elif FF_CODE_PAGE >= 900	/* DBCS fixed code page */
	if (c >= DbcTbl[0]) {
		if (c <= DbcTbl[1]) {
			return 1;
		}
		if (c >= DbcTbl[2] && c <= DbcTbl[3]) {
			return 1;
		}
	}
#else						/* SBCS fixed code page */
	if (c != 0) {
		return 0;        /* Always false */
	}
#endif
	return 0;
}


/* Test if the byte is DBC 2nd byte */
static int dbc_2nd (BYTE c)
{
#if FF_CODE_PAGE == 0		/* Variable code page */
	if (DbcTbl && c >= DbcTbl[4]) {
		if (c <= DbcTbl[5]) {
			return 1;        /* 2nd byte range 1 */
		}
		if (c >= DbcTbl[6] && c <= DbcTbl[7]) {
			return 1;        /* 2nd byte range 2 */
		}
		if (c >= DbcTbl[8] && c <= DbcTbl[9]) {
			return 1;        /* 2nd byte range 3 */
		}
	}
#elif FF_CODE_PAGE >= 900	/* DBCS fixed code page */
	if (c >= DbcTbl[4]) {
		if (c <= DbcTbl[5]) {
			return 1;
		}
		if (c >= DbcTbl[6] && c <= DbcTbl[7]) {
			return 1;
		}
		if (c >= DbcTbl[8] && c <= DbcTbl[9]) {
			return 1;
		}
	}
#else						/* SBCS fixed code page */
	if (c != 0) {
		return 0;        /* Always false */
	}
#endif
	return 0;
}


#if FF_USE_LFN

/* Get a Unicode code point from the TCHAR string in defined API encodeing */
static DWORD
tchar2uni (	/* Returns a character in UTF-16 encoding (>=0x10000 on surrogate pair, 0xFFFFFFFF on decode error) */
	const TCHAR **str		/* Pointer to pointer to TCHAR string in configured encoding */
)
{
	DWORD uc;
	const TCHAR *p = *str;

#if FF_LFN_UNICODE == 1		/* UTF-16 input */
	WCHAR wc;

	uc = *p++;	/* Get an encoding unit */
	if (IsSurrogate(uc)) {	/* Surrogate? */
		wc = *p++;		/* Get low surrogate */
		if (!IsSurrogateH(uc) || !IsSurrogateL(wc)) {
			return 0xFFFFFFFF;        /* Wrong surrogate? */
		}
		uc = uc << 16 | wc;
	}

#elif FF_LFN_UNICODE == 2	/* UTF-8 input */
	BYTE tb;
	int nf;

	uc = (BYTE) * p++;	/* Get an encoding unit */
	if (uc & 0x80) {	/* Multiple byte code? */
		if        ((uc & 0xE0) == 0xC0) {	/* 2-byte sequence? */
			uc &= 0x1F;
			nf = 1;
		}
		else if ((uc & 0xF0) == 0xE0) {	/* 3-byte sequence? */
			uc &= 0x0F;
			nf = 2;
		}
		else if ((uc & 0xF8) == 0xF0) {	/* 4-byte sequence? */
			uc &= 0x07;
			nf = 3;
		}
		else {							/* Wrong sequence */
			return 0xFFFFFFFF;
		}
		do {	/* Get and merge trailing bytes */
			tb = (BYTE) * p++;
			if ((tb & 0xC0) != 0x80) {
				return 0xFFFFFFFF;        /* Wrong sequence? */
			}
			uc = uc << 6 | (tb & 0x3F);
		}
		while (--nf != 0);
		if (uc < 0x80 || IsSurrogate(uc) || uc >= 0x110000) {
			return 0xFFFFFFFF;        /* Wrong code? */
		}
		if (uc >= 0x010000) {
			uc = 0xD800DC00 | ((uc - 0x10000) << 6 & 0x3FF0000) | (uc & 0x3FF);        /* Make a surrogate pair if needed */
		}
	}

#elif FF_LFN_UNICODE == 3	/* UTF-32 input */
	uc = (TCHAR) * p++;	/* Get a unit */
	if (uc >= 0x110000 || IsSurrogate(uc)) {
		return 0xFFFFFFFF;        /* Wrong code? */
	}
	if (uc >= 0x010000) {
		uc = 0xD800DC00 | ((uc - 0x10000) << 6 & 0x3FF0000) | (uc & 0x3FF);        /* Make a surrogate pair if needed */
	}

#else		/* ANSI/OEM input */
	BYTE sb;
	WCHAR wc;

	wc = (BYTE) * p++;			/* Get a byte */
	if (dbc_1st((BYTE)wc)) {	/* Is it a DBC 1st byte? */
		sb = (BYTE) * p++;		/* Get 2nd byte */
		if (!dbc_2nd(sb)) {
			return 0xFFFFFFFF;        /* Invalid code? */
		}
		wc = (wc << 8) + sb;		/* Make a DBC */
	}
	if (wc != 0) {
		wc = ff_oem2uni(wc, CODEPAGE);	/* ANSI/OEM ==> Unicode */
		if (wc == 0) {
			return 0xFFFFFFFF;        /* Invalid code? */
		}
	}
	uc = wc;

#endif
	*str = p;	/* Next read pointer */
	return uc;
}


/* Store a Unicode char in defined API encoding */
static UINT put_utf (	/* Returns number of encoding units written (0:buffer overflow or wrong encoding) */
	DWORD chr,	/* UTF-16 encoded character (Surrogate pair if >=0x10000) */
	TCHAR *buf,	/* Output buffer */
	UINT szb	/* Size of the buffer */
)
{
#if FF_LFN_UNICODE == 1	/* UTF-16 output */
	WCHAR hs, wc;

	hs = (WCHAR)(chr >> 16);
	wc = (WCHAR)chr;
	if (hs == 0) {	/* Single encoding unit? */
		if (szb < 1 || IsSurrogate(wc)) {
			return 0;        /* Buffer overflow or wrong code? */
		}
		*buf = wc;
		return 1;
	}
	if (szb < 2 || !IsSurrogateH(hs) || !IsSurrogateL(wc)) {
		return 0;        /* Buffer overflow or wrong surrogate? */
	}
	*buf++ = hs;
	*buf++ = wc;
	return 2;

#elif FF_LFN_UNICODE == 2	/* UTF-8 output */
	DWORD hc;

	if (chr < 0x80) {	/* Single byte code? */
		if (szb < 1) {
			return 0;        /* Buffer overflow? */
		}
		*buf = (TCHAR)chr;
		return 1;
	}
	if (chr < 0x800) {	/* 2-byte sequence? */
		if (szb < 2) {
			return 0;        /* Buffer overflow? */
		}
		*buf++ = (TCHAR)(0xC0 | (chr >> 6 & 0x1F));
		*buf++ = (TCHAR)(0x80 | (chr >> 0 & 0x3F));
		return 2;
	}
	if (chr < 0x10000) {	/* 3-byte sequence? */
		if (szb < 3 || IsSurrogate(chr)) {
			return 0;        /* Buffer overflow or wrong code? */
		}
		*buf++ = (TCHAR)(0xE0 | (chr >> 12 & 0x0F));
		*buf++ = (TCHAR)(0x80 | (chr >> 6 & 0x3F));
		*buf++ = (TCHAR)(0x80 | (chr >> 0 & 0x3F));
		return 3;
	}
	/* 4-byte sequence */
	if (szb < 4) {
		return 0;        /* Buffer overflow? */
	}
	hc = ((chr & 0xFFFF0000) - 0xD8000000) >> 6;	/* Get high 10 bits */
	chr = (chr & 0xFFFF) - 0xDC00;					/* Get low 10 bits */
	if (hc >= 0x100000 || chr >= 0x400) {
		return 0;        /* Wrong surrogate? */
	}
	chr = (hc | chr) + 0x10000;
	*buf++ = (TCHAR)(0xF0 | (chr >> 18 & 0x07));
	*buf++ = (TCHAR)(0x80 | (chr >> 12 & 0x3F));
	*buf++ = (TCHAR)(0x80 | (chr >> 6 & 0x3F));
	*buf++ = (TCHAR)(0x80 | (chr >> 0 & 0x3F));
	return 4;

#elif FF_LFN_UNICODE == 3	/* UTF-32 output */
	DWORD hc;

	if (szb < 1) {
		return 0;        /* Buffer overflow? */
	}
	if (chr >= 0x10000) {	/* Out of BMP? */
		hc = ((chr & 0xFFFF0000) - 0xD8000000) >> 6;	/* Get high 10 bits */
		chr = (chr & 0xFFFF) - 0xDC00;					/* Get low 10 bits */
		if (hc >= 0x100000 || chr >= 0x400) {
			return 0;        /* Wrong surrogate? */
		}
		chr = (hc | chr) + 0x10000;
	}
	*buf++ = (TCHAR)chr;
	return 1;

#else						/* ANSI/OEM output */
	WCHAR wc;

	wc = ff_uni2oem(chr, CODEPAGE);
	if (wc >= 0x100) {	/* Is this a DBC? */
		if (szb < 2) {
			return 0;
		}
		*buf++ = (char)(wc >> 8);	/* Store DBC 1st byte */
		*buf++ = (TCHAR)wc;			/* Store DBC 2nd byte */
		return 2;
	}
	if (wc == 0 || szb < 1) {
		return 0;        /* Invalid character or buffer overflow? */
	}
	*buf++ = (TCHAR)wc;					/* Store the character */
	return 1;
#endif
}
#endif	/* FF_USE_LFN */


#if FF_FS_REENTRANT
/*-----------------------------------------------------------------------*/
/* Request/Release grant to access the volume                            */
/*-----------------------------------------------------------------------*/

static int lock_volume (	/* 1:Ok, 0:timeout */
	FATFS *fs,				/* Filesystem object to lock */
	int syslock				/* System lock required */
)
{
	int rv;


#if FF_FS_LOCK
	rv = ff_mutex_take(fs->ldrv);	/* Lock the volume */
	if (rv && syslock) {			/* System lock reqiered? */
		rv = ff_mutex_take(FF_VOLUMES);	/* Lock the system */
		if (rv) {
			SysLockVolume = fs->ldrv;
			SysLock = 2;				/* System lock succeeded */
		}
		else {
			ff_mutex_give(fs->ldrv);	/* Failed system lock */
		}
	}
#else
	rv = syslock ? ff_mutex_take(fs->ldrv) : ff_mutex_take(
		     fs->ldrv);	/* Lock the volume (this is to prevent compiler warning) */
#endif
	return rv;
}


static void unlock_volume (
	FATFS *fs,		/* Filesystem object */
	FRESULT res		/* Result code to be returned */
)
{
	if (fs && res != FR_NOT_ENABLED && res != FR_INVALID_DRIVE && res != FR_TIMEOUT) {
#if FF_FS_LOCK
		if (SysLock == 2 && SysLockVolume == fs->ldrv) {	/* Unlock system if it has been locked by this task */
			SysLock = 1;
			ff_mutex_give(FF_VOLUMES);
		}
#endif
		ff_mutex_give(fs->ldrv);	/* Unlock the volume */
	}
}

#endif



#if FF_FS_LOCK
/*-----------------------------------------------------------------------*/
/* File sharing control functions                                       */
/*-----------------------------------------------------------------------*/

static FRESULT chk_share (	/* Check if the file can be accessed */
	DIR *dp,		/* Directory object pointing the file to be checked */
	int acc			/* Desired access type (0:Read mode open, 1:Write mode open, 2:Delete or rename) */
)
{
	UINT i, be;

	/* Search open object table for the object */
	be = 0;
	for (i = 0; i < FF_FS_LOCK; i++) {
		if (Files[i].fs) {	/* Existing entry */
			if (Files[i].fs == dp->obj.fs &&	 	/* Check if the object matches with an open object */
			    Files[i].clu == dp->obj.sclust &&
			    Files[i].ofs == dp->dptr) {
				break;
			}
		}
		else {			/* Blank entry */
			be = 1;
		}
	}
	if (i == FF_FS_LOCK) {	/* The object has not been opened */
		return (!be && acc != 2) ? FR_TOO_MANY_OPEN_FILES : FR_OK;	/* Is there a blank entry for new object? */
	}

	/* The object was opened. Reject any open against writing file and all write mode open */
	return (acc != 0 || Files[i].ctr == 0x100) ? FR_LOCKED : FR_OK;
}


static int enq_share (void)	/* Check if an entry is available for a new object */
{
	UINT i;

	for (i = 0; i < FF_FS_LOCK && Files[i].fs; i++) ;	/* Find a free entry */
	return (i == FF_FS_LOCK) ? 0 : 1;
}


static UINT inc_share (	/* Increment object open counter and returns its index (0:Internal error) */
	DIR *dp,	/* Directory object pointing the file to register or increment */
	int acc		/* Desired access (0:Read, 1:Write, 2:Delete/Rename) */
)
{
	UINT i;


	for (i = 0; i < FF_FS_LOCK; i++) {	/* Find the object */
		if (Files[i].fs == dp->obj.fs
		    && Files[i].clu == dp->obj.sclust
		    && Files[i].ofs == dp->dptr) {
			break;
		}
	}

	if (i == FF_FS_LOCK) {			/* Not opened. Register it as new. */
		for (i = 0; i < FF_FS_LOCK && Files[i].fs; i++) ;	/* Find a free entry */
		if (i == FF_FS_LOCK) {
			return 0;        /* No free entry to register (int err) */
		}
		Files[i].fs = dp->obj.fs;
		Files[i].clu = dp->obj.sclust;
		Files[i].ofs = dp->dptr;
		Files[i].ctr = 0;
	}

	if (acc >= 1 && Files[i].ctr) {
		return 0;        /* Access violation (int err) */
	}

	Files[i].ctr = acc ? 0x100 : Files[i].ctr + 1;	/* Set semaphore value */

	return i + 1;	/* Index number origin from 1 */
}


static FRESULT dec_share (	/* Decrement object open counter */
	UINT i			/* Semaphore index (1..) */
)
{
	UINT n;
	FRESULT res = FR_DISK_ERR;


	if (--i < FF_FS_LOCK) {	/* Index number origin from 0 */
		n = Files[i].ctr;
		if (n == 0x100) {
			n = 0;        /* If write mode open, delete the object semaphore */
		}
		if (n > 0) {
			n--;        /* Decrement read mode open count */
		}
		Files[i].ctr = n;
		if (n == 0) {			/* Delete the object semaphore if open count becomes zero */
			Files[i].fs =
				0;	/* Free the entry <<<If this memory write operation is not in atomic, FF_FS_REENTRANT == 1 and FF_VOLUMES > 1, there is a potential error in this process >>> */
		}
		res = FR_OK;
	}
	else {
		res = FR_INT_ERR;		/* Invalid index number */
	}
	return res;
}


static void clear_share (	/* Clear all lock entries of the volume */
	FATFS *fs
)
{
	UINT i;

	for (i = 0; i < FF_FS_LOCK; i++) {
		if (Files[i].fs == fs) {
			Files[i].fs = 0;
		}
	}
}

#endif	/* FF_FS_LOCK */



/*-----------------------------------------------------------------------*/
/* Move/Flush disk access window in the filesystem object                */
/*-----------------------------------------------------------------------*/
#if !FF_FS_READONLY
static FRESULT sync_window (	/* Returns FR_OK or FR_DISK_ERR */
	FATFS *fs			/* Filesystem object */
)
{
	FRESULT res = FR_DISK_ERR;


	if (fs->wflag) {	/* Is the disk access window dirty? */
		if (disk_write(fs->pdrv, fs->win, fs->winsect, 1) == RES_OK) {	/* Write it back into the volume */
			fs->wflag = 0;	/* Clear window dirty flag */
			if (fs->winsect - fs->fatbase < fs->fsize) {	/* Is it in the 1st FAT? */
				if (fs->n_fats == 2) {
					disk_write(fs->pdrv, fs->win, fs->winsect + fs->fsize, 1);        /* Reflect it to 2nd FAT if needed */
				}
			}
			res = FR_OK;
		}
	}
	else {
		res = FR_OK;
	}

	return res;
}
#endif


static FRESULT move_window (	/* Returns FR_OK or FR_DISK_ERR */
	FATFS *fs,		/* Filesystem object */
	LBA_t sect		/* Sector LBA to make appearance in the fs->win[] */
)
{
	FRESULT res = FR_DISK_ERR;

	if (sect != fs->winsect) {	/* Window offset changed? */
#if !FF_FS_READONLY
		res = sync_window(fs);		/* Write-back changes */
		if (res == FR_OK) {			/* Fill sector window with new data */
#endif
			if (disk_read(fs->pdrv, fs->win, sect, 1) != RES_OK) {
				sect = (LBA_t)0 - 1;	/* Invalidate window if read data is not valid */
				res = FR_DISK_ERR;
			}
			else {
				res = FR_OK;
			}
			fs->winsect = sect;
#if !FF_FS_READONLY
		}
#endif
	}
	else {
		res = FR_OK;
	}
	return res;
}




#if !FF_FS_READONLY
/*-----------------------------------------------------------------------*/
/* Synchronize filesystem and data on the storage                        */
/*-----------------------------------------------------------------------*/

static FRESULT sync_fs (	/* Returns FR_OK or FR_DISK_ERR */
	FATFS *fs		/* Filesystem object */
)
{
	FRESULT res = FR_DISK_ERR;


	res = sync_window(fs);
	if (res == FR_OK) {
		if (fs->fsi_flag == 1) {	/* Allocation changed? */
			fs->fsi_flag = 0;
			if (fs->fs_type == FS_FAT32) {	/* FAT32: Update FSInfo sector */
				/* Create FSInfo structure */
				memset(fs->win, 0, sizeof fs->win);
				st_32(fs->win + FSI_LeadSig, 0x41615252);		/* Leading signature */
				st_32(fs->win + FSI_StrucSig, 0x61417272);		/* Structure signature */
				st_32(fs->win + FSI_Free_Count, fs->free_clst);	/* Number of free clusters */
				st_32(fs->win + FSI_Nxt_Free, fs->last_clst);	/* Last allocated culuster */
				st_32(fs->win + FSI_TrailSig, 0xAA550000);		/* Trailing signature */
				disk_write(fs->pdrv, fs->win, fs->winsect = fs->volbase + 1, 1);	/* Write it into the FSInfo sector (Next to VBR) */
			}
#if FF_FS_EXFAT
			else if (fs->fs_type == FS_EXFAT) {	/* exFAT: Update PercInUse field in BPB */
				if (disk_read(fs->pdrv, fs->win, fs->winsect = fs->volbase, 1) == RES_OK) {	/* Load VBR */
					BYTE perc_inuse = (fs->free_clst <= fs->n_fatent - 2) ? (BYTE)((QWORD)(fs->n_fatent - 2 - fs->free_clst) * 100 / (fs->n_fatent - 2)) : 0xFF;	/* Percent in use 0-100 or 0xFF(unknown) */

					if (fs->win[BPB_PercInUseEx] != perc_inuse) {	/* Write it back into VBR if needed */
						fs->win[BPB_PercInUseEx] = perc_inuse;
						disk_write(fs->pdrv, fs->win, fs->winsect, 1);
					}
				}
			}
#endif
		}
		/* Make sure that no pending write process in the lower layer */
		if (disk_ioctl(fs->pdrv, CTRL_SYNC, 0) != RES_OK) {
			res = FR_DISK_ERR;
		}
	}

	return res;
}

#endif



/*-----------------------------------------------------------------------*/
/* Get physical sector number from cluster number                        */
/*-----------------------------------------------------------------------*/

static LBA_t clst2sect (	/* !=0:Sector number, 0:Failed (invalid cluster#) */
	FATFS *fs,		/* Filesystem object */
	DWORD clst		/* Cluster# to be converted */
)
{
	clst -= 2;		/* Cluster number is origin from 2 */
	if (clst >= fs->n_fatent - 2) {
		return 0;        /* Is it invalid cluster number? */
	}
	return fs->database + (LBA_t)fs->csize * clst;	/* Start sector number of the cluster */
}




/*-----------------------------------------------------------------------*/
/* FAT access - Read value of an FAT entry                               */
/*-----------------------------------------------------------------------*/

static DWORD get_fat (		/* 0xFFFFFFFF:Disk error, 1:Internal error, 2..0x7FFFFFFF:Cluster status */
	FFOBJID *obj,	/* Corresponding object */
	DWORD clst		/* Cluster number to get the value */
)
{
	UINT wc, bc;
	DWORD val;
	FATFS *fs = obj->fs;


	if (clst < 2 || clst >= fs->n_fatent) {	/* Check if in valid range */
		val = 1;	/* Internal error */

	}
	else {
		val = 0xFFFFFFFF;	/* Default value falls on disk error */

		switch (fs->fs_type) {
			case FS_FAT12 :
				bc = (UINT)clst;
				bc += bc / 2;
				if (move_window(fs, fs->fatbase + (bc / SS(fs))) != FR_OK) {
					break;
				}
				wc = fs->win[bc++ % SS(fs)];		/* Get 1st byte of the entry */
				if (move_window(fs, fs->fatbase + (bc / SS(fs))) != FR_OK) {
					break;
				}
				wc |= fs->win[bc % SS(fs)] << 8;	/* Merge 2nd byte of the entry */
				val = (clst & 1) ? (wc >> 4) : (wc & 0xFFF);	/* Adjust bit position */
				break;

			case FS_FAT16 :
				if (move_window(fs, fs->fatbase + (clst / (SS(fs) / 2))) != FR_OK) {
					break;
				}
				val = ld_16(fs->win + clst * 2 % SS(fs));		/* Simple WORD array */
				break;

			case FS_FAT32 :
				if (move_window(fs, fs->fatbase + (clst / (SS(fs) / 4))) != FR_OK) {
					break;
				}
				val = ld_32(fs->win + clst * 4 % SS(fs)) & 0x0FFFFFFF;	/* Simple DWORD array but mask out upper 4 bits */
				break;
#if FF_FS_EXFAT
			case FS_EXFAT :
				if ((obj->objsize != 0 && obj->sclust != 0)
				    || obj->stat == 0) {	/* Object except root dir must have valid data length */
					DWORD cofs = clst - obj->sclust;	/* Offset from start cluster */
					DWORD clen = (DWORD)((LBA_t)((obj->objsize - 1) / SS(fs)) / fs->csize);	/* Number of clusters - 1 */

					if (obj->stat == 2 && cofs <= clen) {	/* Is it a contiguous chain? */
						val = (cofs == clen) ? 0x7FFFFFFF : clst + 1;	/* No data on the FAT, generate the value */
						break;
					}
					if (obj->stat == 3 && cofs < obj->n_cont) {	/* Is it in the 1st fragment? */
						val = clst + 1; 	/* Generate the value */
						break;
					}
					if (obj->stat != 2) {	/* Get value from FAT if FAT chain is valid */
						if (obj->n_frag != 0) {	/* Is it on the growing edge? */
							val = 0x7FFFFFFF;	/* Generate EOC */
						}
						else {
							if (move_window(fs, fs->fatbase + (clst / (SS(fs) / 4))) != FR_OK) {
								break;
							}
							val = ld_32(fs->win + clst * 4 % SS(fs)) & 0x7FFFFFFF;
						}
						break;
					}
				}
				val = 1;	/* Internal error */
				break;
#endif
			default:
				val = 1;	/* Internal error */
		}
	}

	return val;
}




#if !FF_FS_READONLY
/*-----------------------------------------------------------------------*/
/* FAT access - Change value of an FAT entry                             */
/*-----------------------------------------------------------------------*/

static FRESULT put_fat (	/* FR_OK(0):succeeded, !=0:error */
	FATFS *fs,		/* Corresponding filesystem object */
	DWORD clst,		/* FAT index number (cluster number) to be changed */
	DWORD val		/* New value to be set to the entry */
)
{
	UINT bc;
	BYTE *p;
	FRESULT res = FR_INT_ERR;


	if (clst >= 2 && clst < fs->n_fatent) {	/* Check if in valid range */
		switch (fs->fs_type) {
			case FS_FAT12:
				bc = (UINT)clst;
				bc += bc / 2;	/* bc: byte offset of the entry */
				res = move_window(fs, fs->fatbase + (bc / SS(fs)));
				if (res != FR_OK) {
					break;
				}
				p = fs->win + bc++ % SS(fs);
				*p = (clst & 1) ? ((*p & 0x0F) | ((BYTE)val << 4)) : (BYTE)val;	/* Update 1st byte */
				fs->wflag = 1;
				res = move_window(fs, fs->fatbase + (bc / SS(fs)));
				if (res != FR_OK) {
					break;
				}
				p = fs->win + bc % SS(fs);
				*p = (clst & 1) ? (BYTE)(val >> 4) : ((*p & 0xF0) | ((BYTE)(val >> 8) & 0x0F));	/* Update 2nd byte */
				fs->wflag = 1;
				break;

			case FS_FAT16:
				res = move_window(fs, fs->fatbase + (clst / (SS(fs) / 2)));
				if (res != FR_OK) {
					break;
				}
				st_16(fs->win + clst * 2 % SS(fs), (WORD)val);	/* Simple WORD array */
				fs->wflag = 1;
				break;

			case FS_FAT32:
#if FF_FS_EXFAT
			case FS_EXFAT:
#endif
				res = move_window(fs, fs->fatbase + (clst / (SS(fs) / 4)));
				if (res != FR_OK) {
					break;
				}
				if (!FF_FS_EXFAT || fs->fs_type != FS_EXFAT) {
					val = (val & 0x0FFFFFFF) | (ld_32(fs->win + clst * 4 % SS(fs)) & 0xF0000000);
				}
				st_32(fs->win + clst * 4 % SS(fs), val);
				fs->wflag = 1;
				break;
		}
	}
	return res;
}

#endif /* !FF_FS_READONLY */




#if FF_FS_EXFAT && !FF_FS_READONLY
/*-----------------------------------------------------------------------*/
/* exFAT: Accessing FAT and Allocation Bitmap                            */
/*-----------------------------------------------------------------------*/

/*--------------------------------------*/
/* Find a contiguous free cluster block */
/*--------------------------------------*/

static DWORD find_bitmap (	/* 0:Not found, 2..:Cluster block found, 0xFFFFFFFF:Disk error */
	FATFS *fs,	/* Filesystem object */
	DWORD clst,	/* Cluster number to scan from */
	DWORD ncl	/* Number of contiguous clusters to find (1..) */
)
{
	BYTE bm, bv;
	UINT i;
	DWORD val, scl, ctr;


	clst -= 2;	/* The first bit in the bitmap corresponds to cluster #2 */
	if (clst >= fs->n_fatent - 2) {
		clst = 0;
	}
	scl = val = clst;
	ctr = 0;
	for (;;) {
		if (move_window(fs, fs->bitbase + val / 8 / SS(fs)) != FR_OK) {
			return 0xFFFFFFFF;
		}
		i = val / 8 % SS(fs);
		bm = 1 << (val % 8);
		do {
			do {
				bv = fs->win[i] & bm;
				bm <<= 1;		/* Get bit value */
				if (++val >= fs->n_fatent - 2) {	/* Next cluster (with wrap-around) */
					val = 0;
					bm = 0;
					i = SS(fs);
				}
				if (bv == 0) {	/* Is it a free cluster? */
					if (++ctr == ncl) {
						return scl + 2;        /* Check if run length is sufficient for required */
					}
				}
				else {
					scl = val;
					ctr = 0;		/* Encountered a cluster in-use, restart to scan */
				}
				if (val == clst) {
					return 0;        /* All cluster scanned? */
				}
			}
			while (bm != 0);
			bm = 1;
		}
		while (++i < SS(fs));
	}
}


/*----------------------------------------*/
/* Set/Clear a block of allocation bitmap */
/*----------------------------------------*/

static FRESULT change_bitmap (
	FATFS *fs,	/* Filesystem object */
	DWORD clst,	/* Cluster number to change from */
	DWORD ncl,	/* Number of clusters to be changed */
	int bv		/* bit value to be set (0 or 1) */
)
{
	BYTE bm;
	UINT i;
	LBA_t sect;


	clst -= 2;	/* The first bit corresponds to cluster #2 */
	sect = fs->bitbase + clst / 8 / SS(fs);	/* Sector address */
	i = clst / 8 % SS(fs);					/* Byte offset in the sector */
	bm = 1 << (clst % 8);					/* Bit mask in the byte */
	for (;;) {
		if (move_window(fs, sect++) != FR_OK) {
			return FR_DISK_ERR;
		}
		do {
			do {
				if (bv == (int)((fs->win[i] & bm) != 0)) {
					return FR_INT_ERR;        /* Is the bit expected value? */
				}
				fs->win[i] ^= bm;	/* Flip the bit */
				fs->wflag = 1;
				if (--ncl == 0) {
					return FR_OK;        /* All bits processed? */
				}
			}
			while (bm <<= 1);		/* Next bit */
			bm = 1;
		}
		while (++i < SS(fs));		/* Next byte */
		i = 0;
	}
}


/*---------------------------------------------*/
/* Fill the first fragment of the FAT chain    */
/*---------------------------------------------*/

static FRESULT fill_first_frag (
	FFOBJID *obj	/* Pointer to the corresponding object */
)
{
	FRESULT res = FR_DISK_ERR;
	DWORD cl, n;


	if (obj->stat == 3) {	/* Has the object been changed 'fragmented' in this session? */
		for (cl = obj->sclust, n = obj->n_cont; n; cl++, n--) {	/* Create cluster chain on the FAT */
			res = put_fat(obj->fs, cl, cl + 1);
			if (res != FR_OK) {
				return res;
			}
		}
		obj->stat = 0;	/* Change status 'FAT chain is valid' */
	}
	return FR_OK;
}


/*---------------------------------------------*/
/* Fill the last fragment of the FAT chain     */
/*---------------------------------------------*/

static FRESULT fill_last_frag (
	FFOBJID *obj,	/* Pointer to the corresponding object */
	DWORD lcl,		/* Last cluster of the fragment */
	DWORD term		/* Value to set the last FAT entry */
)
{
	FRESULT res = FR_DISK_ERR;


	while (obj->n_frag > 0) {	/* Create the chain of last fragment */
		res = put_fat(obj->fs, lcl - obj->n_frag + 1, (obj->n_frag > 1) ? lcl - obj->n_frag + 2 : term);
		if (res != FR_OK) {
			return res;
		}
		obj->n_frag--;
	}
	return FR_OK;
}

#endif	/* FF_FS_EXFAT && !FF_FS_READONLY */



#if !FF_FS_READONLY
/*-----------------------------------------------------------------------*/
/* FAT handling - Remove a cluster chain                                 */
/*-----------------------------------------------------------------------*/

static FRESULT remove_chain (	/* FR_OK(0):succeeded, !=0:error */
	FFOBJID *obj,		/* Corresponding object */
	DWORD clst,			/* Cluster to remove a chain from */
	DWORD pclst			/* Previous cluster of clst (0 if entire chain) */
)
{
	FRESULT res = FR_DISK_ERR;
	DWORD nxt;
	FATFS *fs = obj->fs;
#if FF_FS_EXFAT || FF_USE_TRIM
	DWORD scl = clst, ecl = clst;
#endif
#if FF_USE_TRIM
	LBA_t rt[2];
#endif

	if (clst < 2 || clst >= fs->n_fatent) {
		return FR_INT_ERR;        /* Check if in valid range */
	}

	/* Mark the previous cluster 'EOC' on the FAT if it exists */
	if (pclst != 0 && (!FF_FS_EXFAT || fs->fs_type != FS_EXFAT || obj->stat != 2)) {
		res = put_fat(fs, pclst, 0xFFFFFFFF);
		if (res != FR_OK) {
			return res;
		}
	}

	/* Remove the chain */
	do {
		nxt = get_fat(obj, clst);			/* Get cluster status */
		if (nxt == 0) {
			break;        /* Empty cluster? */
		}
		if (nxt == 1) {
			return FR_INT_ERR;        /* Internal error? */
		}
		if (nxt == 0xFFFFFFFF) {
			return FR_DISK_ERR;        /* Disk error? */
		}
		if (!FF_FS_EXFAT || fs->fs_type != FS_EXFAT) {
			res = put_fat(fs, clst, 0);		/* Mark the cluster 'free' on the FAT */
			if (res != FR_OK) {
				return res;
			}
		}
		if (fs->free_clst < fs->n_fatent - 2) {	/* Update allocation information if it is valid */
			fs->free_clst++;
			fs->fsi_flag |= 1;
		}
#if FF_FS_EXFAT || FF_USE_TRIM
		if (ecl + 1 == nxt) {	/* Is next cluster contiguous? */
			ecl = nxt;
		}
		else {				/* End of contiguous cluster block */
#if FF_FS_EXFAT
			if (fs->fs_type == FS_EXFAT) {
				res = change_bitmap(fs, scl, ecl - scl + 1, 0);	/* Mark the cluster block 'free' on the bitmap */
				if (res != FR_OK) {
					return res;
				}
			}
#endif
#if FF_USE_TRIM
			rt[0] = clst2sect(fs, scl);					/* Start of data area to be freed */
			rt[1] = clst2sect(fs, ecl) + fs->csize - 1;	/* End of data area to be freed */
			disk_ioctl(fs->pdrv, CTRL_TRIM, rt);		/* Inform storage device that the data in the block may be erased */
#endif
			scl = ecl = nxt;
		}
#endif
		clst = nxt;					/* Next cluster */
	}
	while (clst < fs->n_fatent);	/* Repeat until the last link */

#if FF_FS_EXFAT
	/* Some post processes for chain status */
	if (fs->fs_type == FS_EXFAT) {
		if (pclst == 0) {	/* Has the entire chain been removed? */
			obj->stat = 0;		/* Change the chain status 'initial' */
		}
		else {
			if (obj->stat == 0) {	/* Is it a fragmented chain from the beginning of this session? */
				clst = obj->sclust;		/* Follow the chain to check if it gets contiguous */
				while (clst != pclst) {
					nxt = get_fat(obj, clst);
					if (nxt < 2) {
						return FR_INT_ERR;
					}
					if (nxt == 0xFFFFFFFF) {
						return FR_DISK_ERR;
					}
					if (nxt != clst + 1) {
						break;        /* Not contiguous? */
					}
					clst++;
				}
				if (clst == pclst) {	/* Has the chain got contiguous again? */
					obj->stat = 2;		/* Change the chain status 'contiguous' */
				}
			}
			else {
				if (obj->stat == 3 && pclst >= obj->sclust
				    && pclst <= obj->sclust + obj->n_cont) {	/* Was the chain fragmented in this session and got contiguous again? */
					obj->stat = 2;	/* Change the chain status 'contiguous' */
				}
			}
		}
	}
#endif
	return FR_OK;
}




/*-----------------------------------------------------------------------*/
/* FAT handling - Stretch a chain or Create a new chain                  */
/*-----------------------------------------------------------------------*/

static DWORD create_chain (	/* 0:No free cluster, 1:Internal error, 0xFFFFFFFF:Disk error, >=2:New cluster# */
	FFOBJID *obj,		/* Corresponding object */
	DWORD clst			/* Cluster# to stretch, 0:Create a new chain */
)
{
	DWORD cs, ncl, scl;
	FRESULT res = FR_DISK_ERR;
	FATFS *fs = obj->fs;


	if (clst == 0) {	/* Create a new chain */
		scl = fs->last_clst;				/* Suggested cluster to start to find */
		if (scl == 0 || scl >= fs->n_fatent) {
			scl = 1;
		}
	}
	else {				/* Stretch a chain */
		cs = get_fat(obj, clst);			/* Check the cluster status */
		if (cs < 2) {
			return 1;        /* Test for insanity */
		}
		if (cs == 0xFFFFFFFF) {
			return cs;        /* Test for disk error */
		}
		if (cs < fs->n_fatent) {
			return cs;        /* It is already followed by next cluster */
		}
		scl = clst;							/* Cluster to start to find */
	}
	if (fs->free_clst == 0) {
		return 0;        /* No free cluster */
	}

#if FF_FS_EXFAT
	if (fs->fs_type == FS_EXFAT) {	/* On the exFAT volume */
		ncl = find_bitmap(fs, scl, 1);				/* Find a free cluster */
		if (ncl == 0 || ncl == 0xFFFFFFFF) {
			return ncl;        /* No free cluster or hard error? */
		}
		res = change_bitmap(fs, ncl, 1, 1);			/* Mark the cluster 'in use' */
		if (res == FR_INT_ERR) {
			return 1;
		}
		if (res == FR_DISK_ERR) {
			return 0xFFFFFFFF;
		}
		if (clst == 0) {							/* Is it a new chain? */
			obj->stat = 2;							/* Set status 'contiguous' */
		}
		else {									/* It is a stretched chain */
			if (obj->stat == 2 && ncl != scl + 1) {	/* Is the chain got fragmented? */
				obj->n_cont = scl - obj->sclust;	/* Set size of the contiguous part */
				obj->stat = 3;						/* Change status 'just fragmented' */
			}
		}
		if (obj->stat != 2) {	/* Is the file non-contiguous? */
			if (ncl == clst + 1) {	/* Is the cluster next to previous one? */
				obj->n_frag = obj->n_frag ? obj->n_frag + 1 : 2;	/* Increment size of last framgent */
			}
			else {				/* New fragment */
				if (obj->n_frag == 0) {
					obj->n_frag = 1;
				}
				res = fill_last_frag(obj, clst, ncl);	/* Fill last fragment on the FAT and link it to new one */
				if (res == FR_OK) {
					obj->n_frag = 1;
				}
			}
		}
	}
	else
#endif
	{	/* On the FAT/FAT32 volume */
		ncl = 0;
		if (scl == clst) {						/* Stretching an existing chain? */
			ncl = scl + 1;						/* Test if next cluster is free */
			if (ncl >= fs->n_fatent) {
				ncl = 2;
			}
			cs = get_fat(obj, ncl);				/* Get next cluster status */
			if (cs == 1 || cs == 0xFFFFFFFF) {
				return cs;        /* Test for error */
			}
			if (cs != 0) {						/* Not free? */
				cs = fs->last_clst;				/* Start at suggested cluster if it is valid */
				if (cs >= 2 && cs < fs->n_fatent) {
					scl = cs;
				}
				ncl = 0;
			}
		}
		if (ncl == 0) {	/* The new cluster cannot be contiguous and find another fragment */
			ncl = scl;	/* Start cluster */
			for (;;) {
				ncl++;							/* Next cluster */
				if (ncl >= fs->n_fatent) {		/* Check wrap-around */
					ncl = 2;
					if (ncl > scl) {
						return 0;        /* No free cluster found? */
					}
				}
				cs = get_fat(obj, ncl);			/* Get the cluster status */
				if (cs == 0) {
					break;        /* Found a free cluster? */
				}
				if (cs == 1 || cs == 0xFFFFFFFF) {
					return cs;        /* Test for error */
				}
				if (ncl == scl) {
					return 0;        /* No free cluster found? */
				}
			}
		}
		res = put_fat(fs, ncl, 0xFFFFFFFF);		/* Mark the new cluster 'EOC' */
		if (res == FR_OK && clst != 0) {
			res = put_fat(fs, clst, ncl);		/* Link it from the previous one if needed */
		}
	}

	if (res == FR_OK) {			/* Update allocation information if the function succeeded */
		fs->last_clst = ncl;
		if (fs->free_clst > 0 && fs->free_clst <= fs->n_fatent - 2) {
			fs->free_clst--;
			fs->fsi_flag |= 1;
		}
	}
	else {
		ncl = (res == FR_DISK_ERR) ? 0xFFFFFFFF : 1;	/* Failed. Generate error status */
	}

	return ncl;		/* Return new cluster number or error status */
}

#endif /* !FF_FS_READONLY */




#if FF_USE_FASTSEEK
/*-----------------------------------------------------------------------*/
/* FAT handling - Convert offset into cluster with link map table        */
/*-----------------------------------------------------------------------*/

static DWORD clmt_clust (	/* <2:Error, >=2:Cluster number */
	FIL *fp,		/* Pointer to the file object */
	FSIZE_t ofs		/* File offset to be converted to cluster# */
)
{
	DWORD cl, ncl;
	DWORD *tbl;
	FATFS *fs = fp->obj.fs;


	tbl = fp->cltbl + 1;	/* Top of CLMT */
	cl = (DWORD)(ofs / SS(fs) / fs->csize);	/* Cluster order from top of the file */
	for (;;) {
		ncl = *tbl++;			/* Number of cluters in the fragment */
		if (ncl == 0) {
			return 0;        /* End of table? (error) */
		}
		if (cl < ncl) {
			break;        /* In this fragment? */
		}
		cl -= ncl;
		tbl++;		/* Next fragment */
	}
	return cl + *tbl;	/* Return the cluster number */
}

#endif	/* FF_USE_FASTSEEK */




/*-----------------------------------------------------------------------*/
/* Directory handling - Fill a cluster with zeros                        */
/*-----------------------------------------------------------------------*/

#if !FF_FS_READONLY
static FRESULT dir_clear (	/* Returns FR_OK or FR_DISK_ERR */
	FATFS *fs,		/* Filesystem object */
	DWORD clst		/* Directory table to clear */
)
{
	LBA_t sect;
	UINT n, szb;
	BYTE *ibuf;


	if (sync_window(fs) != FR_OK) {
		return FR_DISK_ERR;        /* Flush disk access window */
	}
	sect = clst2sect(fs, clst);		/* Top of the cluster */
	fs->winsect = sect;				/* Set window to top of the cluster */
	memset(fs->win, 0, sizeof fs->win);	/* Clear window buffer */
#if FF_USE_LFN == 3		/* Quick table clear by using multi-secter write */
	/* Allocate a temporary buffer */
	for (szb = ((DWORD)fs->csize * SS(fs) >= MAX_MALLOC) ? MAX_MALLOC : fs->csize * SS(fs), ibuf = 0; szb > SS(fs)
	     && (ibuf = ff_memalloc(szb)) == 0; szb /= 2) ;
	if (szb > SS(fs)) {		/* Buffer allocated? */
		memset(ibuf, 0, szb);
		szb /= SS(fs);		/* Bytes -> Sectors */
		for (n = 0; n < fs->csize
		     && disk_write(fs->pdrv, ibuf, sect + n, szb) == RES_OK; n += szb) ;	/* Fill the cluster with 0 */
		ff_memfree(ibuf);
	}
	else
#endif
	{
		ibuf = fs->win;
		szb = 1;	/* Use window buffer (many single-sector writes may take a time) */
		for (n = 0; n < fs->csize
		     && disk_write(fs->pdrv, ibuf, sect + n, szb) == RES_OK; n += szb) ;	/* Fill the cluster with 0 */
	}
	return (n == fs->csize) ? FR_OK : FR_DISK_ERR;
}
#endif	/* !FF_FS_READONLY */




/*-----------------------------------------------------------------------*/
/* Directory handling - Set directory index                              */
/*-----------------------------------------------------------------------*/

static FRESULT dir_sdi (	/* FR_OK(0):succeeded, !=0:error */
	DIR *dp,		/* Pointer to directory object */
	DWORD ofs		/* Offset of directory table */
)
{
	DWORD csz, clst;
	FATFS *fs = dp->obj.fs;


	if (ofs >= (DWORD)((FF_FS_EXFAT && fs->fs_type == FS_EXFAT) ? MAX_DIR_EX : MAX_DIR)
	    || ofs % SZDIRE) {	/* Check range of offset and alignment */
		return FR_INT_ERR;
	}
	dp->dptr = ofs;				/* Set current offset */
	clst = dp->obj.sclust;		/* Table start cluster (0:root) */
	if (clst == 0 && fs->fs_type >= FS_FAT32) {	/* Replace cluster# 0 with root cluster# */
		clst = (DWORD)fs->dirbase;
		if (FF_FS_EXFAT) {
			dp->obj.stat = 0;        /* exFAT: Root dir has an FAT chain */
		}
	}

	if (clst == 0) {	/* Static table (root-directory on the FAT volume) */
		if (ofs / SZDIRE >= fs->n_rootdir) {
			return FR_INT_ERR;        /* Is index out of range? */
		}
		dp->sect = fs->dirbase;

	}
	else {			/* Dynamic table (sub-directory or root-directory on the FAT32/exFAT volume) */
		csz = (DWORD)fs->csize * SS(fs);	/* Bytes per cluster */
		while (ofs >= csz) {				/* Follow cluster chain */
			clst = get_fat(&dp->obj, clst);				/* Get next cluster */
			if (clst == 0xFFFFFFFF) {
				return FR_DISK_ERR;        /* Disk error */
			}
			if (clst < 2 || clst >= fs->n_fatent) {
				return FR_INT_ERR;        /* Reached to end of table or internal error */
			}
			ofs -= csz;
		}
		dp->sect = clst2sect(fs, clst);
	}
	dp->clust = clst;					/* Current cluster# */
	if (dp->sect == 0) {
		return FR_INT_ERR;
	}
	dp->sect += ofs / SS(fs);			/* Sector# of the directory entry */
	dp->dir = fs->win + (ofs % SS(fs));	/* Pointer to the entry in the win[] */

	return FR_OK;
}




/*-----------------------------------------------------------------------*/
/* Directory handling - Move directory table index next                  */
/*-----------------------------------------------------------------------*/

static FRESULT dir_next (	/* FR_OK(0):succeeded, FR_NO_FILE:End of table, FR_DENIED:Could not stretch */
	DIR *dp,				/* Pointer to the directory object */
	int stretch				/* 0: Do not stretch table, 1: Stretch table if needed */
)
{
	DWORD ofs, clst;
	FATFS *fs = dp->obj.fs;


	ofs = dp->dptr + SZDIRE;	/* Next entry */
	if (ofs >= (DWORD)((FF_FS_EXFAT && fs->fs_type == FS_EXFAT) ? MAX_DIR_EX : MAX_DIR)) {
		dp->sect = 0;        /* Disable it if the offset reached the max value */
	}
	if (dp->sect == 0) {
		return FR_NO_FILE;        /* Report EOT if it has been disabled */
	}

	if (ofs % SS(fs) == 0) {	/* Sector changed? */
		dp->sect++;				/* Next sector */

		if (dp->clust == 0) {	/* Static table */
			if (ofs / SZDIRE >= fs->n_rootdir) {	/* Report EOT if it reached end of static table */
				dp->sect = 0;
				return FR_NO_FILE;
			}
		}
		else {					/* Dynamic table */
			if ((ofs / SS(fs) & (fs->csize - 1)) == 0) {	/* Cluster changed? */
				clst = get_fat(&dp->obj, dp->clust);		/* Get next cluster */
				if (clst <= 1) {
					return FR_INT_ERR;        /* Internal error */
				}
				if (clst == 0xFFFFFFFF) {
					return FR_DISK_ERR;        /* Disk error */
				}
				if (clst >= fs->n_fatent) {					/* It reached end of dynamic table */
#if !FF_FS_READONLY
					if (!stretch) {								/* If no stretch, report EOT */
						dp->sect = 0;
						return FR_NO_FILE;
					}
					clst = create_chain(&dp->obj, dp->clust);	/* Allocate a cluster */
					if (clst == 0) {
						return FR_DENIED;        /* No free cluster */
					}
					if (clst == 1) {
						return FR_INT_ERR;        /* Internal error */
					}
					if (clst == 0xFFFFFFFF) {
						return FR_DISK_ERR;        /* Disk error */
					}
					if (dir_clear(fs, clst) != FR_OK) {
						return FR_DISK_ERR;        /* Clean up the stretched table */
					}
					if (FF_FS_EXFAT) {
						dp->obj.stat |= 4;        /* exFAT: The directory has been stretched */
					}
#else
					if (!stretch) {
						dp->sect = 0;        /* (this line is to suppress compiler warning) */
					}
					dp->sect = 0;
					return FR_NO_FILE;			/* Report EOT */
#endif
				}
				dp->clust = clst;		/* Initialize data for new cluster */
				dp->sect = clst2sect(fs, clst);
			}
		}
	}
	dp->dptr = ofs;						/* Current entry */
	dp->dir = fs->win + ofs % SS(fs);	/* Pointer to the entry in the win[] */

	return FR_OK;
}




#if !FF_FS_READONLY
/*-----------------------------------------------------------------------*/
/* Directory handling - Reserve a block of directory entries             */
/*-----------------------------------------------------------------------*/

static FRESULT dir_alloc (	/* FR_OK(0):succeeded, !=0:error */
	DIR *dp,				/* Pointer to the directory object */
	UINT n_ent				/* Number of contiguous entries to allocate */
)
{
	FRESULT res = FR_DISK_ERR;
	UINT n;
	FATFS *fs = dp->obj.fs;


	res = dir_sdi(dp, 0);
	if (res == FR_OK) {
		n = 0;
		do {
			res = move_window(fs, dp->sect);
			if (res != FR_OK) {
				break;
			}
#if FF_FS_EXFAT
			if ((fs->fs_type == FS_EXFAT) ? (int)((dp->dir[XDIR_Type] & 0x80) == 0) : (int)(dp->dir[DIR_Name] == DDEM
					|| dp->dir[DIR_Name] == 0)) {	/* Is the entry free? */
#else
			if (dp->dir[DIR_Name] == DDEM || dp->dir[DIR_Name] == 0) {	/* Is the entry free? */
#endif
				if (++n == n_ent) {
					break;        /* Is a block of contiguous free entries found? */
				}
			}
			else {
				n = 0;				/* Not a free entry, restart to search */
			}
			res = dir_next(dp, 1);	/* Next entry with table stretch enabled */
		}
		while (res == FR_OK);
	}

	if (res == FR_NO_FILE) {
		res = FR_DENIED;        /* No directory entry to allocate */
	}
	return res;
}

#endif	/* !FF_FS_READONLY */




/*-----------------------------------------------------------------------*/
/* FAT: Directory handling - Load/Store start cluster number             */
/*-----------------------------------------------------------------------*/

static DWORD ld_clust (	/* Returns the top cluster value of the SFN entry */
	FATFS *fs,			/* Pointer to the fs object */
	const BYTE *dir		/* Pointer to the key entry */
)
{
	DWORD cl;

	cl = ld_16(dir + DIR_FstClusLO);
	if (fs->fs_type == FS_FAT32) {
		cl |= (DWORD)ld_16(dir + DIR_FstClusHI) << 16;
	}

	return cl;
}


#if !FF_FS_READONLY
static void st_clust (
	FATFS *fs,	/* Pointer to the fs object */
	BYTE *dir,	/* Pointer to the key entry */
	DWORD cl	/* Value to be set */
)
{
	st_16(dir + DIR_FstClusLO, (WORD)cl);
	if (fs->fs_type == FS_FAT32) {
		st_16(dir + DIR_FstClusHI, (WORD)(cl >> 16));
	}
}
#endif



#if FF_USE_LFN
/*--------------------------------------------------------*/
/* FAT-LFN: Compare a part of file name with an LFN entry */
/*--------------------------------------------------------*/

static int cmp_lfn (		/* 1:matched, 0:not matched */
	const WCHAR *lfnbuf,	/* Pointer to the LFN to be compared */
	BYTE *dir				/* Pointer to the LFN entry */
)
{
	UINT ni, di;
	WCHAR pchr, chr;


	if (ld_16(dir + LDIR_FstClusLO) != 0) {
		return 0;        /* Check if LDIR_FstClusLO is 0 */
	}

	ni = (UINT)((dir[LDIR_Ord] & 0x3F) - 1) * 13;	/* Offset in the name to be compared */

	for (pchr = 1, di = 0; di < 13; di++) {	/* Process all characters in the entry */
		chr = ld_16(dir + LfnOfs[di]);	/* Pick a character from the entry */
		if (pchr != 0) {
			if (ni >= FF_MAX_LFN + 1 || ff_wtoupper(chr) != ff_wtoupper(lfnbuf[ni++])) {	/* Compare it with name */
				return 0;					/* Not matched */
			}
			pchr = chr;
		}
		else {
			if (chr != 0xFFFF) {
				return 0;	/* Check filler */
			}
		}
	}

	if ((dir[LDIR_Ord] & LLEF) && pchr && lfnbuf[ni]) {
		return 0;        /* Last name segment matched but different length */
	}

	return 1;		/* The part of LFN matched */
}


#if FF_FS_MINIMIZE <= 1 || FF_FS_RPATH >= 2 || FF_USE_LABEL || FF_FS_EXFAT
/*-----------------------------------------------------*/
/* FAT-LFN: Pick a part of file name from an LFN entry */
/*-----------------------------------------------------*/

static int pick_lfn (	/* 1:succeeded, 0:buffer overflow or invalid LFN entry */
	WCHAR *lfnbuf,		/* Pointer to the name buffer to be stored */
	const BYTE *dir		/* Pointer to the LFN entry */
)
{
	UINT ni, di;
	WCHAR pchr, chr;


	if (ld_16(dir + LDIR_FstClusLO) != 0) {
		return 0;        /* Check LDIR_FstClusLO if is 0 */
	}

	ni = (UINT)((dir[LDIR_Ord] & ~LLEF) - 1) * 13;	/* Offset in the name buffer */

	for (pchr = 1, di = 0; di < 13; di++) {		/* Process all characters in the entry */
		chr = ld_16(dir + LfnOfs[di]);		/* Pick a character from the entry */
		if (pchr != 0) {
			if (ni >= FF_MAX_LFN + 1) {
				return 0;        /* Buffer overflow? */
			}
			lfnbuf[ni++] = pchr = chr;			/* Store it */
		}
		else {
			if (chr != 0xFFFF) {
				return 0;        /* Check filler */
			}
		}
	}

	if (dir[LDIR_Ord] & LLEF && pchr != 0) {	/* Put terminator if it is the last LFN part and not terminated */
		if (ni >= FF_MAX_LFN + 1) {
			return 0;	/* Buffer overflow? */
		}
		lfnbuf[ni] = 0;
	}

	return 1;		/* The part of LFN is valid */
}
#endif


#if !FF_FS_READONLY
/*-----------------------------------------*/
/* FAT-LFN: Create an entry of LFN entries */
/*-----------------------------------------*/

static void put_lfn (
	const WCHAR *lfn,	/* Pointer to the LFN */
	BYTE *dir,			/* Pointer to the LFN entry to be created */
	BYTE ord,			/* LFN order (1-20) */
	BYTE sum			/* Checksum of the corresponding SFN */
)
{
	UINT ni, di;
	WCHAR chr;


	dir[LDIR_Chksum] = sum;			/* Set checksum */
	dir[LDIR_Attr] = AM_LFN;		/* Set attribute */
	dir[LDIR_Type] = 0;
	st_16(dir + LDIR_FstClusLO, 0);

	ni = (UINT)(ord - 1) * 13;		/* Offset in the name */
	di = chr = 0;
	do {	/* Fill the directory entry */
		if (chr != 0xFFFF) {
			chr = lfn[ni++];        /* Get an effective character */
		}
		st_16(dir + LfnOfs[di], chr);	/* Set it */
		if (chr == 0) {
			chr = 0xFFFF;	/* Padding characters after the terminator */
		}
	}
	while (++di < 13);
	if (chr == 0xFFFF || !lfn[ni]) {
		ord |= LLEF;	/* Last LFN part is the start of an entry set */
	}
	dir[LDIR_Ord] = ord;			/* Set order in the entry set */
}

#endif	/* !FF_FS_READONLY */
#endif	/* FF_USE_LFN */



#if FF_USE_LFN && !FF_FS_READONLY
/*-----------------------------------------------------------------------*/
/* FAT-LFN: Create a Numbered SFN                                        */
/*-----------------------------------------------------------------------*/

static void gen_numname (
	BYTE *dst,			/* Pointer to the buffer to store numbered SFN */
	const BYTE *src,	/* Pointer to SFN in directory form */
	const WCHAR *lfn,	/* Pointer to LFN */
	WORD seq			/* Sequence number */
)
{
	BYTE ns[8], c;
	UINT i, j;


	mem_cpy(dst, src, 11);	/* Prepare the SFN to be modified */

	if (seq > 5) {	/* In case of many collisions, generate a hash number instead of sequential number */
		WCHAR wc;
		DWORD crc_sreg = seq;

		while (*lfn) {	/* Create a CRC value as a hash of LFN */
			wc = *lfn++;
			for (i = 0; i < 16; i++) {
				crc_sreg = (crc_sreg << 1) + (wc & 1);
				wc >>= 1;
				if (crc_sreg & 0x10000) {
					crc_sreg ^= 0x11021;
				}
			}
		}
		seq = (WORD)crc_sreg;
	}

	/* Make suffix (~ + 4-digit hexadecimal) */
	i = 7;
	do {
		c = (BYTE)((seq % 16) + '0');
		seq /= 16;
		if (c > '9') {
			c += 7;
		}
		ns[i--] = c;
	}
	while (i && seq);
	ns[i] = '~';

	/* Append the suffix to the SFN body */
	for (j = 0; j < i && dst[j] != ' '; j++) {	/* Find the offset to append */
		if (dbc_1st(dst[j])) {	/* To avoid DBC break up */
			if (j == i - 1) {
				break;
			}
			j++;
		}
	}
	do {	/* Append the suffix */
		dst[j++] = (i < 8) ? ns[i++] : ' ';
	}
	while (j < 8);
}
#endif	/* FF_USE_LFN && !FF_FS_READONLY */



#if FF_USE_LFN
/*-----------------------------------------------------------------------*/
/* FAT-LFN: Calculate checksum of an SFN entry                           */
/*-----------------------------------------------------------------------*/

static BYTE sum_sfn (
	const BYTE *dir		/* Pointer to the SFN entry */
)
{
	BYTE sum = 0;
	UINT n = 11;

	do {
		sum = (sum >> 1) + (sum << 7) + *dir++;
	}
	while (--n);
	return sum;
}

#endif	/* FF_USE_LFN */



#if FF_FS_EXFAT
/*-----------------------------------------------------------------------*/
/* exFAT: Checksum                                                       */
/*-----------------------------------------------------------------------*/

static WORD xdir_sum (	/* Get checksum of the directoly entry block */
	const BYTE *dir		/* Directory entry block to be calculated */
)
{
	UINT i, szblk;
	WORD sum;


	szblk = ((UINT)dir[XDIR_NumSec] + 1) * SZDIRE;	/* Number of bytes of the entry block */
	for (i = sum = 0; i < szblk; i++) {
		if (i == XDIR_SetSum) {	/* Skip 2-byte sum field */
			i++;
		}
		else {
			sum = ((sum & 1) ? 0x8000 : 0) + (sum >> 1) + dir[i];
		}
	}
	return sum;
}



static WORD xname_sum (	/* Get check sum (to be used as hash) of the file name */
	const WCHAR *name	/* File name to be calculated */
)
{
	WCHAR chr;
	WORD sum = 0;


	while ((chr = *name++) != 0) {
		chr = (WCHAR)ff_wtoupper(chr);		/* File name needs to be up-case converted */
		sum = ((sum & 1) ? 0x8000 : 0) + (sum >> 1) + (chr & 0xFF);
		sum = ((sum & 1) ? 0x8000 : 0) + (sum >> 1) + (chr >> 8);
	}
	return sum;
}


#if !FF_FS_READONLY && FF_USE_MKFS
static DWORD xsum32 (	/* Returns 32-bit checksum */
	BYTE  dat,			/* Byte to be calculated (byte-by-byte processing) */
	DWORD sum			/* Previous sum value */
)
{
	sum = ((sum & 1) ? 0x80000000 : 0) + (sum >> 1) + dat;
	return sum;
}
#endif



/*------------------------------------*/
/* exFAT: Get a directory entry block */
/*------------------------------------*/

static FRESULT load_xdir (	/* FR_INT_ERR: invalid entry block */
	DIR *dp					/* Reading directory object pointing top of the entry block to load */
)
{
	FRESULT res = FR_DISK_ERR;
	UINT i, sz_ent;
	BYTE *dirb = dp->obj.fs->dirbuf;	/* Pointer to the entry 85+C0+C1s */


	/* Load file-directory entry */
	res = move_window(dp->obj.fs, dp->sect);
	if (res != FR_OK) {
		return res;
	}
	if (dp->dir[XDIR_Type] != ET_FILEDIR) {
		return FR_INT_ERR;        /* Invalid order? */
	}
	mem_cpy(dirb + 0 * SZDIRE, dp->dir, SZDIRE);
	sz_ent = ((UINT)dirb[XDIR_NumSec] + 1) * SZDIRE;	/* Size of this entry block */
	if (sz_ent < 3 * SZDIRE || sz_ent > 19 * SZDIRE) {	/* Invalid block size? */
		return FR_INT_ERR;
	}

	/* Load stream extension entry */
	res = dir_next(dp, 0);
	if (res == FR_NO_FILE) {
		res = FR_INT_ERR;        /* It cannot be */
	}
	if (res != FR_OK) {
		return res;
	}
	res = move_window(dp->obj.fs, dp->sect);
	if (res != FR_OK) {
		return res;
	}
	if (dp->dir[XDIR_Type] != ET_STREAM) {
		return FR_INT_ERR;        /* Invalid order? */
	}
	mem_cpy(dirb + 1 * SZDIRE, dp->dir, SZDIRE);
	if (MAXDIRB(dirb[XDIR_NumName]) > sz_ent) {
		return FR_INT_ERR;	/* Invalid block size for the name? */
	}

	/* Load file name entries */
	i = 2 * SZDIRE;	/* Name offset to load */
	do {
		res = dir_next(dp, 0);
		if (res == FR_NO_FILE) {
			res = FR_INT_ERR;        /* It cannot be */
		}
		if (res != FR_OK) {
			return res;
		}
		res = move_window(dp->obj.fs, dp->sect);
		if (res != FR_OK) {
			return res;
		}
		if (dp->dir[XDIR_Type] != ET_FILENAME) {
			return FR_INT_ERR;        /* Invalid order */
		}
		if (i < MAXDIRB(FF_MAX_LFN)) {
			mem_cpy(dirb + i, dp->dir, SZDIRE);	/* Load name entries only if the object is accessible */
		}
	}
	while ((i += SZDIRE) < sz_ent);

	/* Sanity check (do it for only accessible object) */
	if (i <= MAXDIRB(FF_MAX_LFN)) {
		if (xdir_sum(dirb) != ld_16(dirb + XDIR_SetSum)) {
			return FR_INT_ERR;
		}
	}

	return FR_OK;
}


/*------------------------------------------------------------------*/
/* exFAT: Initialize object allocation info with loaded entry block */
/*------------------------------------------------------------------*/

static void init_alloc_info (
	FFOBJID *dobj,	/* Object allocation information to be initialized */
	DIR* sdir		/* Additional source about containing directory */
)
{
	FATFS *fs = dobj->fs;


	if (sdir) {	/* Initialize the containing directory. This block needs to precede the following. */
		dobj->c_scl = sdir->obj.sclust;
		dobj->c_size = ((DWORD)sdir->obj.objsize & 0xFFFFFF00) | sdir->obj.stat;
		dobj->c_ofs = sdir->blk_ofs;
	}
	dobj->sclust = ld_32(fs->dirbuf + XDIR_FstClus);	/* Start cluster */
	dobj->objsize = ld_64(fs->dirbuf + XDIR_FileSize);	/* Size */
	dobj->stat = fs->dirbuf[XDIR_GenFlags] & 2;			/* Allocation status */
	dobj->n_frag = 0;									/* No last fragment info */
}



#if !FF_FS_READONLY || FF_FS_RPATH
/*------------------------------------------------*/
/* exFAT: Load the object's directory entry block */
/*------------------------------------------------*/

static FRESULT load_obj_xdir (
	DIR *dp,			/* Blank directory object to be used to access containing directory */
	const FFOBJID *obj	/* Object with its containing directory information */
)
{
	FRESULT res = FR_DISK_ERR;

	/* Open object containing directory */
	dp->obj.fs = obj->fs;
	dp->obj.sclust = obj->c_scl;
	dp->obj.stat = (BYTE)obj->c_size;
	dp->obj.objsize = obj->c_size & 0xFFFFFF00;
	dp->obj.n_frag = 0;
	dp->blk_ofs = obj->c_ofs;

	res = dir_sdi(dp, dp->blk_ofs);	/* Goto object's entry block */
	if (res == FR_OK) {
		res = load_xdir(dp);		/* Load the object's entry block */
	}
	return res;
}
#endif


#if !FF_FS_READONLY
/*----------------------------------------*/
/* exFAT: Store the directory entry block */
/*----------------------------------------*/

static FRESULT store_xdir (
	DIR *dp				/* Pointer to the directory object */
)
{
	FRESULT res = FR_DISK_ERR;
	UINT nent;
	BYTE *dirb = dp->obj.fs->dirbuf;	/* Pointer to the directory entry-set block 85+C0+C1s */

	st_16(dirb + XDIR_SetSum, xdir_sum(dirb));	/* Create check sum */

	/* Store the entry set to the directory */
	nent = dirb[XDIR_NumSec] + 1;	/* Number of entries */
	res = dir_sdi(dp, dp->blk_ofs);	/* Top of the entry set */
	while (res == FR_OK) {
		/* Set an entry to the directory */
		res = move_window(dp->obj.fs, dp->sect);
		if (res != FR_OK) {
			break;
		}
		mem_cpy(dp->dir, dirb, SZDIRE);
		dp->obj.fs->wflag = 1;

		if (--nent == 0) {
			break;	/* All done? */
		}
		dirb += SZDIRE;
		res = dir_next(dp, 0);	/* Next entry */
	}
	return (res == FR_OK || res == FR_DISK_ERR) ? res : FR_INT_ERR;
}



/*-------------------------------------------*/
/* exFAT: Create a new directory entry block */
/*-------------------------------------------*/

static void create_xdir (
	BYTE *dirb,			/* Pointer to the directory entry block buffer */
	const WCHAR *lfn	/* Pointer to the object name */
)
{
	UINT i;
	BYTE n_c1, nlen;
	WCHAR chr;


	/* Create file-directory and stream-extension entry (1st and 2nd entry) */
	memset(dirb, 0, 2 * SZDIRE);
	dirb[0 * SZDIRE + XDIR_Type] = ET_FILEDIR;
	dirb[1 * SZDIRE + XDIR_Type] = ET_STREAM;

	/* Create file name entries (3rd entry and follows) */
	i = SZDIRE * 2;	/* Top of file name entries */
	nlen = n_c1 = 0;
	chr = 1;
	do {
		dirb[i++] = ET_FILENAME;
		dirb[i++] = 0;
		do {	/* Fill name field */
			if (chr != 0 && (chr = lfn[nlen]) != 0) {
				nlen++;        /* Get a character if exist */
			}
			st_16(dirb + i, chr); 	/* Store it */
			i += 2;
		}
		while (i % SZDIRE != 0);
		n_c1++;
	}
	while (lfn[nlen]);	/* Fill next C1 entry if any char follows */

	dirb[XDIR_NumName] = nlen;		/* Set name length */
	dirb[XDIR_NumSec] = 1 + n_c1;	/* Set secondary count (C0 + C1s) */
	st_16(dirb + XDIR_NameHash, xname_sum(lfn));	/* Set name hash */
}

#endif	/* !FF_FS_READONLY */
#endif	/* FF_FS_EXFAT */



#if FF_FS_MINIMIZE <= 1 || FF_FS_RPATH >= 2 || FF_USE_LABEL || FF_FS_EXFAT
/*-----------------------------------------------------------------------*/
/* Read an object from the directory                                     */
/*-----------------------------------------------------------------------*/

#define DIR_READ_FILE(dp) dir_read(dp, 0)
#define DIR_READ_LABEL(dp) dir_read(dp, 1)

static FRESULT dir_read (
	DIR *dp,		/* Pointer to the directory object */
	int vol			/* Filtered by 0:file/directory or 1:volume label */
)
{
	FRESULT res = FR_NO_FILE;
	FATFS *fs = dp->obj.fs;
	BYTE attr, et;
#if FF_USE_LFN
	BYTE ord = 0xFF, sum = 0xFF;
#endif

	while (dp->sect) {
		res = move_window(fs, dp->sect);
		if (res != FR_OK) {
			break;
		}
		et = dp->dir[DIR_Name];	/* Test for the entry type */
		if (et == 0) {
			res = FR_NO_FILE;
			break; /* Reached to end of the directory */
		}
#if FF_FS_EXFAT
		if (fs->fs_type == FS_EXFAT) {	/* On the exFAT volume */
			if (FF_USE_LABEL && vol) {
				if (et == ET_VLABEL) {
					break;        /* Volume label entry? */
				}
			}
			else {
				if (et == ET_FILEDIR) {		/* Start of the file entry block? */
					dp->blk_ofs = dp->dptr;	/* Get location of the block */
					res = load_xdir(dp);	/* Load the entry block */
					if (res == FR_OK) {
						dp->obj.attr = fs->dirbuf[XDIR_Attr] & AM_MASK;	/* Get attribute */
					}
					break;
				}
			}
		}
		else
#endif
		{	/* On the FAT/FAT32 volume */
			dp->obj.attr = attr = dp->dir[DIR_Attr] & AM_MASK;	/* Get attribute */
#if FF_USE_LFN		/* LFN configuration */
			if (et == DDEM || et == '.' || (int)((attr & ~AM_ARC) == AM_VOL) != vol) {	/* An entry without valid data */
				ord = 0xFF;
			}
			else {
				if (attr == AM_LFN) {	/* An LFN entry is found */
					if (et & LLEF) {		/* Is it start of an LFN sequence? */
						sum = dp->dir[LDIR_Chksum];
						et &= (BYTE)~LLEF;
						ord = et;
						dp->blk_ofs = dp->dptr;
					}
					/* Check LFN validity and capture it */
					ord = (et == ord && sum == dp->dir[LDIR_Chksum] && pick_lfn(fs->lfnbuf, dp->dir)) ? ord - 1 : 0xFF;
				}
				else {				/* An SFN entry is found */
					if (ord != 0 || sum != sum_sfn(dp->dir)) {	/* Is there a valid LFN? */
						dp->blk_ofs = 0xFFFFFFFF;	/* It has no LFN. */
					}
					break;
				}
			}
#else		/* Non LFN configuration */
			if (et != DDEM && et != '.' && attr != AM_LFN && (int)((attr & ~AM_ARC) == AM_VOL) == vol) {	/* Is it a valid entry? */
				break;
			}
#endif
		}
		res = dir_next(dp, 0);		/* Next entry */
		if (res != FR_OK) {
			break;
		}
	}

	if (res != FR_OK) {
		dp->sect = 0;        /* Terminate the read operation on error or EOT */
	}
	return res;
}

#endif	/* FF_FS_MINIMIZE <= 1 || FF_USE_LABEL || FF_FS_RPATH >= 2 */



/*-----------------------------------------------------------------------*/
/* Directory handling - Find an object in the directory                  */
/*-----------------------------------------------------------------------*/

static FRESULT dir_find (	/* FR_OK(0):succeeded, !=0:error */
	DIR *dp					/* Pointer to the directory object with the file name */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs = dp->obj.fs;
	BYTE et;
#if FF_USE_LFN
	BYTE attr, ord, sum;
#endif

	res = dir_sdi(dp, 0);			/* Rewind directory object */
	if (res != FR_OK) {
		return res;
	}
#if FF_FS_EXFAT
	if (fs->fs_type == FS_EXFAT) {	/* On the exFAT volume */
		BYTE nc;
		UINT di, ni;
		WORD hash = xname_sum(fs->lfnbuf);		/* Hash value of the name to find */

		while ((res = DIR_READ_FILE(dp)) == FR_OK) {	/* Read an item */
#if FF_MAX_LFN < 255
			if (fs->dirbuf[XDIR_NumName] > FF_MAX_LFN) {
				continue;        /* Skip comparison if inaccessible object name */
			}
#endif
			if (ld_16(fs->dirbuf + XDIR_NameHash) != hash) {
				continue;        /* Skip comparison if hash mismatched */
			}
			for (nc = fs->dirbuf[XDIR_NumName], di = SZDIRE * 2, ni = 0; nc; nc--, di += 2, ni++) {	/* Compare the name */
				if ((di % SZDIRE) == 0) {
					di += 2;
				}
				if (ff_wtoupper(ld_16(fs->dirbuf + di)) != ff_wtoupper(fs->lfnbuf[ni])) {
					break;
				}
			}
			if (nc == 0 && !fs->lfnbuf[ni]) {
				break;        /* Name matched? */
			}
		}
		return res;
	}
#endif
	/* On the FAT/FAT32 volume */
#if FF_USE_LFN
	ord = sum = 0xFF;
	dp->blk_ofs = 0xFFFFFFFF;	/* Reset LFN sequence */
#endif
	do {
		res = move_window(fs, dp->sect);
		if (res != FR_OK) {
			break;
		}
		et = dp->dir[DIR_Name];
		if (et == 0) {
			res = FR_NO_FILE;        /* Reached end of directory table */
			break;
		}
#if FF_USE_LFN		/* LFN configuration */
		dp->obj.attr = attr = dp->dir[DIR_Attr] & AM_MASK;
		if (et == DDEM || ((attr & AM_VOL) && attr != AM_LFN)) {	/* An entry without valid data */
			ord = 0xFF;
			dp->blk_ofs = 0xFFFFFFFF;	/* Reset LFN sequence */
		}
		else {
			if (attr == AM_LFN) {			/* Is it an LFN entry? */
				if (!(dp->fn[NSFLAG] & NS_NOLFN)) {
					if (et & LLEF) {		/* Is it start of an entry set? */
						et &= (BYTE)~LLEF;
						ord = et;	/* Number of LFN entries */
						dp->blk_ofs = dp->dptr;	/* Start offset of LFN */
						sum = dp->dir[LDIR_Chksum];	/* Sum of the SFN */
					}
					/* Check validity of the LFN entry and compare it with given name */
					ord = (et == ord && sum == dp->dir[LDIR_Chksum] && cmp_lfn(fs->lfnbuf, dp->dir)) ? ord - 1 : 0xFF;
				}
			}
			else {					/* SFN entry */
				if (ord == 0 && sum == sum_sfn(dp->dir)) {
					break;        /* LFN matched? */
				}
				if (!(dp->fn[NSFLAG] & NS_LOSS) && !memcmp(dp->dir, dp->fn, 11)) {
					break;        /* SFN matched? */
				}
				ord = 0xFF;
				dp->blk_ofs = 0xFFFFFFFF;	/* Not matched, reset LFN sequence */
			}
		}
#else		/* Non LFN configuration */
		dp->obj.attr = dp->dir[DIR_Attr] & AM_MASK;
		if (!(dp->dir[DIR_Attr] & AM_VOL) && !memcmp(dp->dir, dp->fn, 11)) {
			break;        /* Is it a valid entry? */
		}
#endif
		res = dir_next(dp, 0);	/* Next entry */
	}
	while (res == FR_OK);

	return res;
}




#if !FF_FS_READONLY
/*-----------------------------------------------------------------------*/
/* Register an object to the directory                                   */
/*-----------------------------------------------------------------------*/

static FRESULT
dir_register (	/* FR_OK:succeeded, FR_DENIED:no free entry or too many SFN collision, FR_DISK_ERR:disk error */
	DIR *dp						/* Target directory with object name to be created */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs = dp->obj.fs;
#if FF_USE_LFN		/* LFN configuration */
	UINT n, len, n_ent;
	BYTE sn[12];


	if (dp->fn[NSFLAG] & (NS_DOT | NS_NONAME)) {
		return FR_INVALID_NAME;        /* Check name validity */
	}
	for (len = 0; fs->lfnbuf[len]; len++) ;	/* Get lfn length */

#if FF_FS_EXFAT
	if (fs->fs_type == FS_EXFAT) {	/* On the exFAT volume */
		n_ent = (len + 14) / 15 + 2;	/* Number of entries to allocate (85+C0+C1s) */
		res = dir_alloc(dp, n_ent);		/* Allocate directory entries */
		if (res != FR_OK) {
			return res;
		}
		dp->blk_ofs = dp->dptr - SZDIRE * (n_ent - 1);	/* Set the allocated entry block offset */

		if (dp->obj.stat & 4) {			/* Has the directory been stretched by new allocation? */
			dp->obj.stat &= ~4;
			res = fill_first_frag(&dp->obj);	/* Fill the first fragment on the FAT if needed */
			if (res != FR_OK) {
				return res;
			}
			res = fill_last_frag(&dp->obj, dp->clust, 0xFFFFFFFF);	/* Fill the last fragment on the FAT if needed */
			if (res != FR_OK) {
				return res;
			}
			if (dp->obj.sclust != 0) {		/* Is it a sub-directory? */
				DIR dj;

				res = load_obj_xdir(&dj, &dp->obj);	/* Load the object status */
				if (res != FR_OK) {
					return res;
				}
				dp->obj.objsize += (DWORD)fs->csize * SS(fs);		/* Increase the directory size by cluster size */
				st_64(fs->dirbuf + XDIR_FileSize, dp->obj.objsize);
				st_64(fs->dirbuf + XDIR_ValidFileSize, dp->obj.objsize);
				fs->dirbuf[XDIR_GenFlags] = dp->obj.stat | 1;		/* Update the allocation status */
				res = store_xdir(&dj);				/* Store the object status */
				if (res != FR_OK) {
					return res;
				}
#if FF_FS_RPATH	/* Refrect changes to the current dir chain if the stretched dir is in it */
				for (n = 1; n <= fs->xcwds.depth && dp->obj.sclust != fs->xcwds.tbl[n].d_scl; n++) ;	/* Check if the dir is in the current dir path */
				if (n <= fs->xcwds.depth) {		/* If exist, update it */
					fs->xcwds.tbl[n].d_size = (DWORD)dp->obj.objsize | dp->obj.stat;
				}
#endif
			}
		}

		create_xdir(fs->dirbuf, fs->lfnbuf);	/* Create on-memory directory block to be written later */
		return FR_OK;
	}
#endif
	/* On the FAT/FAT32 volume */
	mem_cpy(sn, dp->fn, 12);
	if (sn[NSFLAG] & NS_LOSS) {			/* When LFN is out of 8.3 format, generate a numbered name */
		dp->fn[NSFLAG] = NS_NOLFN;		/* Find only SFN */
		for (n = 1; n < 100; n++) {
			gen_numname(dp->fn, sn, fs->lfnbuf, (WORD)n);	/* Generate a numbered name */
			res = dir_find(dp);				/* Check if the name collides with existing SFN */
			if (res != FR_OK) {
				break;
			}
		}
		if (n == 100) {
			return FR_DENIED;        /* Abort if too many collisions */
		}
		if (res != FR_NO_FILE) {
			return res;        /* Abort if the result is other than 'not collided' */
		}
		dp->fn[NSFLAG] = sn[NSFLAG];
	}

	/* Create an SFN with/without LFNs. */
	n_ent = (sn[NSFLAG] & NS_LFN) ? (len + 12) / 13 + 1 : 1;	/* Number of entries to allocate */
	res = dir_alloc(dp, n_ent);		/* Allocate entries */
	if (res == FR_OK && --n_ent) {	/* Set LFN entry if needed */
		res = dir_sdi(dp, dp->dptr - n_ent * SZDIRE);
		if (res == FR_OK) {
			BYTE sum = sum_sfn(dp->fn);	/* Checksum value of the SFN tied to the LFN */

			do {					/* Store LFN entries in bottom first */
				res = move_window(fs, dp->sect);
				if (res != FR_OK) {
					break;
				}
				put_lfn(fs->lfnbuf, dp->dir, (BYTE)n_ent, sum);
				fs->wflag = 1;
				res = dir_next(dp, 0);	/* Next entry */
			}
			while (res == FR_OK && --n_ent);
		}
	}

#else	/* Non LFN configuration */
	res = dir_alloc(dp, 1);		/* Allocate an entry for SFN */

#endif

	/* Set SFN entry */
	if (res == FR_OK) {
		res = move_window(fs, dp->sect);
		if (res == FR_OK) {
			memset(dp->dir, 0, SZDIRE);	/* Clean the entry */
			mem_cpy(dp->dir + DIR_Name, dp->fn, 11);	/* Put SFN */
#if FF_USE_LFN
			dp->dir[DIR_NTres] = dp->fn[NSFLAG] & (NS_BODY | NS_EXT);	/* Put low-case flags */
#endif
			fs->wflag = 1;
		}
	}

	return res;
}

#endif /* !FF_FS_READONLY */



#if !FF_FS_READONLY && FF_FS_MINIMIZE == 0
/*-----------------------------------------------------------------------*/
/* Remove an object from the directory                                   */
/*-----------------------------------------------------------------------*/

static FRESULT dir_remove (	/* FR_OK:Succeeded, FR_DISK_ERR:A disk error */
	DIR *dp					/* Directory object pointing the entry to be removed */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs = dp->obj.fs;
#if FF_USE_LFN		/* LFN configuration */
	DWORD last = dp->dptr;

	res = (dp->blk_ofs == 0xFFFFFFFF) ? FR_OK : dir_sdi(dp, dp->blk_ofs);	/* Goto top of the entry block if LFN is exist */
	if (res == FR_OK) {
		do {
			res = move_window(fs, dp->sect);
			if (res != FR_OK) {
				break;
			}
			if (FF_FS_EXFAT && fs->fs_type == FS_EXFAT) {	/* On the exFAT volume */
				dp->dir[XDIR_Type] &= 0x7F;	/* Clear the entry InUse flag. */
			}
			else {										/* On the FAT/FAT32 volume */
				dp->dir[DIR_Name] = DDEM;	/* Mark the entry 'deleted'. */
			}
			fs->wflag = 1;
			if (dp->dptr >= last) {
				break;        /* If reached last entry then all entries of the object has been deleted. */
			}
			res = dir_next(dp, 0);	/* Next entry */
		}
		while (res == FR_OK);
		if (res == FR_NO_FILE) {
			res = FR_INT_ERR;
		}
	}
#else			/* Non LFN configuration */

	res = move_window(fs, dp->sect);
	if (res == FR_OK) {
		dp->dir[DIR_Name] = DDEM;	/* Mark the entry 'deleted'.*/
		fs->wflag = 1;
	}
#endif

	return res;
}

#endif /* !FF_FS_READONLY && FF_FS_MINIMIZE == 0 */



#if FF_FS_MINIMIZE <= 1 || FF_FS_RPATH >= 2
/*-----------------------------------------------------------------------*/
/* Get file information from directory entry                             */
/*-----------------------------------------------------------------------*/

static void get_fileinfo (
	DIR *dp,			/* Pointer to the directory object */
	FILINFO *fno		/* Pointer to the file information to be filled */
)
{
	UINT si, di;
#if FF_USE_LFN
	WCHAR wc, hs;
	FATFS *fs = dp->obj.fs;
	UINT nw;
#else
	TCHAR c;
#endif


	fno->fname[0] = 0;
	if (dp->sect == 0) {
		return;        /* Exit if read pointer has reached end of directory */
	}

#if FF_USE_LFN		/* LFN configuration */
#if FF_FS_EXFAT
	if (fs->fs_type == FS_EXFAT) {	/* exFAT volume */
		UINT nc = 0;

		si = SZDIRE * 2;
		di = 0;	/* 1st C1 entry in the entry block */
		hs = 0;
		while (nc < fs->dirbuf[XDIR_NumName]) {
			if (si >= MAXDIRB(FF_MAX_LFN)) {	/* Truncated directory block? */
				di = 0;
				break;
			}
			if ((si % SZDIRE) == 0) {
				si += 2;        /* Skip entry type field */
			}
			wc = ld_16(fs->dirbuf + si);
			si += 2;
			nc++;	/* Get a character */
			if (hs == 0 && IsSurrogate(wc)) {	/* Is it a surrogate? */
				hs = wc;
				continue;				/* Get low surrogate */
			}
			nw = put_utf((DWORD)hs << 16 | wc, &fno->fname[di], FF_LFN_BUF - di);	/* Store it in API encoding */
			if (nw == 0) {						/* Buffer overflow or wrong char? */
				di = 0;
				break;
			}
			di += nw;
			hs = 0;
		}
		if (hs != 0) {
			di = 0;        /* Broken surrogate pair? */
		}
		if (di == 0) {
			fno->fname[di++] = '\?';        /* Inaccessible object name? */
		}
		fno->fname[di] = 0;						/* Terminate the name */
		fno->altname[0] = 0;					/* exFAT does not support SFN */

		fno->fattrib = fs->dirbuf[XDIR_Attr] & AM_MASKX;	/* Attribute */
		fno->fsize = (fno->fattrib & AM_DIR) ? 0 : ld_64(fs->dirbuf + XDIR_FileSize);	/* Size */
		fno->ftime = ld_16(fs->dirbuf + XDIR_ModTime + 0);	/* Last modified time */
		fno->fdate = ld_16(fs->dirbuf + XDIR_ModTime + 2);	/* Last modified date */
#if FF_FS_CRTIME
		fno->crtime = ld_16(fs->dirbuf + XDIR_CrtTime + 0);	/* Created time */
		fno->crdate = ld_16(fs->dirbuf + XDIR_CrtTime + 2);	/* Created date */
#endif
		return;
	}
	else
#endif
	{	/* FAT/FAT32 volume */
		if (dp->blk_ofs != 0xFFFFFFFF) {	/* Get LFN if available */
			si = di = 0;
			hs = 0;
			while (fs->lfnbuf[si] != 0) {
				wc = fs->lfnbuf[si++];		/* Get an LFN character (UTF-16) */
				if (hs == 0 && IsSurrogate(wc)) {	/* Is it a surrogate? */
					hs = wc;
					continue;		/* Get low surrogate */
				}
				nw = put_utf((DWORD)hs << 16 | wc, &fno->fname[di], FF_LFN_BUF - di);	/* Store it in API encoding */
				if (nw == 0) {				/* Buffer overflow or wrong char? */
					di = 0;
					break;
				}
				di += nw;
				hs = 0;
			}
			if (hs != 0) {
				di = 0;        /* Broken surrogate pair? */
			}
			fno->fname[di] = 0;		/* Terminate the LFN (null string means LFN is invalid) */
		}
	}

	si = di = 0;
	while (si < 11) {		/* Get SFN from SFN entry */
		wc = dp->dir[si++];			/* Get a char */
		if (wc == ' ') {
			continue;        /* Skip padding spaces */
		}
		if (wc == RDDEM) {
			wc = DDEM;        /* Restore replaced DDEM character */
		}
		if (si == 9 && di < FF_SFN_BUF) {
			fno->altname[di++] = '.';        /* Insert a . if extension is exist */
		}
#if FF_LFN_UNICODE >= 1	/* Unicode output */
		if (dbc_1st((BYTE)wc) && si != 8 && si != 11 && dbc_2nd(dp->dir[si])) {	/* Make a DBC if needed */
			wc = wc << 8 | dp->dir[si++];
		}
		wc = ff_oem2uni(wc, CODEPAGE);		/* ANSI/OEM -> Unicode */
		if (wc == 0) {				/* Wrong char in the current code page? */
			di = 0;
			break;
		}
		nw = put_utf(wc, &fno->altname[di], FF_SFN_BUF - di);	/* Store it in API encoding */
		if (nw == 0) {				/* Buffer overflow? */
			di = 0;
			break;
		}
		di += nw;
#else					/* ANSI/OEM output */
		fno->altname[di++] = (TCHAR)wc;	/* Store it without any conversion */
#endif
	}
	fno->altname[di] = 0;	/* Terminate the SFN  (null string means SFN is invalid) */

	if (!fno->fname[0]) {	/* If LFN is invalid, altname[] needs to be copied to fname[] */
		if (di == 0) {		/* If LFN and SFN both are invalid, */
			fno->fname[di++] = '\?';	/* This object is inaccessible due to wrong buffer or locale settings */
		}
		else {
			BYTE lcflg = NS_BODY;

			for (si = di = 0; fno->altname[si]; si++, di++) {	/* Copy altname[] to fname[] with case information */
				wc = (WCHAR)fno->altname[si];
				if (wc == '.') {
					lcflg = NS_EXT;
				}
				if (IsUpper(wc) && (dp->dir[DIR_NTres] & lcflg)) {
					wc += 0x20;
				}
				fno->fname[di] = (TCHAR)wc;
			}
		}
		fno->fname[di] = 0;	/* Terminate the LFN */
		if (!dp->dir[DIR_NTres]) {
			fno->altname[0] = 0;        /* Altname is not needed if neither LFN nor case info is exist. */
		}
	}

#else	/* Non-LFN configuration */
	si = di = 0;
	while (si < 11) {		/* Copy name body and extension */
		c = (TCHAR)dp->dir[si++];
		if (c == ' ') {
			continue;        /* Skip padding spaces */
		}
		if (c == RDDEM) {
			c = DDEM;        /* Restore replaced DDEM character */
		}
		if (si == 9) {
			fno->fname[di++] = '.';        /* Insert a . if extension is exist */
		}
		fno->fname[di++] = c;
	}
	fno->fname[di] = 0;		/* Terminate the SFN */
#endif

	fno->fattrib = dp->dir[DIR_Attr] & AM_MASK;		/* Attribute */
	fno->fsize = ld_32(dp->dir + DIR_FileSize);		/* Size */
	fno->ftime = ld_16(dp->dir + DIR_ModTime + 0);	/* Last modified time */
	fno->fdate = ld_16(dp->dir + DIR_ModTime + 2);	/* Last Modified date */
#if FF_FS_CRTIME
	fno->crtime = ld_16(dp->dir + DIR_CrtTime + 0);	/* Created time */
	fno->crdate = ld_16(dp->dir + DIR_CrtTime + 2);	/* Created date */
#endif
}

#endif /* FF_FS_MINIMIZE <= 1 || FF_FS_RPATH >= 2 */



#if FF_USE_FIND && FF_FS_MINIMIZE <= 1
/*-----------------------------------------------------------------------*/
/* Pattern matching                                                      */
/*-----------------------------------------------------------------------*/

#define FIND_RECURS	4	/* Maximum number of wildcard terms in the pattern to limit recursion */


static DWORD get_achar (	/* Get a character and advance ptr */
	const TCHAR **ptr		/* Pointer to pointer to the ANSI/OEM or Unicode string */
)
{
	DWORD chr;


#if FF_USE_LFN && FF_LFN_UNICODE >= 1	/* Unicode input */
	chr = tchar2uni(ptr);
	if (chr == 0xFFFFFFFF) {
		chr = 0;        /* Wrong UTF encoding is recognized as end of the string */
	}
	chr = ff_wtoupper(chr);

#else									/* ANSI/OEM input */
	chr = (BYTE) * (*ptr)++;				/* Get a byte */
	if (IsLower(chr)) {
		chr -= 0x20;        /* To upper ASCII char */
	}
#if FF_CODE_PAGE == 0
	if (ExCvt && chr >= 0x80) {
		chr = ExCvt[chr - 0x80];        /* To upper (SBCS extended char) */
	}
#elif FF_CODE_PAGE < 900
	if (chr >= 0x80) {
		chr = ExCvt[chr - 0x80];        /* To upper (SBCS extended char) */
	}
#endif
#if FF_CODE_PAGE == 0 || FF_CODE_PAGE >= 900
	if (dbc_1st((BYTE)chr)) {	/* Get DBC 2nd byte if needed */
		chr = dbc_2nd((BYTE) **ptr) ? chr << 8 | (BYTE) * (*ptr)++ : 0;
	}
#endif

#endif
	return chr;
}


static int pattern_match (	/* 0:mismatched, 1:matched */
	const TCHAR *pat,	/* Matching pattern */
	const TCHAR *nam,	/* String to be tested */
	UINT skip,			/* Number of pre-skip chars (number of ?s, b8:infinite (* specified)) */
	UINT recur			/* Recursion count */
)
{
	const TCHAR *pptr;
	const TCHAR *nptr;
	DWORD pchr, nchr;
	UINT sk;


	while ((skip & 0xFF) != 0) {		/* Pre-skip name chars */
		if (!get_achar(&nam)) {
			return 0;        /* Branch mismatched if less name chars */
		}
		skip--;
	}
	if (*pat == 0 && skip) {
		return 1;        /* Matched? (short circuit) */
	}

	do {
		pptr = pat;
		nptr = nam;			/* Top of pattern and name to match */
		for (;;) {
			if (*pptr == '\?' || *pptr == '*') {	/* Wildcard term? */
				if (recur == 0) {
					return 0;        /* Too many wildcard terms? */
				}
				sk = 0;
				do {	/* Analyze the wildcard term */
					if (*pptr++ == '\?') {
						sk++;
					}
					else {
						sk |= 0x100;
					}
				}
				while (*pptr == '\?' || *pptr == '*');
				if (pattern_match(pptr, nptr, sk, recur - 1)) {
					return 1;        /* Test new branch (recursive call) */
				}
				nchr = *nptr;
				break;	/* Branch mismatched */
			}
			pchr = get_achar(&pptr);	/* Get a pattern char */
			nchr = get_achar(&nptr);	/* Get a name char */
			if (pchr != nchr) {
				break;        /* Branch mismatched? */
			}
			if (pchr == 0) {
				return 1;        /* Branch matched? (matched at end of both strings) */
			}
		}
		get_achar(&nam);			/* nam++ */
	}
	while (skip && nchr);		/* Retry until end of name if infinite search is specified */

	return 0;
}

#endif /* FF_USE_FIND && FF_FS_MINIMIZE <= 1 */



/*-----------------------------------------------------------------------*/
/* Pick a top segment and create the object name in directory form       */
/*-----------------------------------------------------------------------*/

static FRESULT create_name (	/* FR_OK: successful, FR_INVALID_NAME: could not create */
	DIR *dp,					/* Pointer to the directory object */
	const TCHAR **path			/* Pointer to pointer to the segment in the path string */
)
{
#if FF_USE_LFN		/* LFN configuration */
	BYTE b, cf;
	WCHAR wc;
	WCHAR *lfn;
	const TCHAR *p;
	DWORD uc;
	UINT i, ni, si, di;


	/* Create an LFN into LFN working buffer */
	p = *path;
	lfn = dp->obj.fs->lfnbuf;
	di = 0;
	for (;;) {
		uc = tchar2uni(&p);			/* Get a character */
		if (uc == 0xFFFFFFFF) {
			return FR_INVALID_NAME;        /* Invalid code or UTF decode error */
		}
		if (uc >= 0x10000) {
			lfn[di++] = (WCHAR)(uc >> 16);        /* Store high surrogate if needed */
		}
		wc = (WCHAR)uc;
		if (wc < ' ' || IsSeparator(wc)) {
			break;        /* Break if end of the path or a separator is found */
		}
		if (wc < 0x80 && strchr("*:<>|\"\?\x7F", (int)wc)) {
			return FR_INVALID_NAME;        /* Reject illegal characters for LFN */
		}
		if (di >= FF_MAX_LFN) {
			return FR_INVALID_NAME;        /* Reject too long name */
		}
		lfn[di++] = wc;				/* Store the Unicode character */
	}
	if (wc < ' ') {				/* Stopped at end of the path? */
		cf = NS_LAST;			/* Last segment */
	}
	else {					/* Stopped at a separator */
		while (IsSeparator(*p)) {
			p++;        /* Skip duplicated separators if exist */
		}
		cf = 0;					/* Next segment may follow */
		if (IsTerminator(*p)) {
			cf = NS_LAST;        /* Ignore terminating separator */
		}
	}
	*path = p;					/* Return pointer to the next segment */

#if FF_FS_RPATH
	if ((di == 1 && lfn[di - 1] == '.') ||
	    (di == 2 && lfn[di - 1] == '.' && lfn[di - 2] == '.')) {	/* Is this segment a dot name? */
		lfn[di] = 0;
		for (i = 0; i < 11; i++) {	/* Create dot name for SFN entry */
			dp->fn[i] = (i < di) ? '.' : ' ';
		}
		dp->fn[i] = cf | NS_DOT;	/* This is a dot entry */
		return FR_OK;
	}
#endif
	while (di) {					/* Snip off trailing spaces and dots if exist */
		wc = lfn[di - 1];
		if (wc != ' ' && wc != '.') {
			break;
		}
		di--;
	}
	lfn[di] = 0;							/* LFN is created into the working buffer */
	if (di == 0) {
		return FR_INVALID_NAME;        /* Reject null name */
	}

	/* Create SFN in directory form */
	for (si = 0; lfn[si] == ' '; si++) ;	/* Remove leading spaces */
	if (si > 0 || lfn[si] == '.') {
		cf |= NS_LOSS | NS_LFN;        /* Is there any leading space or dot? */
	}
	while (di > 0 && lfn[di - 1] != '.') {
		di--;        /* Find last dot (di<=si: no extension) */
	}

	memset(dp->fn, ' ', 11);
	i = b = 0;
	ni = 8;
	for (;;) {
		wc = lfn[si++];					/* Get an LFN character */
		if (wc == 0) {
			break;        /* Break on end of the LFN */
		}
		if (wc == ' ' || (wc == '.' && si != di)) {	/* Remove embedded spaces and dots */
			cf |= NS_LOSS | NS_LFN;
			continue;
		}

		if (i >= ni || si == di) {		/* End of field? */
			if (ni == 11) {				/* Name extension overflow? */
				cf |= NS_LOSS | NS_LFN;
				break;
			}
			if (si != di) {
				cf |= NS_LOSS | NS_LFN;        /* Name body overflow? */
			}
			if (si > di) {
				break;        /* No name extension? */
			}
			si = di;
			i = 8;
			ni = 11;
			b <<= 2;		/* Enter name extension */
			continue;
		}

		if (wc >= 0x80) {	/* Is this an extended character? */
			cf |= NS_LFN;	/* LFN entry needs to be created */
#if FF_CODE_PAGE == 0
			if (ExCvt) {	/* In SBCS cfg */
				wc = ff_uni2oem(wc, CODEPAGE);			/* Unicode ==> ANSI/OEM code */
				if (wc & 0x80) {
					wc = ExCvt[wc & 0x7F];        /* Convert extended character to upper (SBCS) */
				}
			}
			else {		/* In DBCS cfg */
				wc = ff_uni2oem(ff_wtoupper(wc), CODEPAGE);	/* Unicode ==> Up-convert ==> ANSI/OEM code */
			}
#elif FF_CODE_PAGE < 900	/* In SBCS cfg */
			wc = ff_uni2oem(wc, CODEPAGE);			/* Unicode ==> ANSI/OEM code */
			if (wc & 0x80) {
				wc = ExCvt[wc & 0x7F];        /* Convert extended character to upper (SBCS) */
			}
#else						/* In DBCS cfg */
			wc = ff_uni2oem(ff_wtoupper(wc), CODEPAGE);	/* Unicode ==> Up-convert ==> ANSI/OEM code */
#endif
		}

		if (wc >= 0x100) {				/* Is this a DBC? */
			if (i >= ni - 1) {			/* Field overflow? */
				cf |= NS_LOSS | NS_LFN;
				i = ni;
				continue;		/* Next field */
			}
			dp->fn[i++] = (BYTE)(wc >> 8);	/* Put 1st byte */
		}
		else {						/* SBC */
			if (wc == 0 || strchr("+,;=[]", (int)wc)) {	/* Replace illegal characters for SFN */
				wc = '_';
				cf |= NS_LOSS | NS_LFN;/* Lossy conversion */
			}
			else {
				if (IsUpper(wc)) {		/* ASCII upper case? */
					b |= 2;
				}
				if (IsLower(wc)) {		/* ASCII lower case? */
					b |= 1;
					wc -= 0x20;
				}
			}
		}
		dp->fn[i++] = (BYTE)wc;
	}

	if (dp->fn[0] == DDEM) {
		dp->fn[0] = RDDEM;        /* If the first character collides with DDEM, replace it with RDDEM */
	}

	if (ni == 8) {
		b <<= 2;        /* Shift capital flags if no extension */
	}
	if ((b & 0x0C) == 0x0C || (b & 0x03) == 0x03) {
		cf |= NS_LFN;        /* LFN entry needs to be created if composite capitals */
	}
	if (!(cf & NS_LFN)) {				/* When LFN is in 8.3 format without extended character, NT flags are created */
		if (b & 0x01) {
			cf |= NS_EXT;        /* NT flag (Extension has small capital letters only) */
		}
		if (b & 0x04) {
			cf |= NS_BODY;        /* NT flag (Body has small capital letters only) */
		}
	}

	dp->fn[NSFLAG] = cf;	/* SFN is created into dp->fn[] */

	return FR_OK;


#else	/* FF_USE_LFN : Non-LFN configuration */
	BYTE c, d;
	BYTE *sfn;
	UINT ni, si, i;
	const char *p;

	/* Create file name in directory form */
	p = *path;
	sfn = dp->fn;
	memset(sfn, ' ', 11);
	si = i = 0;
	ni = 8;
#if FF_FS_RPATH
	if (p[si] == '.') { /* Is this a dot entry? */
		for (;;) {	/* Copy one or two dots */
			c = (BYTE)p[si++];
			if (c != '.' || si >= 3) {
				break;
			}
			sfn[i++] = c;
		}
		if (IsSeparator(c)) {
			while (IsSeparator(p[si])) si++;	/* Skip duplicated separators */
			if ((BYTE)p[si] <= ' ') c = 0;		/* Terminate if no segment follows */
		} else if (c > ' ') {			/* Not in dot name */
			return FR_INVALID_NAME;
		}
		*path = p + si;					/* Return pointer to the next segment */
		sfn[NSFLAG] = (c <= ' ') ? NS_LAST | NS_DOT : NS_DOT;	/* Set last segment flag if end of the path */
		return FR_OK;
	}
#endif
	for (;;) {
		c = (BYTE)p[si++];				/* Get a byte */
		if (c <= ' ') {
			break;        /* Break if end of the path name */
		}
		if (IsSeparator(c)) {			/* Break if a separator is found */
			while (IsSeparator(p[si])) {
				si++; 				/* Skip duplicated separators */
			}
			break;
		}
		if (c == '.' || i >= ni) {		/* End of body or field overflow? */
			if (ni == 11 || c != '.') {
				return FR_INVALID_NAME;        /* Field overflow or invalid dot? */
			}
			i = 8;
			ni = 11;				/* Enter file extension field */
			continue;
		}
#if FF_CODE_PAGE == 0
		if (ExCvt && c >= 0x80) {		/* Is SBC extended character? */
			c = ExCvt[c & 0x7F];		/* To upper SBC extended character */
		}
#elif FF_CODE_PAGE < 900
		if (c >= 0x80) {				/* Is SBC extended character? */
			c = ExCvt[c & 0x7F];		/* To upper SBC extended character */
		}
#endif
		if (dbc_1st(c)) {				/* Check if it is a DBC 1st byte */
			d = (BYTE)p[si++];			/* Get 2nd byte */
			if (!dbc_2nd(d) || i >= ni - 1) {
				return FR_INVALID_NAME;        /* Reject invalid DBC */
			}
			sfn[i++] = c;
			sfn[i++] = d;
		}
		else {						/* SBC */
			if (strchr("*+,:;<=>[]|\"\?\x7F", (int)c)) {
				return FR_INVALID_NAME;        /* Reject illegal chrs for SFN */
			}
			if (IsLower(c)) {
				c -= 0x20;        /* To upper */
			}
			sfn[i++] = c;
		}
	}
	*path = &p[si];						/* Return pointer to the next segment */
	if (i == 0) {
		return FR_INVALID_NAME;        /* Reject nul string */
	}

	if (sfn[0] == DDEM) {
		sfn[0] = RDDEM;        /* If the first character collides with DDEM, replace it with RDDEM */
	}
	sfn[NSFLAG] = (c <= ' ' || p[si] <= ' ') ? NS_LAST : 0;	/* Set last segment flag if end of the path */

	return FR_OK;
#endif /* FF_USE_LFN */
}




/*-----------------------------------------------------------------------*/
/* Follow a file path                                                    */
/*-----------------------------------------------------------------------*/

static FRESULT follow_path (	/* FR_OK(0): successful, !=0: error code */
	DIR *dp,					/* Directory object to return last directory and found object */
	const TCHAR *path			/* Full-path string to find a file or directory */
)
{
	FRESULT res = FR_DISK_ERR;
	BYTE ns;
	FATFS *fs = dp->obj.fs;


	/* Determines the start directory (current directory or forced root directory) */
#if FF_FS_RPATH
	if (!IsSeparator(*path) && (FF_STR_VOLUME_ID != 2 || !IsTerminator(*path))) {	/* Without heading separator */
		dp->obj.sclust = fs->cdir;			/* Start at the current directory */
	}
	else
#endif
	{										/* With heading separator */
		while (IsSeparator(*path)) {
			path++;							/* Strip heading separators */
		}
		dp->obj.sclust = 0;					/* Start at the root directory */
	}
#if FF_FS_EXFAT
	dp->obj.n_frag = 0;	/* Invalidate last fragment counter of the object */
#if FF_FS_RPATH
	if (fs->fs_type == FS_EXFAT) {	/* exFAT: Retrieve the start-directory's status */
		if (dp->obj.sclust) {	/* Start directory is a sub-directory */
			/* Load the current directory chain into working buffer and initialize directory object as current dir */
			mem_cpy(&fs->xcwds2, &fs->xcwds, (UINT)(sizeof fs->xcwds2));
			dp->obj.stat = (BYTE)fs->xcwds2.tbl[fs->xcwds2.depth].d_size;
			dp->obj.objsize = fs->xcwds2.tbl[fs->xcwds2.depth].d_size & 0xFFFFFF00;
			dp->obj.c_scl = fs->xcwds2.tbl[fs->xcwds2.depth - 1].d_scl;
			dp->obj.c_size = fs->xcwds2.tbl[fs->xcwds2.depth - 1].d_size & 0xFFFFFF00;
			dp->obj.c_ofs = fs->xcwds2.tbl[fs->xcwds2.depth - 1].nxt_ofs;
		} else {				/* Start directory is the root directory */
			/* Clear the directory path working buffer as root directory */
			memset(&fs->xcwds2, 0, sizeof fs->xcwds2);
		}
	}
#endif
#endif

	if ((UINT)*path < ' ') {				/* Null path name is the origin directory itself */
		dp->fn[NSFLAG] = NS_NONAME;
		res = dir_sdi(dp, 0);

	}
	else {								/* Follow path */
		for (;;) {
			res = create_name(dp, &path);	/* Get a segment name of the path */
			if (res != FR_OK) {
				break;
			}
			ns = dp->fn[NSFLAG];
#if FF_FS_EXFAT && FF_FS_RPATH
			if (fs->fs_type == FS_EXFAT && (ns & NS_DOT)) {	/* Is it a dot name? */
				/* There is no dot entry in exFAT volume, so it needs to follow the parent directory with recorded path */
				if (fs->lfnbuf[1] == '.' && fs->xcwds2.depth) {	/* ".." in the sub-dir? */
					fs->xcwds2.depth--;	/* Get into the parent directory and load the directory info */
					dp->obj.sclust = fs->xcwds2.tbl[fs->xcwds2.depth].d_scl;
					dp->obj.stat = (BYTE)fs->xcwds2.tbl[fs->xcwds2.depth].d_size;
					dp->obj.objsize = fs->xcwds2.tbl[fs->xcwds2.depth].d_size & 0xFFFFFF00;
					if (fs->xcwds2.depth) {		/* Load containing dir info if needed */
						dp->obj.c_scl = fs->xcwds2.tbl[fs->xcwds2.depth - 1].d_scl;
						dp->obj.c_size = fs->xcwds2.tbl[fs->xcwds2.depth - 1].d_size;
						dp->obj.c_ofs = fs->xcwds2.tbl[fs->xcwds2.depth - 1].nxt_ofs;
					}
				}
				dp->obj.attr |= AM_DIR;			/* This is a directory */
				dp->fn[NSFLAG] |= NS_NONAME;	/* but dot names in exFAT volume are not directory entry */
				if (ns & NS_LAST) {
					break;		/* Last segment? */
				}
				continue;		/* Follow next segment */
			}
#endif
			res = dir_find(dp);				/* Find an object with the segment name */
			if (res != FR_OK) {				/* Failed to find the object */
				if (res == FR_NO_FILE) {	/* Object is not found */
					if (FF_FS_RPATH && (ns & NS_DOT)) {	/* If dot entry is not exist, stay there (may be root dir in FAT volume) */
						if (!(ns & NS_LAST)) {
							continue;        /* Continue to follow if not last segment */
						}
						dp->fn[NSFLAG] = NS_NONAME;
						res = FR_OK;
					}
					else {							/* Could not find the object */
						if (!(ns & NS_LAST)) {
							res = FR_NO_PATH;        /* Adjust error code if not last segment */
						}
					}
				}
				break;
			}
#if FF_FS_EXFAT && FF_FS_RPATH
			if (fs->fs_type == FS_EXFAT && (dp->obj.attr & AM_DIR)) {	/* Record the path if it is a sub-directory */
				fs->xcwds2.tbl[fs->xcwds2.depth].nxt_ofs = dp->blk_ofs;
				if (++fs->xcwds2.depth >= sizeof fs->xcwds2.tbl / sizeof fs->xcwds2.tbl[0]) {	/* Is it too deep path? */
					res = FR_NOT_ENOUGH_CORE; break;
				}
				fs->xcwds2.tbl[fs->xcwds2.depth].d_scl = ld_32(fs->dirbuf + XDIR_FstClus);
				fs->xcwds2.tbl[fs->xcwds2.depth].d_size = ld_32(fs->dirbuf + XDIR_FileSize) | (fs->dirbuf[XDIR_GenFlags] & 2);
			}
#endif
			if (ns & NS_LAST) {
				break;        /* If last segment matched, the function completed */
			}
			/* Get into the sub-directory */
			if (!(dp->obj.attr & AM_DIR)) {
				res = FR_NO_PATH;	/* It is not a sub-directory and cannot follow the path */
				break;
			}
#if FF_FS_EXFAT
			if (fs->fs_type == FS_EXFAT) {
				init_alloc_info(&dp->obj, dp);	/* Open next directory */
			}
			else
#endif
			{
				dp->obj.sclust = ld_clust(fs, fs->win + dp->dptr % SS(fs));	/* Open next directory */
			}
		}
	}

	return res;
}




/*-----------------------------------------------------------------------*/
/* Get logical drive number from path name                               */
/*-----------------------------------------------------------------------*/

static int get_ldnumber (	/* Returns logical drive number (-1:invalid drive number or null pointer) */
	const TCHAR **path		/* Pointer to pointer to the path name */
)
{
	const TCHAR *tp;
	const TCHAR *tt;
	TCHAR chr;
#ifdef XPAR_XUFSPSXC_NUM_INSTANCES
	TCHAR vol_id[4];
	int index = 0;
#endif
	int i;
#if FF_STR_VOLUME_ID		/* Find string volume ID */
	const char *vsp;
	char vchr;
#endif

	tt = tp = *path;
	if (!tp) {
		return -1;	/* Invalid path name? */
	}
	do {					/* Find a colon in the path */
		chr = *tt++;
#ifdef XPAR_XUFSPSXC_NUM_INSTANCES
		if (index < 4) {
			vol_id[index] = chr;
		}

		index++;
#endif
	}
	while (!IsTerminator(chr) && chr != ':');

	if (chr == ':') {	/* Is there a DOS/Windows style volume ID? */
		i = FF_VOLUMES;
#ifdef XPAR_XUFSPSXC_NUM_INSTANCES
		if (index <= 4) {
			vol_id[index - 1] = '\0';
			i = atoi(vol_id);
		}
#else
		if (IsDigit(*tp) && tp + 2 == tt) {	/* Is it a numeric volume ID + colon? */
			i = (int) * tp - '0';	/* Get the logical drive number */
		}
#endif
#if FF_STR_VOLUME_ID == 1	/* Arbitrary string volume ID is enabled */
		else {
			i = 0;	/* Find volume ID string in the preconfigured table */
			do {
				vsp = VolumeStr[i];
				tp = *path;	/* Preconfigured string and path name to test */
				do {	/* Compare the volume ID with path name in case-insensitive */
					vchr = *vsp++;
					chr = *tp++;
					if (IsLower(vchr)) {
						vchr -= 0x20;
					}
					if (IsLower(chr)) {
						chr -= 0x20;
					}
				}
				while (vchr && (TCHAR)vchr == chr);
			}
			while ((vchr || tp != tt) && ++i < FF_VOLUMES);	/* Repeat for each id until pattern match */
		}
#endif
		if (i >= FF_VOLUMES) return -1;	/* Not found or invalid volume ID */
		*path = tt;		/* Snip the drive prefix off */
		return i;		/* Return the found drive number */
	}
#if FF_STR_VOLUME_ID == 2		/* Unix style volume ID is enabled */
	if (*tp == '/') {			/* Is there a volume ID? */
		while (*(tp + 1) == '/') {
			tp++;        /* Skip duplicated separator */
		}
		i = 0;
		do {
			vsp = VolumeStr[i]; tt = tp;	/* Preconfigured string and path name to test */
			do {	/* Compare the volume ID with path name in case-insensitive */
				vchr = *vsp++;
				chr = *(++tt);
				if (IsLower(vchr)) {
					vchr -= 0x20;
				}
				if (IsLower(chr)) {
					chr -= 0x20;
				}
			}
			while (vchr && (TCHAR)vchr == chr);
		}
		while ((vchr || (chr != '/' && !IsTerminator(chr))) && ++i < FF_VOLUMES);	/* Repeat for each ID until pattern match */
		if (i >= FF_VOLUMES) return -1;	/* Not found (invalid volume ID) */
		*path = tt;		/* Snip the node name off */
		return i;		/* Return the found drive number */
	}
#endif
	/* No drive prefix */
#if FF_FS_RPATH
	return (int)CurrVol;	/* Default drive is current drive */
#else
	return 0;		/* Default drive is 0 */
#endif
}




/*-----------------------------------------------------------------------*/
/* GPT support functions                                                 */
/*-----------------------------------------------------------------------*/

#if FF_LBA64

/* Calculate CRC32 in byte-by-byte */

static DWORD crc32 (	/* Returns next CRC value */
	DWORD crc,			/* Current CRC value */
	BYTE d				/* A byte to be processed */
)
{
	BYTE b;


	for (b = 1; b; b <<= 1) {
		crc ^= (d & b) ? 1 : 0;
		crc = (crc & 1) ? crc >> 1 ^ 0xEDB88320 : crc >> 1;
	}
	return crc;
}


/* Check validity of GPT header */

static int test_gpt_header (	/* 0:Invalid, 1:Valid */
	const BYTE *gpth			/* Pointer to the GPT header */
)
{
	UINT i;
	DWORD bcc, hlen;


	if (memcmp(gpth + GPTH_Sign, "EFI PART" "\0\0\1", 12)) {
		return 0;        /* Check signature and version (1.0) */
	}
	hlen = ld_32(gpth + GPTH_Size);							/* Check header size */
	if (hlen < 92 || hlen > FF_MIN_SS) {
		return 0;
	}
	for (i = 0, bcc = 0xFFFFFFFF; i < hlen; i++) {			/* Check header BCC */
		bcc = crc32(bcc, i - GPTH_Bcc < 4 ? 0 : gpth[i]);
	}
	if (~bcc != ld_32(gpth + GPTH_Bcc)) {
		return 0;
	}
	if (ld_32(gpth + GPTH_PteSize) != SZ_GPTE) {
		return 0;        /* Table entry size (must be SZ_GPTE bytes) */
	}
	if (ld_32(gpth + GPTH_PtNum) > 128) {
		return 0;        /* Table size (must be 128 entries or less) */
	}

	return 1;
}

#if !FF_FS_READONLY && FF_USE_MKFS

/* Generate random value */
static DWORD make_rand (	/* Returns a seed value for next */
	DWORD seed,		/* Seed value */
	BYTE *buff,		/* Output buffer */
	UINT n			/* Data length */
)
{
	UINT r;


	if (seed == 0) {
		seed = 1;
	}
	do {
		for (r = 0; r < 8; r++) {
			seed = seed & 1 ? seed >> 1 ^ 0xA3000000 : seed >> 1;        /* Shift 8 bits the 32-bit LFSR */
		}
		*buff++ = (BYTE)seed;
	}
	while (--n);
	return seed;
}

#endif
#endif



/*-----------------------------------------------------------------------*/
/* Load a sector and check if it is an FAT VBR                           */
/*-----------------------------------------------------------------------*/

/* Check what the sector is */

static UINT
check_fs (	/* 0:FAT/FAT32 VBR, 1:exFAT VBR, 2:Not FAT and valid BS, 3:Not FAT and invalid BS, 4:Disk error */
	FATFS *fs,			/* Filesystem object */
	LBA_t sect			/* Sector to load and check if it is an FAT-VBR or not */
)
{
	WORD w, sign;
	BYTE b;


	fs->wflag = 0;
	fs->winsect = (LBA_t)0 - 1;		/* Invaidate window */
	if (move_window(fs, sect) != FR_OK) {
		return 4;        /* Load the boot sector */
	}
	sign = ld_16(fs->win + BS_55AA);
#if FF_FS_EXFAT
	if (sign == 0xAA55 && !memcmp(fs->win + BS_JmpBoot, "\xEB\x76\x90" "EXFAT   ", 11)) {
		return 1;        /* It is an exFAT VBR */
	}
#endif
	b = fs->win[BS_JmpBoot];
	if (b == 0xEB || b == 0xE9 || b == 0xE8) {	/* Valid JumpBoot code? (short jump, near jump or near call) */
		if (sign == 0xAA55 && !memcmp(fs->win + BS_FilSysType32, "FAT32   ", 8)) {
			return 0;	/* It is an FAT32 VBR */
		}
		/* FAT volumes created in the early MS-DOS era lack BS_55AA and BS_FilSysType, so FAT VBR needs to be identified without them. */
		w = ld_16(fs->win + BPB_BytsPerSec);
		b = fs->win[BPB_SecPerClus];
		if ((w & (w - 1)) == 0 && w >= FF_MIN_SS && w <= FF_MAX_SS	/* Properness of sector size (512-4096 and 2^n) */
		    && b != 0 && (b & (b - 1)) == 0				/* Properness of cluster size (2^n) */
		    && ld_16(fs->win + BPB_RsvdSecCnt) != 0		/* Properness of number of reserved sectors (MNBZ) */
		    && (UINT)fs->win[BPB_NumFATs] - 1 <= 1		/* Properness of FATs (1 or 2) */
		    && ld_16(fs->win + BPB_RootEntCnt) != 0		/* Properness of root dir size (MNBZ) */
		    && (ld_16(fs->win + BPB_TotSec16) >= 128
			|| ld_32(fs->win + BPB_TotSec32) >= 0x10000)	/* Properness of volume size (>=128) */
		    && ld_16(fs->win + BPB_FATSz16) != 0) {		/* Properness of FAT size (MNBZ) */
			return 0;	/* It can be presumed an FAT VBR */
		}
	}
	return sign == 0xAA55 ? 2 : 3;	/* Not an FAT VBR (with valid or invalid BS) */
}


/* Find an FAT volume */
/* (It supports only generic partitioning rules, MBR, GPT and SFD) */

static UINT find_volume (	/* Returns BS status found in the hosting drive */
	FATFS *fs,		/* Filesystem object */
	UINT part		/* Partition to fined = 0:find as SFD and partitions, >0:forced partition number */
)
{
	UINT fmt, i;
	DWORD mbr_pt[4];


	fmt = check_fs(fs, 0);				/* Load sector 0 and check if it is an FAT VBR as SFD format */
	if (fmt != 2 && (fmt >= 3 || part == 0)) {
		return fmt;        /* Returns if it is an FAT VBR as auto scan, not a BS or disk error */
	}

	/* Sector 0 is not an FAT VBR or forced partition number wants a partitioned drive */

#if FF_LBA64
	if (fs->win[MBR_Table + PTE_System] == 0xEE) {	/* GPT protective MBR? */
		DWORD n_ent, v_ent, ofs;
		QWORD pt_lba;

		if (move_window(fs, 1) != FR_OK) {
			return 4;        /* Load GPT header sector (next to MBR) */
		}
		if (!test_gpt_header(fs->win)) {
			return 3;        /* Check if GPT header is valid */
		}
		n_ent = ld_32(fs->win + GPTH_PtNum);		/* Number of entries */
		pt_lba = ld_64(fs->win + GPTH_PtOfs);		/* Table location */
		for (v_ent = i = 0; i < n_ent; i++) {		/* Find FAT partition */
			if (move_window(fs, pt_lba + i * SZ_GPTE / SS(fs)) != FR_OK) {
				return 4;        /* PT sector */
			}
			ofs = i * SZ_GPTE % SS(fs);												/* Offset in the sector */
			if (!memcmp(fs->win + ofs + GPTE_PtGuid, GUID_MS_Basic, 16)) {	/* MS basic data partition? */
				v_ent++;	/* Order of MS BDP */
				fmt = check_fs(fs, ld_64(fs->win + ofs + GPTE_FstLba));	/* Load VBR and check status */
				if (part == 0 && fmt <= 1) {
					return fmt;			/* Auto search (valid FAT volume found first) */
				}
				if (part != 0 && v_ent == part) {
					return fmt;        /* Forced partition order (regardless of it is valid or not) */
				}
			}
		}
		return 3;	/* Not found */
	}
#endif
	if (FF_MULTI_PARTITION && part > 4) {
		return 3;	/* MBR has four primary partitions max (FatFs does not support logical partition) */
	}
	for (i = 0; i < 4; i++) {		/* Load partition offset in the MBR */
		mbr_pt[i] = ld_32(fs->win + MBR_Table + i * SZ_PTE + PTE_StLba);
	}
	i = part ? part - 1 : 0;		/* Table index to find first */
	do {							/* Find an FAT volume */
		fmt = mbr_pt[i] ? check_fs(fs, mbr_pt[i]) : 3;	/* Check if the partition is FAT */
	}
	while (part == 0 && fmt >= 2 && ++i < 4);
	return fmt;
}




/*-----------------------------------------------------------------------*/
/* Determine logical drive number and mount the volume if needed         */
/*-----------------------------------------------------------------------*/

static FRESULT mount_volume (	/* FR_OK(0): successful, !=0: an error occurred */
	const TCHAR **path,			/* Pointer to pointer to the path name (drive number) */
	FATFS **rfs,				/* Pointer to pointer to the found filesystem object */
	BYTE mode					/* Desiered access mode to check write protection */
)
{
	int vol;
	FATFS *fs;
	DSTATUS stat;
	LBA_t bsect;
	UINT fmt;


	/* Get logical drive number */
	*rfs = 0;
	vol = get_ldnumber(path);
	if (vol < 0) {
		return FR_INVALID_DRIVE;
	}

	/* Check if the filesystem object is valid or not */
	fs = FatFs[vol];					/* Get pointer to the filesystem object */
	if (!fs) {
		return FR_NOT_ENABLED;        /* Is the filesystem object available? */
	}
#if FF_FS_REENTRANT
	if (!lock_volume(fs, 1)) {
		return FR_TIMEOUT;        /* Lock the volume, and system if needed */
	}
#endif
	*rfs = fs;							/* Return pointer to the filesystem object */

	mode &= (BYTE)~FA_READ;				/* Desired access mode, write access or not */
	if (fs->fs_type != 0) {				/* If the volume has been mounted */
		stat = disk_status(fs->pdrv);
		if (!(stat & STA_NOINIT)) {		/* and the physical drive is kept initialized */
			if (!FF_FS_READONLY && mode && (stat & STA_PROTECT)) {	/* Check write protection if needed */
				return FR_WRITE_PROTECTED;
			}
			return FR_OK;				/* The filesystem object is already valid */
		}
	}

	/* The filesystem object is not valid. */
	/* Following code attempts to mount the volume. (find an FAT volume, analyze the BPB and initialize the filesystem object) */

	fs->fs_type = 0;					/* Invalidate the filesystem object */
	stat = disk_initialize(fs->pdrv);	/* Initialize the volume hosting physical drive */
	if (stat & STA_NOINIT) { 			/* Check if the initialization succeeded */
		return FR_NOT_READY;			/* Failed to initialize due to no medium or hard error */
	}
	if (!FF_FS_READONLY && mode && (stat & STA_PROTECT)) { /* Check disk write protection if needed */
		return FR_WRITE_PROTECTED;
	}
#if FF_MAX_SS != FF_MIN_SS				/* Get sector size (multiple sector size cfg only) */
	if (disk_ioctl(fs->pdrv, GET_SECTOR_SIZE, &SS(fs)) != RES_OK) {
		return FR_DISK_ERR;
	}
	if (SS(fs) > FF_MAX_SS || SS(fs) < FF_MIN_SS || (SS(fs) & (SS(fs) - 1))) {
		return FR_DISK_ERR;
	}
#endif

	/* Find an FAT volume on the hosting drive */
	fmt = find_volume(fs, LD2PT(vol));
	if (fmt == 4) {
		return FR_DISK_ERR;        /* An error occurred in the disk I/O layer */
	}
	if (fmt >= 2) {
		return FR_NO_FILESYSTEM;        /* No FAT volume is found */
	}
	bsect = fs->winsect;					/* Volume offset in the hosting physical drive */

	/* An FAT volume is found (bsect). Following code initializes the filesystem object */

#if FF_FS_EXFAT
	if (fmt == 1) {
		QWORD maxlba;
		DWORD so, cv, bcl, ncl, i;

		for (i = BPB_ZeroedEx; i < BPB_ZeroedEx + 53 && fs->win[i] == 0; i++) ;	/* Check zero filler */
		if (i < BPB_ZeroedEx + 53) {
			return FR_NO_FILESYSTEM;
		}

		if (ld_16(fs->win + BPB_FSVerEx) != 0x100) {
			return FR_NO_FILESYSTEM;        /* Check exFAT version (must be version 1.0) */
		}

		if (1 << fs->win[BPB_BytsPerSecEx] != SS(fs)) {	/* (BPB_BytsPerSecEx must be equal to the physical sector size) */
			return FR_NO_FILESYSTEM;
		}

		maxlba = ld_64(fs->win + BPB_TotSecEx) + bsect;	/* Last LBA of the volume + 1 */
		if (!FF_LBA64 && maxlba >= 0x100000000) {
			return FR_NO_FILESYSTEM;        /* (It cannot be accessed in 32-bit LBA) */
		}

		fs->fsize = ld_32(fs->win + BPB_FatSzEx);		/* Number of sectors per FAT */

		fs->n_fats = fs->win[BPB_NumFATsEx];			/* Number of FATs */
		if (fs->n_fats != 1) {
			return FR_NO_FILESYSTEM;        /* (Supports only 1 FAT) */
		}

		fs->csize = 1 << fs->win[BPB_SecPerClusEx];		/* Cluster size */
		if (fs->csize == 0)	{
			return FR_NO_FILESYSTEM;        /* (Must be 1..32768 sectors) */
		}

		ncl = ld_32(fs->win + BPB_NumClusEx);			/* Number of clusters */
		if (ncl > MAX_EXFAT) {
			return FR_NO_FILESYSTEM;        /* (Too many clusters) */
		}
		fs->n_fatent = ncl + 2;

		/* Boundaries and Limits */
		fs->volbase = bsect;
		fs->database = bsect + ld_32(fs->win + BPB_DataOfsEx);
		fs->fatbase = bsect + ld_32(fs->win + BPB_FatOfsEx);
		if (maxlba < (QWORD)fs->database + ncl * fs->csize) {
			return FR_NO_FILESYSTEM;        /* (Volume size must not be smaller than the size required) */
		}
		fs->dirbase = ld_32(fs->win + BPB_RootClusEx);

		/* Get bitmap location and check if it is contiguous (implementation assumption) */
		so = i = 0;
		for (;;) {	/* Find the bitmap entry in the root directory (in only first cluster) */
			if (i == 0) {
				if (so >= fs->csize) {
					return FR_NO_FILESYSTEM;        /* Not found? */
				}
				if (move_window(fs, clst2sect(fs, (DWORD)fs->dirbase) + so) != FR_OK) {
					return FR_DISK_ERR;
				}
				so++;
			}
			if (fs->win[i] == ET_BITMAP) {
				break;		/* Is it a bitmap entry? */
			}
			i = (i + SZDIRE) % SS(fs);	/* Next entry */
		}
		bcl = ld_32(fs->win + i + 20);				/* Bitmap cluster */
		if (bcl < 2 || bcl >= fs->n_fatent) {
			return FR_NO_FILESYSTEM;        /* (Wrong cluster#) */
		}
		fs->bitbase = fs->database + fs->csize * (bcl - 2);	/* Bitmap sector */
		for (;;) {	/* Check if bitmap is contiguous */
			if (move_window(fs, fs->fatbase + bcl / (SS(fs) / 4)) != FR_OK) {
				return FR_DISK_ERR;
			}
			cv = ld_32(fs->win + bcl % (SS(fs) / 4) * 4);
			if (cv == 0xFFFFFFFF) {
				break;        /* Last link? */
			}
			if (cv != ++bcl) {
				return FR_NO_FILESYSTEM;        /* Fragmented bitmap? */
			}
		}

#if !FF_FS_READONLY
		fs->last_clst = fs->free_clst = 0xFFFFFFFF;		/* Invalidate cluster allocation information */
#endif
		fmt = FS_EXFAT;			/* FAT sub-type */
	}
	else
#endif	/* FF_FS_EXFAT */
	{
		DWORD tsect, sysect, fasize, nclst, szbfat;
		WORD nrsv;

		if (ld_16(fs->win + BPB_BytsPerSec) != SS(fs)) {
			return FR_NO_FILESYSTEM;        /* (BPB_BytsPerSec must be equal to the physical sector size) */
		}

		fasize = ld_16(fs->win + BPB_FATSz16);		/* Number of sectors per FAT */
		if (fasize == 0) {
			fasize = ld_32(fs->win + BPB_FATSz32);
		}
		fs->fsize = fasize;

		fs->n_fats = fs->win[BPB_NumFATs];				/* Number of FATs */
		if (fs->n_fats != 1 && fs->n_fats != 2) {
			return FR_NO_FILESYSTEM;        /* (Must be 1 or 2) */
		}
		fasize *= fs->n_fats;							/* Number of sectors for FAT area */

		fs->csize = fs->win[BPB_SecPerClus];			/* Cluster size */
		if (fs->csize == 0 || (fs->csize & (fs->csize - 1))) {
			return FR_NO_FILESYSTEM;        /* (Must be power of 2) */
		}

		fs->n_rootdir = ld_16(fs->win + BPB_RootEntCnt);	/* Number of root directory entries */
		if (fs->n_rootdir % (SS(fs) / SZDIRE)) {
			return FR_NO_FILESYSTEM;        /* (Must be sector aligned) */
		}

		tsect = ld_16(fs->win + BPB_TotSec16);			/* Number of sectors on the volume */
		if (tsect == 0) {
			tsect = ld_32(fs->win + BPB_TotSec32);
		}

		nrsv = ld_16(fs->win + BPB_RsvdSecCnt);			/* Number of reserved sectors */
		if (nrsv == 0) {
			return FR_NO_FILESYSTEM;        /* (Must not be 0) */
		}

		/* Determine the FAT sub type */
		sysect = nrsv + fasize + fs->n_rootdir / (SS(fs) / SZDIRE);	/* RSV + FAT + DIR */
		if (tsect < sysect) {
			return FR_NO_FILESYSTEM;        /* (Invalid volume size) */
		}
		nclst = (tsect - sysect) / fs->csize;			/* Number of clusters */
		if (nclst == 0) {
			return FR_NO_FILESYSTEM;        /* (Invalid volume size) */
		}
		fmt = 0;
		if (nclst <= MAX_FAT32) {
			fmt = FS_FAT32;
		}
		if (nclst <= MAX_FAT16) {
			fmt = FS_FAT16;
		}
		if (nclst <= MAX_FAT12) {
			fmt = FS_FAT12;
		}
		if (fmt == 0) {
			return FR_NO_FILESYSTEM;
		}

		/* Boundaries and Limits */
		fs->n_fatent = nclst + 2;						/* Number of FAT entries */
		fs->volbase = bsect;							/* Volume start sector */
		fs->fatbase = bsect + nrsv; 					/* FAT start sector */
		fs->database = bsect + sysect;					/* Data start sector */
		if (fmt == FS_FAT32) {
			if (ld_16(fs->win + BPB_FSVer32) != 0) {
				return FR_NO_FILESYSTEM;        /* (Must be FAT32 revision 0.0) */
			}
			if (fs->n_rootdir != 0) {
				return FR_NO_FILESYSTEM;        /* (BPB_RootEntCnt must be 0) */
			}
			fs->dirbase = ld_32(fs->win + BPB_RootClus32);	/* Root directory start cluster */
			szbfat = fs->n_fatent * 4;					/* (Needed FAT size) */
		}
		else {
			if (fs->n_rootdir == 0)	{
				return FR_NO_FILESYSTEM;        /* (BPB_RootEntCnt must not be 0) */
			}
			fs->dirbase = fs->fatbase + fasize;			/* Root directory start sector */
			szbfat = (fmt == FS_FAT16) ?				/* (Needed FAT size) */
				 fs->n_fatent * 2 : fs->n_fatent * 3 / 2 + (fs->n_fatent & 1);
		}
		if (fs->fsize < (szbfat + (SS(fs) - 1)) / SS(fs)) {
			return FR_NO_FILESYSTEM;        /* (BPB_FATSz must not be less than the size needed) */
		}

#if !FF_FS_READONLY
		/* Get FSInfo if available */
		fs->last_clst = fs->free_clst = 0xFFFFFFFF;		/* Invalidate cluster allocation information */
		fs->fsi_flag = 0x80;	/* Disable FSInfo by default */
		if (fmt == FS_FAT32
		    && ld_16(fs->win + BPB_FSInfo32) == 1	/* FAT32: Enable FSInfo feature only if FSInfo sector is next to VBR */
		    && move_window(fs, bsect + 1) == FR_OK) {
			fs->fsi_flag = 0;
			if (   ld_32(fs->win + FSI_LeadSig) == 0x41615252	/* Load FSInfo data if available */
			    && ld_32(fs->win + FSI_StrucSig) == 0x61417272
			    && ld_32(fs->win + FSI_TrailSig) == 0xAA550000) {
#if (FF_FS_NOFSINFO & 1) == 0	/* Get free cluster count if trust it */
				fs->free_clst = ld_32(fs->win + FSI_Free_Count);
#endif
#if (FF_FS_NOFSINFO & 2) == 0	/* Get next free cluster if rtust it */
				fs->last_clst = ld_32(fs->win + FSI_Nxt_Free);
#endif
			}
		}
#endif	/* !FF_FS_READONLY */
	}

	fs->fs_type = (BYTE)fmt;/* FAT sub-type (the filesystem object gets valid) */
	fs->id = ++Fsid;		/* Volume mount ID */

#if FF_USE_LFN == 1			/* Initialize pointers to the static working buffers */
	fs->lfnbuf = LfnBuf;	/* LFN working buffer */
#if FF_FS_EXFAT
	fs->dirbuf = DirBuf;	/* Directory block scratchpad buuffer */
#endif
#endif

#if FF_FS_RPATH				/* Set the current directory top layer (root) */
	fs->cdir = 0;
#if FF_FS_EXFAT
	memset(&fs->xcwds, 0, sizeof fs->xcwds);
#endif
#endif

#if FF_FS_LOCK				/* Clear file lock semaphores */
	clear_share(fs);
#endif

	return FR_OK;
}




/*-----------------------------------------------------------------------*/
/* Check if the file/directory object is valid or not                    */
/*-----------------------------------------------------------------------*/

static FRESULT validate (	/* Returns FR_OK or FR_INVALID_OBJECT */
	FFOBJID *obj,			/* Pointer to the FFOBJID, the 1st member in the FIL/DIR structure, to check validity */
	FATFS **rfs				/* Pointer to pointer to the owner filesystem object to return */
)
{
	FRESULT res = FR_INVALID_OBJECT;


	if (obj && obj->fs && obj->fs->fs_type && obj->id == obj->fs->id) {	/* Test if the object is valid */
#if FF_FS_REENTRANT
		if (lock_volume(obj->fs, 0)) {	/* Take a grant to access the volume */
			if (!(disk_status(obj->fs->pdrv) & STA_NOINIT)) { /* Test if the hosting physical drive is kept initialized */
				res = FR_OK;
			}
			else {
				unlock_volume(obj->fs, FR_OK);	/* Invalidated volume, abort to access */
			}
		}
		else {	/* Could not take */
			res = FR_TIMEOUT;
		}
#else
		if (!(disk_status(obj->fs->pdrv) & STA_NOINIT)) { /* Test if the hosting physical drive is kept initialized */
			res = FR_OK;
		}
#endif
	}
	*rfs = (res == FR_OK) ? obj->fs : 0;	/* Return corresponding filesystem object if it is valid */
	return res;
}




/*---------------------------------------------------------------------------

   Public Functions (FatFs API)

----------------------------------------------------------------------------*/



/*-----------------------------------------------------------------------*/
/* API: Mount/Unmount a Logical Drive                                    */
/*-----------------------------------------------------------------------*/

FRESULT f_mount (
	FATFS *fs,			/* Pointer to the filesystem object to be registered (NULL:unmount)*/
	const TCHAR *path,	/* Logical drive number to be mounted/unmounted */
	BYTE opt			/* Mount option: 0=Do not mount (delayed mount), 1=Mount immediately */
)
{
	FATFS *cfs;
	int vol;
	FRESULT res = FR_DISK_ERR;
	const TCHAR *rp = path;


	/* Get volume ID (logical drive number) */
	vol = get_ldnumber(&rp);
	if (vol < 0) {
		return FR_INVALID_DRIVE;
	}

	cfs = FatFs[vol];			/* Pointer to the filesystem object of the volume */
	if (cfs) {					/* Unregister current filesystem object */
		FatFs[vol] = 0;
#if FF_FS_LOCK					/* Clear file lock semaphores correspond to this volume */
		clear_share(cfs);
#endif
#if FF_FS_REENTRANT				/* Discard mutex of the current volume */
		ff_mutex_delete(vol);
#endif
		cfs->fs_type = 0;		/* Invalidate the filesystem object to be unregistered */
	}

	if (fs) {					/* Register new filesystem object */
		fs->pdrv = LD2PD(vol);	/* Volume hosting physical drive */
#if FF_FS_REENTRANT				/* Create a volume mutex */
		fs->ldrv = (BYTE)vol;	/* Owner volume ID */
		if (!ff_mutex_create(vol)) {
			return FR_INT_ERR;
		}
#if FF_FS_LOCK
		if (SysLock == 0) {		/* Create a system mutex if needed */
			if (!ff_mutex_create(FF_VOLUMES)) {
				ff_mutex_delete(vol);
				return FR_INT_ERR;
			}
			SysLock = 1;		/* System mutex is ready */
		}
#endif
#endif
		fs->fs_type = 0;		/* Invalidate the new filesystem object */
		FatFs[vol] = fs;		/* Register it */
	}

	if (opt == 0) {
		return FR_OK;        /* Do not mount now, it will be mounted in subsequent file functions */
	}

	res = mount_volume(&path, &fs, 0);	/* Force mounted the volume in this function */
	LEAVE_FF(fs, res);
}




/*-----------------------------------------------------------------------*/
/* API: Open or Create a File                                            */
/*-----------------------------------------------------------------------*/

FRESULT f_open (
	FIL *fp,			/* Pointer to the blank file object */
	const TCHAR *path,	/* Pointer to the file name */
	BYTE mode			/* Access mode and open mode flags */
)
{
	FRESULT res = FR_DISK_ERR;
	DIR dj = {0};
	FATFS *fs;
	DEF_NAMEBUFF


	if (!fp) {
		return FR_INVALID_OBJECT;	/* Reject null pointer */
	}

	/* Get logical drive number and mount the volume if needed */
	mode &= FF_FS_READONLY ? FA_READ : FA_READ | FA_WRITE | FA_CREATE_ALWAYS | FA_CREATE_NEW | FA_OPEN_ALWAYS |
		FA_OPEN_APPEND;
	res = mount_volume(&path, &fs, mode);

	if (res == FR_OK) {
		fp->obj.fs = fs;
		dj.obj.fs = fs;
		INIT_NAMEBUFF(fs);
		res = follow_path(&dj, path);	/* Follow the file path */
#if !FF_FS_READONLY	/* Read/Write configuration */
		if (res == FR_OK) {
			if (dj.fn[NSFLAG] & NS_NONAME) {	/* Origin directory itself? */
				res = FR_INVALID_NAME;
			}
#if FF_FS_LOCK
			else {
				res = chk_share(&dj, (mode & ~FA_READ) ? 1 : 0);	/* Check if the file can be used */
			}
#endif
		}
		/* Create or Open a file */
		if (mode & (FA_CREATE_ALWAYS | FA_OPEN_ALWAYS | FA_CREATE_NEW)) {
			if (res != FR_OK) {					/* No file, create new */
				if (res == FR_NO_FILE) {		/* There is no file to open, create a new entry */
#if FF_FS_LOCK
					res = enq_share() ? dir_register(&dj) : FR_TOO_MANY_OPEN_FILES;
#else
					res = dir_register(&dj);
#endif
				}
				mode |= FA_CREATE_ALWAYS;		/* File is created */
			}
			else {								/* An object with the same name is already existing */
				if (mode & FA_CREATE_NEW) {
					res = FR_EXIST;				/* Cannot create as new file */
				}
				else {
					if (dj.obj.attr & (AM_RDO | AM_DIR)) {
						res = FR_DENIED;	/* Cannot overwrite it (R/O or DIR) */
					}
				}
			}
			if (res == FR_OK && (mode & FA_CREATE_ALWAYS)) {	/* Truncate the file if overwrite mode */
				DWORD tm = GET_FATTIME();
#if FF_FS_EXFAT
				if (fs->fs_type == FS_EXFAT) {
					/* Get current allocation info */
					init_alloc_info(&fp->obj, 0);
					/* Set exFAT directory entry block initial state */
					memset(fs->dirbuf + 2, 0, 30);	/* Clear 85 entry except for NumSec */
					memset(fs->dirbuf + 38, 0, 26);	/* Clear C0 entry except for NumName and NameHash */
					fs->dirbuf[XDIR_Attr] = AM_ARC;
					st_32(fs->dirbuf + XDIR_CrtTime, tm);	/* Set created time */
					st_32(fs->dirbuf + XDIR_ModTime, tm);	/* Set modified time (tmp setting) */
					fs->dirbuf[XDIR_GenFlags] = 1;
					res = store_xdir(&dj);
					if (res == FR_OK && fp->obj.sclust != 0) {	/* Remove the cluster chain if exist */
						res = remove_chain(&fp->obj, fp->obj.sclust, 0);
						fs->last_clst = fp->obj.sclust - 1;		/* Reuse the cluster hole */
					}
				}
				else
#endif
				{
					DWORD cl;
					/* Set FAT directory entry initial state */
					st_32(dj.dir + DIR_CrtTime, tm);	/* Set created time */
					st_32(dj.dir + DIR_ModTime, tm);	/* Set modified time (tmp setting) */
					cl = ld_clust(fs, dj.dir);			/* Get current cluster chain */
					dj.dir[DIR_Attr] = AM_ARC;			/* Reset attribute */
					st_clust(fs, dj.dir, 0);			/* Reset file allocation info */
					st_32(dj.dir + DIR_FileSize, 0);
					fs->wflag = 1;
					if (cl != 0) {						/* Remove the cluster chain if exist */
						LBA_t sc = fs->winsect;

						res = remove_chain(&dj.obj, cl, 0);
						if (res == FR_OK) {
							res = move_window(fs, sc);
							fs->last_clst = cl - 1;		/* Reuse the cluster hole */
						}
					}
				}
			}
		}
		else {	/* Open an existing file */
			if (res == FR_OK) {					/* Is the object existing? */
				if (dj.obj.attr & AM_DIR) {		/* File open against a directory */
					res = FR_NO_FILE;
				}
				else {
					if ((mode & FA_WRITE) && (dj.obj.attr & AM_RDO)) { /* Write mode open against R/O file */
						res = FR_DENIED;
					}
				}
			}
		}
		if (res == FR_OK) {
			if (mode & FA_CREATE_ALWAYS) {
				mode |= FA_MODIFIED;        /* Set file change flag if created or overwritten */
			}
			fp->dir_sect = fs->winsect;			/* Pointer to the directory entry */
			fp->dir_ptr = dj.dir;
#if FF_FS_LOCK
			fp->obj.lockid = inc_share(&dj, (mode & ~FA_READ) ? 1 : 0);	/* Lock the file for this session */
			if (fp->obj.lockid == 0) {
				res = FR_INT_ERR;
			}
#endif
		}

#else	/* R/O configuration */
		if (res == FR_OK) {
			if (dj.fn[NSFLAG] & NS_NONAME) {	/* Is it origin directory itself? */
				res = FR_INVALID_NAME;
			}
			else {
				if (dj.obj.attr & AM_DIR) {		/* Is it a directory? */
					res = FR_NO_FILE;
				}
			}
		}
#endif

		if (res == FR_OK) {
#if FF_FS_EXFAT
			if (fs->fs_type == FS_EXFAT) {
				init_alloc_info(&fp->obj, &dj);
			}
			else
#endif
			{
				fp->obj.sclust = ld_clust(fs, dj.dir);					/* Get object allocation info */
				fp->obj.objsize = ld_32(dj.dir + DIR_FileSize);
			}
#if FF_USE_FASTSEEK
			fp->cltbl = 0;		/* Disable fast seek mode */
#endif
			fp->obj.id = fs->id;	/* Set current volume mount ID */
			fp->flag = mode;	/* Set file access mode */
			fp->err = 0;		/* Clear error flag */
			fp->sect = 0;		/* Invalidate current data sector */
			fp->fptr = 0;		/* Set file pointer top of the file */
#if !FF_FS_READONLY
#if !FF_FS_TINY
			memset(fp->buf, 0, sizeof fp->buf);	/* Clear sector buffer */
#endif
			if ((mode & FA_SEEKEND) && fp->obj.objsize > 0) {	/* Seek to end of file if FA_OPEN_APPEND is specified */
				DWORD bcs, clst;
				FSIZE_t ofs;

				fp->fptr = fp->obj.objsize;			/* Offset to seek */
				bcs = (DWORD)fs->csize * SS(fs);	/* Cluster size in byte */
				clst = fp->obj.sclust;				/* Follow the cluster chain */
				for (ofs = fp->obj.objsize; res == FR_OK && ofs > bcs; ofs -= bcs) {
					clst = get_fat(&fp->obj, clst);
					if (clst <= 1) {
						res = FR_INT_ERR;
					}
					if (clst == 0xFFFFFFFF) {
						res = FR_DISK_ERR;
					}
				}
				fp->clust = clst;
				if (res == FR_OK && ofs % SS(fs)) {	/* Fill sector buffer if not on the sector boundary */
					LBA_t sec = clst2sect(fs, clst);

					if (sec == 0) {
						res = FR_INT_ERR;
					}
					else {
						fp->sect = sec + (DWORD)(ofs / SS(fs));
#if !FF_FS_TINY
						if (disk_read(fs->pdrv, fp->buf, fp->sect, 1) != RES_OK) {
							res = FR_DISK_ERR;
						}
#endif
					}
				}
#if FF_FS_LOCK
				if (res != FR_OK) {
					dec_share(fp->obj.lockid);        /* Decrement file open counter if seek failed */
				}
#endif
			}
#endif
		}

		FREE_NAMEBUFF();
	}

	if (res != FR_OK) {
		fp->obj.fs = 0;        /* Invalidate file object on error */
	}

	LEAVE_FF(fs, res);
}




/*-----------------------------------------------------------------------*/
/* API: Read File                                                        */
/*-----------------------------------------------------------------------*/

FRESULT f_read (
	FIL *fp, 	/* Open file to be read */
	void *buff,	/* Data buffer to store the read data */
	UINT btr,	/* Number of bytes to read */
	UINT *br	/* Number of bytes read */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;
	LBA_t sect;
	FSIZE_t remain;
	UINT rcnt, cc, csect;
	BYTE *rbuff = (BYTE *)buff;


	*br = 0;	/* Clear read byte counter */
	res = validate(&fp->obj, &fs);				/* Check validity of the file object */
	if (res != FR_OK || (res = (FRESULT)fp->err) != FR_OK) {
		LEAVE_FF(fs, res);        /* Check validity */
	}
	if (!(fp->flag & FA_READ)) {
		LEAVE_FF(fs, FR_DENIED);        /* Check access mode */
	}
	remain = fp->obj.objsize - fp->fptr;
	if (btr > remain) {
		btr = (UINT)remain;        /* Truncate btr by remaining bytes */
	}

	for ( ; btr > 0; btr -= rcnt, *br += rcnt, rbuff += rcnt, fp->fptr += rcnt) {	/* Repeat until btr bytes read */
		if (fp->fptr % SS(fs) == 0) {			/* On the sector boundary? */
			csect = (UINT)(fp->fptr / SS(fs) & (fs->csize - 1));	/* Sector offset in the cluster */
			if (csect == 0) {					/* On the cluster boundary? */
				DWORD clst;

				if (fp->fptr == 0) {			/* On the top of the file? */
					clst = fp->obj.sclust;		/* Follow cluster chain from the origin */
				}
				else {						/* Middle or end of the file */
#if FF_USE_FASTSEEK
					if (fp->cltbl) {
						clst = clmt_clust(fp, fp->fptr);	/* Get cluster# from the CLMT */
					}
					else
#endif
					{
						clst = get_fat(&fp->obj, fp->clust);	/* Follow cluster chain on the FAT */
					}
				}
				if (clst < 2) {
					ABORT(fs, FR_INT_ERR);
				}
				if (clst == 0xFFFFFFFF) {
					ABORT(fs, FR_DISK_ERR);
				}
				fp->clust = clst;				/* Update current cluster */
			}
			sect = clst2sect(fs, fp->clust);	/* Get current sector */
			if (sect == 0) {
				ABORT(fs, FR_INT_ERR);
			}
			sect += csect;
			cc = btr / SS(fs);					/* When remaining bytes >= sector size, */
			if (cc > 0) {						/* Read maximum contiguous sectors directly */
				if (csect + cc > fs->csize) {	/* Clip at cluster boundary */
					cc = fs->csize - csect;
				}
				if (disk_read(fs->pdrv, rbuff, sect, cc) != RES_OK) {
					ABORT(fs, FR_DISK_ERR);
				}
#if !FF_FS_READONLY && FF_FS_MINIMIZE <= 2		/* Replace one of the read sectors with cached data if it contains a dirty sector */
#if FF_FS_TINY
				if (fs->wflag && fs->winsect - sect < cc) {
					mem_cpy(rbuff + ((fs->winsect - sect) * SS(fs)), fs->win, SS(fs));
				}
#else
				if ((fp->flag & FA_DIRTY) && fp->sect - sect < cc) {
					mem_cpy(rbuff + ((fp->sect - sect) * SS(fs)), fp->buf, SS(fs));
				}
#endif
#endif
				rcnt = SS(fs) * cc;				/* Number of bytes transferred */
				continue;
			}
#if !FF_FS_TINY
			if (fp->sect != sect) {			/* Load data sector if not in cache */
#if !FF_FS_READONLY
				if (fp->flag & FA_DIRTY) {		/* Write-back dirty sector cache */
					if (disk_write(fs->pdrv, fp->buf, fp->sect, 1) != RES_OK) {
						ABORT(fs, FR_DISK_ERR);
					}
					fp->flag &= (BYTE)~FA_DIRTY;
				}
#endif
				if (disk_read(fs->pdrv, fp->buf, sect, 1) != RES_OK) {
					ABORT(fs, FR_DISK_ERR);        /* Fill sector cache */
				}
			}
#endif
			fp->sect = sect;
		}
		rcnt = SS(fs) - (UINT)fp->fptr % SS(fs);	/* Number of bytes remains in the sector */
		if (rcnt > btr) {
			rcnt = btr;        /* Clip it by btr if needed */
		}
#if FF_FS_TINY
		if (move_window(fs, fp->sect) != FR_OK) {
			ABORT(fs, FR_DISK_ERR);        /* Move sector window */
		}
		mem_cpy(rbuff, fs->win + fp->fptr % SS(fs), rcnt);	/* Extract partial sector */
#else
		mem_cpy(rbuff, fp->buf + fp->fptr % SS(fs), rcnt);	/* Extract partial sector */
#endif
		/*
		 * For unaligned sector size read, file system send local buffer to the low-level driver,
		 * once the transfer completed then it copies the data from local buffer to user buffer.
		 * This results to data available in Cache but not in memory. To avoid this problem,
		 * Flush the DCache for the unaligned sector size read, to make sure both Cache
		 * and memory contents are same.
		 */
		Xil_DCacheFlushRange((INTPTR)rbuff, rcnt);
	}

	LEAVE_FF(fs, FR_OK);
}




#if !FF_FS_READONLY
/*-----------------------------------------------------------------------*/
/* API: Write File                                                       */
/*-----------------------------------------------------------------------*/

FRESULT f_write (
	FIL *fp,			/* Open file to be written */
	const void *buff,	/* Data to be written */
	UINT btw,			/* Number of bytes to write */
	UINT *bw			/* Number of bytes written */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;
	DWORD clst;
	LBA_t sect;
	UINT wcnt, cc, csect;
	const BYTE *wbuff = (const BYTE *)buff;


	*bw = 0;	/* Clear write byte counter */
	res = validate(&fp->obj, &fs);			/* Check validity of the file object */
	if (res != FR_OK || (res = (FRESULT)fp->err) != FR_OK) {
		LEAVE_FF(fs, res);        /* Check validity */
	}
	if (!(fp->flag & FA_WRITE)) {
		LEAVE_FF(fs, FR_DENIED);        /* Check access mode */
	}

	/* Check fptr wrap-around (file size cannot reach 4 GiB at FAT volume) */
	if ((!FF_FS_EXFAT || fs->fs_type != FS_EXFAT) && (DWORD)(fp->fptr + btw) < (DWORD)fp->fptr) {
		btw = (UINT)(0xFFFFFFFF - (DWORD)fp->fptr);
	}

	for ( ; btw > 0;
	      btw -= wcnt, *bw += wcnt, wbuff += wcnt, fp->fptr += wcnt,
	      fp->obj.objsize = (fp->fptr > fp->obj.objsize) ? fp->fptr : fp->obj.objsize) {	/* Repeat until all data written */
		if (fp->fptr % SS(fs) == 0) {		/* On the sector boundary? */
			csect = (UINT)(fp->fptr / SS(fs)) & (fs->csize - 1);	/* Sector offset in the cluster */
			if (csect == 0) {				/* On the cluster boundary? */
				if (fp->fptr == 0) {		/* On the top of the file? */
					clst = fp->obj.sclust;	/* Follow from the origin */
					if (clst == 0) {		/* If no cluster is allocated, */
						clst = create_chain(&fp->obj, 0);	/* create a new cluster chain */
					}
				}
				else {					/* On the middle or end of the file */
#if FF_USE_FASTSEEK
					if (fp->cltbl) {
						clst = clmt_clust(fp, fp->fptr);	/* Get cluster# from the CLMT */
					}
					else
#endif
					{
						clst = create_chain(&fp->obj, fp->clust);	/* Follow or stretch cluster chain on the FAT */
					}
				}
				if (clst == 0) {
					break;        /* Could not allocate a new cluster (disk full) */
				}
				if (clst == 1) {
					ABORT(fs, FR_INT_ERR);
				}
				if (clst == 0xFFFFFFFF) {
					ABORT(fs, FR_DISK_ERR);
				}
				fp->clust = clst;			/* Update current cluster */
				if (fp->obj.sclust == 0) {
					fp->obj.sclust = clst;        /* Set start cluster if the first write */
				}
			}
#if FF_FS_TINY
			if (fs->winsect == fp->sect && sync_window(fs) != FR_OK) {
				ABORT(fs, FR_DISK_ERR);        /* Write-back sector cache */
			}
#else
			if (fp->flag & FA_DIRTY) {		/* Write-back sector cache */
				if (disk_write(fs->pdrv, fp->buf, fp->sect, 1) != RES_OK) {
					ABORT(fs, FR_DISK_ERR);
				}
				fp->flag &= (BYTE)~FA_DIRTY;
			}
#endif
			sect = clst2sect(fs, fp->clust);	/* Get current sector */
			if (sect == 0) {
				ABORT(fs, FR_INT_ERR);
			}
			sect += csect;
			cc = btw / SS(fs);				/* When remaining bytes >= sector size, */
			if (cc > 0) {					/* Write maximum contiguous sectors directly */
				if (csect + cc > fs->csize) {	/* Clip at cluster boundary */
					cc = fs->csize - csect;
				}
				if (disk_write(fs->pdrv, wbuff, sect, cc) != RES_OK) {
					ABORT(fs, FR_DISK_ERR);
				}
#if FF_FS_MINIMIZE <= 2
#if FF_FS_TINY
				if (fs->winsect - sect < cc) {	/* Refill sector cache if it gets invalidated by the direct write */
					mem_cpy(fs->win, wbuff + ((fs->winsect - sect) * SS(fs)), SS(fs));
					fs->wflag = 0;
				}
#else
				if (fp->sect - sect < cc) { /* Refill sector cache if it gets invalidated by the direct write */
					mem_cpy(fp->buf, wbuff + ((fp->sect - sect) * SS(fs)), SS(fs));
					fp->flag &= (BYTE)~FA_DIRTY;
				}
#endif
#endif
				wcnt = SS(fs) * cc;		/* Number of bytes transferred */
				continue;
			}
#if FF_FS_TINY
			if (fp->fptr >= fp->obj.objsize) {	/* Avoid silly cache filling on the growing edge */
				if (sync_window(fs) != FR_OK) {
					ABORT(fs, FR_DISK_ERR);
				}
				fs->winsect = sect;
			}
#else
			if (fp->sect != sect && 		/* Fill sector cache with file data */
			    fp->fptr < fp->obj.objsize &&
			    disk_read(fs->pdrv, fp->buf, sect, 1) != RES_OK) {
				ABORT(fs, FR_DISK_ERR);
			}
#endif
			fp->sect = sect;
		}
		wcnt = SS(fs) - (UINT)fp->fptr % SS(fs);	/* Number of bytes remains in the sector */
		if (wcnt > btw) {
			wcnt = btw;        /* Clip it by btw if needed */
		}
#if FF_FS_TINY
		if (move_window(fs, fp->sect) != FR_OK) {
			ABORT(fs, FR_DISK_ERR);        /* Move sector window */
		}
		mem_cpy(fs->win + fp->fptr % SS(fs), wbuff, wcnt);	/* Fit data to the sector */
		fs->wflag = 1;
#else
		mem_cpy(fp->buf + fp->fptr % SS(fs), wbuff, wcnt);	/* Fit data to the sector */
		fp->flag |= FA_DIRTY;
#endif
	}

	fp->flag |= FA_MODIFIED;				/* Set file change flag */

	LEAVE_FF(fs, FR_OK);
}




/*-----------------------------------------------------------------------*/
/* API: Synchronize the File                                             */
/*-----------------------------------------------------------------------*/

FRESULT f_sync (
	FIL *fp		/* Open file to be synced */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;


	res = validate(&fp->obj, &fs);	/* Check validity of the file object */
	if (res == FR_OK) {
		if (fp->flag & FA_MODIFIED) {	/* Is there any change to the file? */
#if !FF_FS_TINY
			if (fp->flag & FA_DIRTY) {	/* Write-back cached data if needed */
				if (disk_write(fs->pdrv, fp->buf, fp->sect, 1) != RES_OK) {
					LEAVE_FF(fs, FR_DISK_ERR);
				}
				fp->flag &= (BYTE)~FA_DIRTY;
			}
#endif
			/* Update the directory entry */
#if FF_FS_EXFAT
			if (fs->fs_type == FS_EXFAT) {
				res = fill_first_frag(&fp->obj);	/* Fill first fragment on the FAT if needed */
				if (res == FR_OK) {
					res = fill_last_frag(&fp->obj, fp->clust, 0xFFFFFFFF);	/* Fill last fragment on the FAT if needed */
				}
				if (res == FR_OK) {
					DIR dj;
					DEF_NAMEBUFF

					INIT_NAMEBUFF(fs);
					res = load_obj_xdir(&dj, &fp->obj);	/* Load directory entry block */
					if (res == FR_OK) {
						fs->dirbuf[XDIR_Attr] |= AM_ARC;					/* Set archive attribute to indicate that the file has been changed */
						fs->dirbuf[XDIR_GenFlags] = fp->obj.stat | 1;		/* Update file allocation information */
						st_32(fs->dirbuf + XDIR_FstClus, fp->obj.sclust);	/* Update start cluster */
						st_64(fs->dirbuf + XDIR_FileSize, fp->obj.objsize);	/* Update file size */
						st_64(fs->dirbuf + XDIR_ValidFileSize, fp->obj.objsize);	/* (FatFs does not support Valid File Size feature) */
						st_32(fs->dirbuf + XDIR_ModTime, GET_FATTIME());	/* Update modified time */
						fs->dirbuf[XDIR_ModTime10] = 0;
						fs->dirbuf[XDIR_ModTZ] = 0;
						st_32(fs->dirbuf + XDIR_AccTime, 0);				/* Invalidate last access time */
						fs->dirbuf[XDIR_AccTZ] = 0;
						res = store_xdir(&dj);								/* Restore it to the directory */
						if (res == FR_OK) {
							res = sync_fs(fs);
							fp->flag &= (BYTE)~FA_MODIFIED;
						}
					}
					FREE_NAMEBUFF();
				}
			}
			else
#endif
			{
				res = move_window(fs, fp->dir_sect);
				if (res == FR_OK) {
					BYTE *dir = fp->dir_ptr;

					dir[DIR_Attr] |= AM_ARC;					/* Set archive attribute to indicate that the file has been changed */
					st_clust(fp->obj.fs, dir, fp->obj.sclust);	/* Update file allocation information  */
					st_32(dir + DIR_FileSize, (DWORD)fp->obj.objsize);	/* Update file size */
					st_32(dir + DIR_ModTime, GET_FATTIME());	/* Update modified time */
					st_16(dir + DIR_LstAccDate, 0);				/* Invalidate last access date */
					fs->wflag = 1;
					res = sync_fs(fs);							/* Restore it to the directory */
					fp->flag &= (BYTE)~FA_MODIFIED;
				}
			}
		}
	}

	LEAVE_FF(fs, res);
}

#endif /* !FF_FS_READONLY */




/*-----------------------------------------------------------------------*/
/* API: Close File                                                       */
/*-----------------------------------------------------------------------*/

FRESULT f_close (
	FIL *fp		/* Open file to be closed */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;

#if !FF_FS_READONLY
	res = f_sync(fp);					/* Flush cached data */
	if (res == FR_OK)
#endif
	{
		res = validate(&fp->obj, &fs);	/* Lock volume */
		if (res == FR_OK) {
#if FF_FS_LOCK
			res = dec_share(fp->obj.lockid);		/* Decrement file open counter */
			if (res == FR_OK) {
				fp->obj.fs = 0;        /* Invalidate file object */
			}
#else
			fp->obj.fs = 0;	/* Invalidate file object */
#endif
#if FF_FS_REENTRANT
			unlock_volume(fs, FR_OK);		/* Unlock volume */
#endif
		}
	}
	return res;
}




#if FF_FS_RPATH >= 1
/*-----------------------------------------------------------------------*/
/* API: Change Current Drive                                             */
/*-----------------------------------------------------------------------*/

FRESULT f_chdrive (
	const TCHAR *path		/* Drive number to set */
)
{
	int vol;


	/* Get logical drive number */
	vol = get_ldnumber(&path);
	if (vol < 0) {
		return FR_INVALID_DRIVE;
	}
	CurrVol = (BYTE)vol;	/* Set it as current volume */

	return FR_OK;
}



/*-----------------------------------------------------------------------*/
/* API: Change Current Directory                                         */
/*-----------------------------------------------------------------------*/

FRESULT f_chdir (
	const TCHAR *path	/* Pointer to the directory path */
)
{
	FRESULT res = FR_DISK_ERR;
	DIR dj;
	FATFS *fs;
	DEF_NAMEBUFF


	res = mount_volume(&path, &fs, 0);	/* Get logical drive and mount the volume if needed */
	if (res == FR_OK) {
		dj.obj.fs = fs;
		INIT_NAMEBUFF(fs);
		res = follow_path(&dj, path);		/* Follow the path */
		if (res == FR_OK) {					/* Follow completed */
			if (dj.fn[NSFLAG] & NS_NONAME) {	/* Is it the start directory itself? */
#if FF_FS_EXFAT
				if (fs->fs_type == FS_EXFAT) {
					mem_cpy(&fs->xcwds, &fs->xcwds2, (UINT)(sizeof fs->xcwds));
				}
#endif
				fs->cdir = dj.obj.sclust;
			}
			else {
				if (dj.obj.attr & AM_DIR) {	/* It is a sub-directory */
#if FF_FS_EXFAT
					if (fs->fs_type == FS_EXFAT) {
						mem_cpy(&fs->xcwds, &fs->xcwds2, (UINT)(sizeof fs->xcwds));
						fs->cdir = fs->xcwds.tbl[fs->xcwds.depth].d_scl;	/* Sub-directory cluster */
					}
					else
#endif
					{
						fs->cdir = ld_clust(fs, dj.dir);	/* Sub-directory cluster */
					}
				}
				else {
					res = FR_NO_PATH;		/* Reached but a file */
				}
			}
		}
		FREE_NAMEBUFF();
		if (res == FR_NO_FILE) {
			res = FR_NO_PATH;
		}
#if FF_STR_VOLUME_ID == 2	/* Also current drive is changed if in Unix style volume ID */
		if (res == FR_OK) {
			UINT i;

			for (i = FF_VOLUMES - 1; i && fs != FatFs[i]; i--) ;	/* Set current drive */
			CurrVol = (BYTE)i;
		}
#endif
	}

	LEAVE_FF(fs, res);
}



#if FF_FS_RPATH >= 2
/*-----------------------------------------------------------------------*/
/* API: Get Current Directory                                            */
/*-----------------------------------------------------------------------*/

FRESULT f_getcwd (
	TCHAR* buff,	/* Pointer to the buffer to store the current directory path */
	UINT len		/* Size of buff in unit of TCHAR */
)
{
	FRESULT res = FR_DISK_ERR;
	DIR dj;
	FATFS *fs;
#if FF_VOLUMES >= 2
	UINT vl;
#if FF_STR_VOLUME_ID
	const char *vid;
#endif
#endif
	FILINFO fno;
	DEF_NAMEBUFF


	buff[0] = 0;	/* A null str to get current drive */
	res = mount_volume((const TCHAR**)&buff, &fs, 0);
	if (res == FR_OK) {
		dj.obj.fs = fs;
		INIT_NAMEBUFF(fs);
#if FF_FS_EXFAT
		if (fs->fs_type == FS_EXFAT) {	/* On the exFAT volume */
			UINT wi = 0;
			UINT di, ni;

#if FF_VOLUMES >= 2			/* Add drive prefix if needed */
#if FF_STR_VOLUME_ID == 0	/* Numeric volume ID */
			if (wi < len) buff[wi++] = '0' + CurrVol;
#else						/* String volume ID */
			if (FF_STR_VOLUME_ID == 2 && wi < len) buff[wi++] = '/';
			for (vid = (const char*)VolumeStr[CurrVol]; *vid && wi < len; buff[wi++] = *vid++) ;
#endif
			if (FF_STR_VOLUME_ID <= 1 && wi < len) buff[wi++] = ':';
#endif
			if (wi < len) buff[wi++] = '/';
			for (di = 0; wi < len && di < fs->xcwds.depth; di++) {	/* Follow current directory path with cwd structure */
				dj.obj.sclust = fs->xcwds.tbl[di].d_scl;		/* Open this directory */
				dj.obj.stat = (BYTE)fs->xcwds.tbl[di].d_size;
				dj.obj.objsize = fs->xcwds.tbl[di].d_size & 0xFFFFFF00;
				res = dir_sdi(&dj, fs->xcwds.tbl[di].nxt_ofs);	/* Find next directory */
				if (res != FR_OK) break;
				res = DIR_READ_FILE(&dj);						/* Get the directory name */
				if (res != FR_OK) break;
				get_fileinfo(&dj, &fno);
				if (di > 0 && wi < len) buff[wi++] = '/';		/* Add the directory name with a directory separator */
				for (ni = 0; fno.fname[ni] && wi < len; buff[wi++] = fno.fname[ni++]) ;
			}
			if (wi == len) {	/* Buffer overflow? */
				res = FR_NOT_ENOUGH_CORE;
			} else {
				buff[wi] = 0;	/* Terminate the string */
			}
		}
		else
#endif
		{	/* On the FAT/FAT32 volume */
			TCHAR *tp = buff;
			UINT i, nl;
			DWORD ccl;

			/* Follow parent directories toward the root directory and create the cwd path */
			i = len;			/* Bottom of buffer (directory stack base) */
			dj.obj.sclust = fs->cdir;				/* Start to follow upper directory from current directory */
			while ((ccl = dj.obj.sclust) != 0) {	/* Repeat while current directory is a sub-directory */
				res = dir_sdi(&dj, 1 * SZDIRE);		/* Get parent directory */
				if (res != FR_OK) {
					break;
				}
				res = move_window(fs, dj.sect);
				if (res != FR_OK) {
					break;
				}
				dj.obj.sclust = ld_clust(fs, dj.dir);	/* Go to parent directory */
				res = dir_sdi(&dj, 0);
				if (res != FR_OK) {
					break;
				}
				do {								/* Find the entry links to this sub-directory */
					res = DIR_READ_FILE(&dj);
					if (res != FR_OK) {
						break;
					}
					if (ccl == ld_clust(fs, dj.dir)) {
						break;        /* Found the entry */
					}
					res = dir_next(&dj, 0);
				}
				while (res == FR_OK);
				if (res == FR_NO_FILE) {
					res = FR_INT_ERR;		/* It cannot be 'not found'. */
				}
				if (res != FR_OK) {
					break;
				}
				get_fileinfo(&dj, &fno);			/* Get the directory name and push it to the buffer */
				for (nl = 0; fno.fname[nl]; nl++) ;	/* Name length */
				if (i < nl + 1) {	/* Insufficient space to store the path name? */
					res = FR_NOT_ENOUGH_CORE;
					break;
				}
				while (n) {
					buff[--i] = fno.fname[--nl];	/* Stack the name */
				}
				buff[--i] = '/';
			}
			if (res == FR_OK) {
				if (i == len) {
					buff[--i] = '/';	/* Is it the root-directory? */
				}
#if FF_VOLUMES >= 2			/* Put drive prefix */
				vl = 0;
#if FF_STR_VOLUME_ID >= 1	/* String volume ID */
				for (nl = 0, vid = (const char*)VolumeStr[CurrVol]; vid[nl]; nl++) ;	/* Volume ID length */
				if (i >= nl + 2) {
					if (FF_STR_VOLUME_ID == 2) {
						*tp++ = '/';	/* Unix style */
					}
					for (vl = 0; vl < nl; *tp++ = vid[vl], vl++) ;
					if (FF_STR_VOLUME_ID == 1) {
						*tp++ = ':';	/* DOS/Windows style */
					}
					vl++;
				}
#else						/* Numeric volume ID */
				if (i >= 3) {
					*tp++ = '0' + CurrVol;
					*tp++ = ':';
					vl = 2;
				}
#endif
				if (vl == 0) {
					res = FR_NOT_ENOUGH_CORE;
				}
#endif
				/* Add current directory path */
				if (res == FR_OK) {
					do {	/* Copy stacked path string */
						*tp++ = buff[i++];
					} while (i < len);
				}
			}
			*tp = 0;
		}
		FREE_NAMEBUFF();
	}

	LEAVE_FF(fs, res);
}

#endif /* FF_FS_RPATH >= 2 */
#endif /* FF_FS_RPATH >= 1 */



#if FF_FS_MINIMIZE <= 2
/*-----------------------------------------------------------------------*/
/* API: Seek File Read/Write Pointer                                     */
/*-----------------------------------------------------------------------*/

FRESULT f_lseek (
	FIL *fp,		/* Pointer to the file object */
	FSIZE_t ofs		/* File pointer from top of file */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;
	DWORD clst, bcs;
	LBA_t nsect;
	FSIZE_t ifptr;


	res = validate(&fp->obj, &fs);		/* Check validity of the file object */
	if (res == FR_OK) {
		res = (FRESULT)fp->err;
	}
#if FF_FS_EXFAT && !FF_FS_READONLY
	if (res == FR_OK && fs->fs_type == FS_EXFAT) {
		res = fill_last_frag(&fp->obj, fp->clust, 0xFFFFFFFF);	/* Fill last fragment on the FAT if needed */
	}
#endif
	if (res != FR_OK) {
		LEAVE_FF(fs, res);
	}

#if FF_USE_FASTSEEK
	if (fp->cltbl) {	/* Fast seek */
		DWORD cl, pcl, ncl, tcl, tlen, ulen;
		DWORD *tbl;
		LBA_t dsc;

		if (ofs == CREATE_LINKMAP) {	/* Create CLMT */
			tbl = fp->cltbl;
			tlen = *tbl++;
			ulen = 2;	/* Given table size and required table size */
			cl = fp->obj.sclust;		/* Origin of the chain */
			if (cl != 0) {
				do {
					/* Get a fragment */
					tcl = cl;
					ncl = 0;
					ulen += 2;	/* Top, length and used items */
					do {
						pcl = cl;
						ncl++;
						cl = get_fat(&fp->obj, cl);
						if (cl <= 1) {
							ABORT(fs, FR_INT_ERR);
						}
						if (cl == 0xFFFFFFFF) {
							ABORT(fs, FR_DISK_ERR);
						}
					}
					while (cl == pcl + 1);
					if (ulen <= tlen) {		/* Store the length and top of the fragment */
						*tbl++ = ncl;
						*tbl++ = tcl;
					}
				}
				while (cl < fs->n_fatent);	/* Repeat until end of chain */
			}
			*fp->cltbl = ulen;	/* Number of items used */
			if (ulen <= tlen) {
				*tbl = 0;		/* Terminate table */
			}
			else {
				res = FR_NOT_ENOUGH_CORE;	/* Given table size is smaller than required */
			}
		}
		else {						/* Fast seek */
			if (ofs > fp->obj.objsize) {
				ofs = fp->obj.objsize;        /* Clip offset at the file size */
			}
			fp->fptr = ofs;				/* Set file pointer */
			if (ofs > 0) {
				fp->clust = clmt_clust(fp, ofs - 1);
				dsc = clst2sect(fs, fp->clust);
				if (dsc == 0) {
					ABORT(fs, FR_INT_ERR);
				}
				dsc += (DWORD)((ofs - 1) / SS(fs)) & (fs->csize - 1);
				if (fp->fptr % SS(fs) && dsc != fp->sect) {	/* Refill sector cache if needed */
#if !FF_FS_TINY
#if !FF_FS_READONLY
					if (fp->flag & FA_DIRTY) {		/* Write-back dirty sector cache */
						if (disk_write(fs->pdrv, fp->buf, fp->sect, 1) != RES_OK) {
							ABORT(fs, FR_DISK_ERR);
						}
						fp->flag &= (BYTE)~FA_DIRTY;
					}
#endif
					if (disk_read(fs->pdrv, fp->buf, dsc, 1) != RES_OK) {
						ABORT(fs, FR_DISK_ERR);        /* Load current sector */
					}
#endif
					fp->sect = dsc;
				}
			}
		}
	}
	else
#endif

		/* Normal Seek */
	{
#if FF_FS_EXFAT
		if (fs->fs_type != FS_EXFAT && ofs >= 0x100000000) {
			ofs = 0xFFFFFFFF;        /* Clip at 4 GiB - 1 if at FATxx */
		}
#endif
		if (ofs > fp->obj.objsize && (FF_FS_READONLY
					      || !(fp->flag & FA_WRITE))) {	/* In read-only mode, clip offset with the file size */
			ofs = fp->obj.objsize;
		}
		ifptr = fp->fptr;
		fp->fptr = nsect = 0;
		if (ofs > 0) {
			bcs = (DWORD)fs->csize * SS(fs);	/* Cluster size (byte) */
			if (ifptr > 0 &&
			    (ofs - 1) / bcs >= (ifptr - 1) / bcs) {	/* When seek to same or following cluster, */
				fp->fptr = (ifptr - 1) & ~(FSIZE_t)(bcs - 1);	/* start from the current cluster */
				ofs -= fp->fptr;
				clst = fp->clust;
			}
			else {									/* When seek to back cluster, */
				clst = fp->obj.sclust;					/* start from the first cluster */
#if !FF_FS_READONLY
				if (clst == 0) {						/* If no cluster chain, create a new chain */
					clst = create_chain(&fp->obj, 0);
					if (clst == 1) {
						ABORT(fs, FR_INT_ERR);
					}
					if (clst == 0xFFFFFFFF) {
						ABORT(fs, FR_DISK_ERR);
					}
					fp->obj.sclust = clst;
				}
#endif
				fp->clust = clst;
			}
			if (clst != 0) {
				while (ofs > bcs) {						/* Cluster following loop */
					ofs -= bcs;
					fp->fptr += bcs;
#if !FF_FS_READONLY
					if (fp->flag & FA_WRITE) {			/* Check if in write mode or not */
						if (FF_FS_EXFAT && fp->fptr > fp->obj.objsize) {	/* No FAT chain object needs correct objsize to generate FAT value */
							fp->obj.objsize = fp->fptr;
							fp->flag |= FA_MODIFIED;
						}
						clst = create_chain(&fp->obj, clst);	/* Follow chain with forceed stretch */
						if (clst == 0) {				/* Clip file size in case of disk full */
							ofs = 0;
							break;
						}
					}
					else
#endif
					{
						clst = get_fat(&fp->obj, clst);	/* Follow cluster chain if not in write mode */
					}
					if (clst == 0xFFFFFFFF) {
						ABORT(fs, FR_DISK_ERR);
					}
					if (clst <= 1 || clst >= fs->n_fatent) {
						ABORT(fs, FR_INT_ERR);
					}
					fp->clust = clst;
				}
				fp->fptr += ofs;
				if (ofs % SS(fs)) {
					nsect = clst2sect(fs, clst);	/* Current sector */
					if (nsect == 0) {
						ABORT(fs, FR_INT_ERR);
					}
					nsect += (DWORD)(ofs / SS(fs));
				}
			}
		}
		if (!FF_FS_READONLY && fp->fptr > fp->obj.objsize) {	/* Set file change flag if the file size is extended */
			fp->obj.objsize = fp->fptr;
			fp->flag |= FA_MODIFIED;
		}
		if (fp->fptr % SS(fs) && nsect != fp->sect) {	/* Fill sector cache if needed */
#if !FF_FS_TINY
#if !FF_FS_READONLY
			if (fp->flag & FA_DIRTY) {			/* Write-back dirty sector cache */
				if (disk_write(fs->pdrv, fp->buf, fp->sect, 1) != RES_OK) {
					ABORT(fs, FR_DISK_ERR);
				}
				fp->flag &= (BYTE)~FA_DIRTY;
			}
#endif
			if (disk_read(fs->pdrv, fp->buf, nsect, 1) != RES_OK) {
				ABORT(fs, FR_DISK_ERR);        /* Fill sector cache */
			}
#endif
			fp->sect = nsect;
		}
	}

	LEAVE_FF(fs, res);
}



#if FF_FS_MINIMIZE <= 1
/*-----------------------------------------------------------------------*/
/* API: Create a Directory Object                                        */
/*-----------------------------------------------------------------------*/

FRESULT f_opendir (
	DIR *dp,			/* Pointer to directory object to create */
	const TCHAR *path	/* Pointer to the directory path */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;
	DEF_NAMEBUFF


	if (!dp) {
		return FR_INVALID_OBJECT;	/* Reject null pointer */
	}

	res = mount_volume(&path, &fs, 0);	/* Get logical drive and mount the volume if needed */
	if (res == FR_OK) {
		dp->obj.fs = fs;
		INIT_NAMEBUFF(fs);
		res = follow_path(dp, path);			/* Follow the path to the directory */
		if (res == FR_OK) {						/* Follow completed */
			if (!(dp->fn[NSFLAG] & NS_NONAME)) {	/* It is neither the origin directory itself nor dot name in exFAT */
				if (dp->obj.attr & AM_DIR) {		/* This object is a sub-directory */
#if FF_FS_EXFAT
					if (fs->fs_type == FS_EXFAT) {
						init_alloc_info(&dp->obj, dp);	/* Get object allocation info */
					}
					else
#endif
					{
						dp->obj.sclust = ld_clust(fs, dp->dir);	/* Get object allocation info */
					}
				}
				else {						/* This object is a file */
					res = FR_NO_PATH;
				}
			}
			if (res == FR_OK) {
				dp->obj.id = fs->id;		/* Set current volume mount ID */
				res = dir_sdi(dp, 0);		/* Rewind directory */
#if FF_FS_LOCK
				if (res == FR_OK) {
					if (dp->obj.sclust) {	/* Is this a sub-directory? */
						dp->obj.lockid = inc_share(dp, 0);	/* Lock the sub-directory */
						if (!dp->obj.lockid) {
							res = FR_TOO_MANY_OPEN_FILES;
						}
					}
					else {
						dp->obj.lockid = 0;	/* Root directory does not need to be locked */
					}
				}
#endif
			}
		}
		FREE_NAMEBUFF();
		if (res == FR_NO_FILE) {
			res = FR_NO_PATH;
		}
	}
	if (res != FR_OK) {
		dp->obj.fs = 0;        /* Invalidate the directory object if function failed */
	}

	LEAVE_FF(fs, res);
}




/*-----------------------------------------------------------------------*/
/* API: Close Directory                                                  */
/*-----------------------------------------------------------------------*/

FRESULT f_closedir (
	DIR *dp		/* Pointer to the directory object to be closed */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;


	res = validate(&dp->obj, &fs);	/* Check validity of the file object */
	if (res == FR_OK) {
#if FF_FS_LOCK
		if (dp->obj.lockid) {
			res = dec_share(dp->obj.lockid);        /* Decrement sub-directory open counter */
		}
		if (res == FR_OK) {
			dp->obj.fs = 0;        /* Invalidate directory object */
		}
#else
		dp->obj.fs = 0;	/* Invalidate directory object */
#endif
#if FF_FS_REENTRANT
		unlock_volume(fs, FR_OK);	/* Unlock volume */
#endif
	}
	return res;
}




/*-----------------------------------------------------------------------*/
/* API: Read Directory Entries in Sequence                               */
/*-----------------------------------------------------------------------*/

FRESULT f_readdir (
	DIR *dp,			/* Pointer to the open directory object */
	FILINFO *fno		/* Pointer to file information to return */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;
	DEF_NAMEBUFF


	res = validate(&dp->obj, &fs);	/* Check validity of the directory object */
	if (res == FR_OK) {
		if (!fno) {
			res = dir_sdi(dp, 0);		/* Rewind the directory object */
		}
		else {
			INIT_NAMEBUFF(fs);
			fno->fname[0] = 0;				/* Clear file information */
			res = DIR_READ_FILE(dp);		/* Read an item */
			if (res == FR_NO_FILE) {
				res = FR_OK;        /* Ignore end of directory */
			}
			if (res == FR_OK) {				/* A valid entry is found */
				get_fileinfo(dp, fno);		/* Get the object information */
				res = dir_next(dp, 0);		/* Increment index for next */
				if (res == FR_NO_FILE) {
					res = FR_OK;        /* Ignore end of directory now */
				}
			}
			FREE_NAMEBUFF();
		}
	}

	if (fno && res != FR_OK) {
		fno->fname[0] = 0;	/* Clear the file information if any error occurred */
	}
	LEAVE_FF(fs, res);
}



#if FF_USE_FIND
/*-----------------------------------------------------------------------*/
/* API: Find Next File                                                   */
/*-----------------------------------------------------------------------*/

FRESULT f_findnext (
	DIR *dp,		/* Pointer to the open directory object */
	FILINFO *fno	/* Pointer to the file information structure */
)
{
	FRESULT res = FR_DISK_ERR;


	for (;;) {
		res = f_readdir(dp, fno);		/* Get a directory item */
		if (res != FR_OK || !fno || !fno->fname[0]) {
			break;        /* Terminate if any error or end of directory */
		}
		if (pattern_match(dp->pat, fno->fname, 0, FIND_RECURS)) {
			break;        /* Test for the file name */
		}
#if FF_USE_LFN && FF_USE_FIND == 2
		if (pattern_match(dp->pat, fno->altname, 0, FIND_RECURS)) {
			break;        /* Test for alternative name if exist */
		}
#endif
	}
	return res;
}



/*-----------------------------------------------------------------------*/
/* API: Find First File                                                  */
/*-----------------------------------------------------------------------*/

FRESULT f_findfirst (
	DIR *dp,				/* Pointer to the blank directory object */
	FILINFO *fno,			/* Pointer to the file information structure */
	const TCHAR *path,		/* Pointer to the directory to open */
	const TCHAR *pattern	/* Pointer to the matching pattern */
)
{
	FRESULT res = FR_DISK_ERR;


	dp->pat = pattern;		/* Save pointer to pattern string */
	res = f_opendir(dp, path);		/* Open the target directory */
	if (res == FR_OK) {
		res = f_findnext(dp, fno);	/* Find the first item */
	}
	return res;
}

#endif	/* FF_USE_FIND */



#if FF_FS_MINIMIZE == 0
/*-----------------------------------------------------------------------*/
/* API: Get File Status                                                  */
/*-----------------------------------------------------------------------*/

FRESULT f_stat (
	const TCHAR *path,	/* Pointer to the file path */
	FILINFO *fno		/* Pointer to file information to return */
)
{
	FRESULT res = FR_DISK_ERR;
	DIR dj = {0};
	DEF_NAMEBUFF


	/* Get logical drive and mount the volume if needed */
	res = mount_volume(&path, &dj.obj.fs, 0);
	if (res == FR_OK) {
		INIT_NAMEBUFF(dj.obj.fs);
		res = follow_path(&dj, path);	/* Follow the file path */
		if (res == FR_OK) {				/* Follow completed */
			if (dj.fn[NSFLAG] & NS_NONAME) {	/* It is origin directory */
				res = FR_INVALID_NAME;
			}
			else {							/* Found an object */
				if (fno) {
					get_fileinfo(&dj, fno);
				}
			}
		}
		FREE_NAMEBUFF();
	}

	if (fno && res != FR_OK) {
		fno->fname[0] = 0;	/* Invalidate the file information if an error occurred */
	}
	LEAVE_FF(dj.obj.fs, res);
}



#if !FF_FS_READONLY
/*-----------------------------------------------------------------------*/
/* API: Get Number of Free Clusters                                      */
/*-----------------------------------------------------------------------*/

FRESULT f_getfree (
	const TCHAR *path,	/* Logical drive number */
	DWORD *nclst,		/* Pointer to a variable to return number of free clusters */
	FATFS **fatfs		/* Pointer to a pointer to return corresponding filesystem object */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;
	DWORD nfree, clst, stat;
	LBA_t sect;
	UINT i;
	FFOBJID obj;


	/* Get logical drive and mount the volume if needed */
	res = mount_volume(&path, &fs, 0);
	if (res == FR_OK) {
		*fatfs = fs;				/* Return ptr to the fs object */
		/* If free_clst is valid, return it without full FAT scan */
		if (fs->free_clst <= fs->n_fatent - 2) {
			*nclst = fs->free_clst;
		}
		else {
			/* Scan FAT to obtain the correct free cluster count */
			nfree = 0;
			if (fs->fs_type == FS_FAT12) {	/* FAT12: Scan bit field FAT entries */
				clst = 2;
				obj.fs = fs;
				do {
					stat = get_fat(&obj, clst);
					if (stat == 0xFFFFFFFF) {
						res = FR_DISK_ERR;
						break;
					}
					if (stat == 1) {
						res = FR_INT_ERR;
						break;
					}
					if (stat == 0) {
						nfree++;
					}
				}
				while (++clst < fs->n_fatent);
			}
			else {
#if FF_FS_EXFAT
				if (fs->fs_type == FS_EXFAT) {	/* exFAT: Scan allocation bitmap */
					BYTE bm;
					UINT b;

					clst = fs->n_fatent - 2;	/* Number of clusters */
					sect = fs->bitbase;			/* Bitmap sector */
					i = 0;						/* Offset in the sector */
					do {	/* Counts numbuer of clear bits (free clusters) with zero in the bitmap */
						if (i == 0) {	/* New sector? */
							res = move_window(fs, sect++);
							if (res != FR_OK) {
								break;
							}
						}
						for (b = 8, bm = ~fs->win[i]; b && clst; b--, clst--) {	/* Count clear bits in a byte */
							nfree += bm & 1;
							bm >>= 1;
						}
						i = (i + 1) % SS(fs);	/* Next byte */
					}
					while (clst);
				}
				else
#endif
				{	/* FAT16/32: Scan WORD/DWORD FAT entries */
					clst = fs->n_fatent;	/* Number of entries */
					sect = fs->fatbase;		/* Top of the FAT */
					i = 0;					/* Offset in the sector */
					do {	/* Counts numbuer of entries with zero in the FAT */
						if (i == 0) {	/* New sector? */
							res = move_window(fs, sect++);
							if (res != FR_OK) {
								break;
							}
						}
						if (fs->fs_type == FS_FAT16) {
							if (ld_16(fs->win + i) == 0) {	/* FAT16: Is this cluster free? */
								nfree++;
							}
							i += 2;	/* Next entry */
						}
						else {
							if ((ld_32(fs->win + i) & 0x0FFFFFFF) == 0) {	/* FAT32: Is this cluster free? */
								nfree++;
							}
							i += 4;	/* Next entry */
						}
						i %= SS(fs);
					}
					while (--clst);
				}
			}
			if (res == FR_OK) {		/* Update parameters if succeeded */
				*nclst = nfree;			/* Return the free clusters */
				fs->free_clst = nfree;	/* Now free cluster count is valid */
				fs->fsi_flag |= 1;		/* FAT32/exfAT : Allocation information is to be updated */
			}
		}
	}

	LEAVE_FF(fs, res);
}




/*-----------------------------------------------------------------------*/
/* API: Truncate File                                                    */
/*-----------------------------------------------------------------------*/

FRESULT f_truncate (
	FIL *fp		/* Pointer to the file object */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;
	DWORD ncl;


	/* Check validity of the file object */
	res = validate(&fp->obj, &fs);
	if (res != FR_OK || (res = (FRESULT)fp->err) != FR_OK) {
		LEAVE_FF(fs, res);
	}
	if (!(fp->flag & FA_WRITE)) {
		LEAVE_FF(fs, FR_DENIED);        /* Check access mode */
	}

	if (fp->fptr < fp->obj.objsize) {	/* Process when fptr is not on the eof */
		if (fp->fptr == 0) {	/* When set file size to zero, remove entire cluster chain */
			res = remove_chain(&fp->obj, fp->obj.sclust, 0);
			fp->obj.sclust = 0;
		}
		else {				/* When truncate a part of the file, remove remaining clusters */
			ncl = get_fat(&fp->obj, fp->clust);
			res = FR_OK;
			if (ncl == 0xFFFFFFFF) {
				res = FR_DISK_ERR;
			}
			if (ncl == 1) {
				res = FR_INT_ERR;
			}
			if (res == FR_OK && ncl < fs->n_fatent) {
				res = remove_chain(&fp->obj, ncl, fp->clust);
			}
		}
		fp->obj.objsize = fp->fptr;	/* Set file size to current read/write point */
		fp->flag |= FA_MODIFIED;
#if !FF_FS_TINY
		if (res == FR_OK && (fp->flag & FA_DIRTY)) {
			if (disk_write(fs->pdrv, fp->buf, fp->sect, 1) != RES_OK) {
				res = FR_DISK_ERR;
			}
			else {
				fp->flag &= (BYTE)~FA_DIRTY;
			}
		}
#endif
		if (res != FR_OK) {
			ABORT(fs, res);
		}
	}

	LEAVE_FF(fs, res);
}




/*-----------------------------------------------------------------------*/
/* API: Delete a File/Directory                                          */
/*-----------------------------------------------------------------------*/

FRESULT f_unlink (
	const TCHAR *path		/* Pointer to the file or directory path */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;
	DIR dj, sdj;
	DWORD dclst = 0;
#if FF_FS_EXFAT
	FFOBJID obj;
#endif
	DEF_NAMEBUFF

	/* Get logical drive and mount the volume if needed */
	res = mount_volume(&path, &fs, FA_WRITE);
	if (res == FR_OK) {
		dj.obj.fs = fs;
		INIT_NAMEBUFF(fs);
		res = follow_path(&dj, path);	/* Follow the path to the object */
		if (res == FR_OK) {
			if (dj.fn[NSFLAG] & (NS_DOT | NS_NONAME)) {
				res = FR_INVALID_NAME;	/* It must be a real object */
			} else if (dj.obj.attr & AM_RDO) {
				res = FR_DENIED;		/* The object must not be read-only */
#if FF_FS_LOCK
			} else {
				res = chk_share(&dj, 2);	/* Check if the object is in use */
#endif
			}
		}
		if (res == FR_OK) {		/* The object is accessible */
#if FF_FS_EXFAT
			obj.fs = fs;
			if (fs->fs_type == FS_EXFAT) {
				init_alloc_info(&obj, 0);
				dclst = obj.sclust;
			} else
#endif
			{
				dclst = ld_clust(fs, dj.dir);
			}
			if (dj.obj.attr & AM_DIR) {		/* Is the object a sub-directory? */
#if FF_FS_RPATH
				if (dclst == fs->cdir) {
					res = FR_DENIED;		/* Current directory cannot be removed */
				}
				else
#endif
				{
					sdj.obj.fs = fs;		/* Open the sub-directory */
					sdj.obj.sclust = dclst;
#if FF_FS_EXFAT
					if (fs->fs_type == FS_EXFAT) {
						sdj.obj.objsize = obj.objsize;
						sdj.obj.stat = obj.stat;
					}
#endif
					res = dir_sdi(&sdj, 0);
					if (res == FR_OK) {
						res = DIR_READ_FILE(&sdj);			/* Check if the sub-directory is empty */
						if (res == FR_OK) {
							res = FR_DENIED;	/* Not empty? */
						}
						if (res == FR_NO_FILE) {
							res = FR_OK;	/* Empty? */
						}
					}
				}
			}
		}
		if (res == FR_OK) {		/* It is ready to remove the object */
			res = dir_remove(&dj);				/* Remove the directory entry */
			if (res == FR_OK && dclst != 0) {	/* Remove the cluster chain if exist */
#if FF_FS_EXFAT
				res = remove_chain(&obj, dclst, 0);
#else
				res = remove_chain(&dj.obj, dclst, 0);
#endif
			}
			if (res == FR_OK) {
				res = sync_fs(fs);
			}
		}
		FREE_NAMEBUFF();
	}

	LEAVE_FF(fs, res);
}




/*-----------------------------------------------------------------------*/
/* API: Create a Directory                                               */
/*-----------------------------------------------------------------------*/

FRESULT f_mkdir (
	const TCHAR *path		/* Pointer to the directory path */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;
	DIR dj;
	FFOBJID sobj;
	DWORD dcl, pcl, tm;
	DEF_NAMEBUFF


	res = mount_volume(&path, &fs, FA_WRITE);	/* Get logical drive and mount the volume if needed */
	if (res == FR_OK) {
		dj.obj.fs = fs;
		INIT_NAMEBUFF(fs);
		res = follow_path(&dj, path);			/* Follow the file path */
		if (res == FR_OK) {						/* Invalid name or name collision */
			res = (dj.fn[NSFLAG] & (NS_DOT | NS_NONAME)) ? FR_INVALID_NAME : FR_EXIST;
		}
		if (res == FR_NO_FILE) {				/* It is clear to create a new directory */
			sobj.fs = fs;						/* New object ID to create a new chain */
			dcl = create_chain(&sobj, 0);		/* Allocate a cluster for the new directory */
			res = FR_OK;
			if (dcl == 0) {
				res = FR_DENIED;        /* No space to allocate a new cluster? */
			}
			if (dcl == 1) {
				res = FR_INT_ERR;        /* Any insanity? */
			}
			if (dcl == 0xFFFFFFFF) {
				res = FR_DISK_ERR;        /* Disk error? */
			}
			tm = GET_FATTIME();
			if (res == FR_OK) {
				res = dir_clear(fs, dcl);		/* Clear the allocated cluster as new directory table */
				if (res == FR_OK) {
					if (!FF_FS_EXFAT || fs->fs_type != FS_EXFAT) {	/* Create dot entries (FAT only) */
						memset(fs->win + DIR_Name, ' ', 11);	/* Create "." entry */
						fs->win[DIR_Name] = '.';
						fs->win[DIR_Attr] = AM_DIR;
						st_32(fs->win + DIR_ModTime, tm);
						st_clust(fs, fs->win, dcl);
						mem_cpy(fs->win + SZDIRE, fs->win, SZDIRE);	/* Create ".." entry */
						fs->win[SZDIRE + 1] = '.';
						pcl = dj.obj.sclust;
						st_clust(fs, fs->win + SZDIRE, pcl);
						fs->wflag = 1;
					}
					res = dir_register(&dj);	/* Register the object to the parent directory */
				}
			}
			if (res == FR_OK) {
#if FF_FS_EXFAT
				if (fs->fs_type == FS_EXFAT) {	/* Initialize directory entry block */
					st_32(fs->dirbuf + XDIR_CrtTime, tm);	/* Created time */
					st_32(fs->dirbuf + XDIR_ModTime, tm);
					st_32(fs->dirbuf + XDIR_FstClus, dcl);	/* Table start cluster */
					st_32(fs->dirbuf + XDIR_FileSize, (DWORD)fs->csize * SS(fs));	/* Directory size needs to be valid */
					st_32(fs->dirbuf + XDIR_ValidFileSize, (DWORD)fs->csize * SS(fs));
					fs->dirbuf[XDIR_GenFlags] = 3;			/* Initialize the object flag */
					fs->dirbuf[XDIR_Attr] = AM_DIR;			/* Attribute */
					res = store_xdir(&dj);
				}
				else
#endif
				{
					st_32(dj.dir + DIR_CrtTime, tm);	/* Created time */
					st_32(dj.dir + DIR_ModTime, tm);
					st_clust(fs, dj.dir, dcl);			/* Table start cluster */
					dj.dir[DIR_Attr] = AM_DIR;			/* Attribute */
					fs->wflag = 1;
				}
				if (res == FR_OK) {
					res = sync_fs(fs);
				}
			}
			else {
				(void)remove_chain(&sobj, dcl, 0);		/* Could not register, remove the allocated cluster */
			}
		}
		FREE_NAMEBUFF();
	}

	LEAVE_FF(fs, res);
}




/*-----------------------------------------------------------------------*/
/* API: Rename a File/Directory                                          */
/*-----------------------------------------------------------------------*/

FRESULT f_rename (
	const TCHAR *path_old,	/* Pointer to the object name to be renamed */
	const TCHAR *path_new	/* Pointer to the new name */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;
	DIR djo, djn;
	BYTE buf[FF_FS_EXFAT ? SZDIRE * 2 : SZDIRE], *dir;
	DEF_NAMEBUFF


	get_ldnumber(&path_new);	/* Snip the drive number of new name off */
	res = mount_volume(&path_old, &fs, FA_WRITE);	/* Get logical drive of the old object */
	if (res == FR_OK) {
		djo.obj.fs = fs;
		INIT_NAMEBUFF(fs);
		res = follow_path(&djo, path_old);	/* Check old object */
		if (res == FR_OK) {
			if (djo.fn[NSFLAG] & (NS_DOT | NS_NONAME)) {
				res = FR_INVALID_NAME;		/* Object must not be a dot name or blank name */
#if FF_FS_LOCK
			} else {
				res = chk_share(&djo, 2);	/* Check if the object is in use */
#endif
			}
		}
		if (res == FR_OK) {					/* It is ready to rename the object */
#if FF_FS_EXFAT
			if (fs->fs_type == FS_EXFAT) {	/* At exFAT volume */
#if FF_FS_RPATH
				UINT i;
				DWORD dscl = ld_32(fs->dirbuf + XDIR_FstClus);

				for (i = 1; i <= fs->xcwds.depth && dscl != fs->xcwds.tbl[i].d_scl; i++) ;	/* Check if the object is a sub-dir in the current dir path */
				if (i <= fs->xcwds.depth) {
					res = FR_DENIED;	/* Reject to rename a sub-dir in the current dir path */
				} else
#endif
				{
					mem_cpy(buf, fs->dirbuf, SZDIRE * 2);	/* Save 85+C0 entry of old object */
					mem_cpy(&djn, &djo, (UINT)(sizeof djn));
					res = follow_path(&djn, path_new);		/* Check if new object name collides with an existing one */
				}
				if (res == FR_OK) {					/* Is new name already in use by another object? */
					res = (djn.obj.sclust == djo.obj.sclust && djn.dptr == djo.dptr) ? FR_NO_FILE : FR_EXIST;
				}
				if (res == FR_NO_FILE) { 			/* It is a valid path and no name collision */
					res = dir_register(&djn);		/* Register the new entry */
					if (res == FR_OK) {
						BYTE nf, nn;
						WORD nh;

						nf = fs->dirbuf[XDIR_NumSec];	/* Save name length and hash */
						nn = fs->dirbuf[XDIR_NumName];
						nh = ld_16(fs->dirbuf + XDIR_NameHash);
						mem_cpy(fs->dirbuf, buf, SZDIRE * 2);	/* Restore 85+C0 entry */
						fs->dirbuf[XDIR_NumSec] = nf;	/* Restore name length and hash */
						fs->dirbuf[XDIR_NumName] = nn;
						st_16(fs->dirbuf + XDIR_NameHash, nh);
						if (!(fs->dirbuf[XDIR_Attr] & AM_DIR)) {
							fs->dirbuf[XDIR_Attr] |= AM_ARC;        /* Set archive attribute if it is a file */
						}
						/* Start of critical section where an interruption can cause a cross-link */
						res = store_xdir(&djn);
					}
				}
			}
			else
#endif
			{	/* At FAT/FAT32 volume */
				mem_cpy(buf, djo.dir, SZDIRE);			/* Save directory entry of the object */
				mem_cpy(&djn, &djo, (UINT)(sizeof djn));	/* Duplicate the directory object */
				res = follow_path(&djn, path_new);		/* Make sure if new object name is not in use */
				if (res == FR_OK) {						/* Is new name already in use by another object? */
					res = (djn.obj.sclust == djo.obj.sclust && djn.dptr == djo.dptr) ? FR_NO_FILE : FR_EXIST;
				}
				if (res == FR_NO_FILE) { 				/* It is a valid path and no name collision */
					res = dir_register(&djn);			/* Register the new entry */
					if (res == FR_OK) {
						dir = djn.dir;					/* Copy directory entry of the object except name */
						mem_cpy(dir + 13, buf + 13, SZDIRE - 13);
						dir[DIR_Attr] = buf[DIR_Attr];
						if (!(dir[DIR_Attr] & AM_DIR)) {
							dir[DIR_Attr] |= AM_ARC;        /* Set archive attribute if it is a file */
						}
						fs->wflag = 1;
						if ((dir[DIR_Attr] & AM_DIR) && djo.obj.sclust != djn.obj.sclust) {	/* Update .. entry in the sub-directory being moved if needed */
							LBA_t sect = clst2sect(fs, ld_clust(fs, dir));

							if (sect == 0) {
								res = FR_INT_ERR;
							}
							else {
								/* Start of critical section where an interruption can cause a cross-link */
								res = move_window(fs, sect);
								dir = fs->win + SZDIRE * 1;	/* Pointer to .. entry */
								if (res == FR_OK && dir[1] == '.') {
									st_clust(fs, dir, djn.obj.sclust);
									fs->wflag = 1;
								}
							}
						}
					}
				}
			}
			if (res == FR_OK) {		/* New entry has been created */
				res = dir_remove(&djo);	/* Remove old entry */
				if (res == FR_OK) {
					res = sync_fs(fs);
				}
			}
			/* End of the critical section */
		}
		FREE_NAMEBUFF();
	}

	LEAVE_FF(fs, res);
}

#endif /* !FF_FS_READONLY */
#endif /* FF_FS_MINIMIZE == 0 */
#endif /* FF_FS_MINIMIZE <= 1 */
#endif /* FF_FS_MINIMIZE <= 2 */



#if FF_USE_CHMOD && !FF_FS_READONLY
/*-----------------------------------------------------------------------*/
/* API: Change Attribute                                                 */
/*-----------------------------------------------------------------------*/

FRESULT f_chmod (
	const TCHAR *path,	/* Pointer to the file path */
	BYTE attr,			/* Attribute bits to set/clear */
	BYTE mask			/* Attribute mask to change */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;
	DIR dj;
	DEF_NAMEBUFF


	/* Get logical drive and mount the volume if needed */
	res = mount_volume(&path, &fs, FA_WRITE);

	if (res == FR_OK) {
		dj.obj.fs = fs;
		INIT_NAMEBUFF(fs);
		res = follow_path(&dj, path);	/* Follow the file path */
		if (res == FR_OK && (dj.fn[NSFLAG] & (NS_DOT | NS_NONAME))) {
			res = FR_INVALID_NAME;        /* Check object validity */
		}
		if (res == FR_OK) {
			mask &= AM_RDO | AM_HID | AM_SYS | AM_ARC;	/* Valid attribute mask */
#if FF_FS_EXFAT
			if (fs->fs_type == FS_EXFAT) {
				fs->dirbuf[XDIR_Attr] = (attr & mask) | (fs->dirbuf[XDIR_Attr] & (BYTE)~mask);	/* Apply attribute change */
				res = store_xdir(&dj);
			}
			else
#endif
			{
				dj.dir[DIR_Attr] = (attr & mask) | (dj.dir[DIR_Attr] & (BYTE)~mask);	/* Apply attribute change */
				fs->wflag = 1;
			}
			if (res == FR_OK) {
				res = sync_fs(fs);
			}
		}
		FREE_NAMEBUFF();
	}

	LEAVE_FF(fs, res);
}




/*-----------------------------------------------------------------------*/
/* API: Change Timestamp                                                 */
/*-----------------------------------------------------------------------*/

FRESULT f_utime (
	const TCHAR *path,	/* Pointer to the file/directory name */
	const FILINFO* fno	/* Timestamp to be set */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;
	DIR dj;
	DEF_NAMEBUFF


	/* Get logical drive and mount the volume if needed */
	res = mount_volume(&path, &fs, FA_WRITE);

	if (res == FR_OK) {
		dj.obj.fs = fs;
		INIT_NAMEBUFF(fs);
		res = follow_path(&dj, path);	/* Follow the file path */
		if (res == FR_OK && (dj.fn[NSFLAG] & (NS_DOT | NS_NONAME))) {
			res = FR_INVALID_NAME;        /* Check object validity */
		}
		if (res == FR_OK) {
#if FF_FS_EXFAT
			if (fs->fs_type == FS_EXFAT) {	/* On the exFAT volume */
				if (fno->fdate) {	/* Change last modified time if needed */
					st_32(fs->dirbuf + XDIR_ModTime, (DWORD)fno->fdate << 16 | fno->ftime);
					fs->dirbuf[XDIR_ModTime10] = 0;
					fs->dirbuf[XDIR_ModTZ] = 0;
				}
#if FF_FS_CRTIME
				if (fno->crdate) {	/* Change created time if needed */
					st_32(fs->dirbuf + XDIR_CrtTime, (DWORD)fno->crdate << 16 | fno->crtime);
					fs->dirbuf[XDIR_CrtTime10] = 0;
					fs->dirbuf[XDIR_CrtTZ] = 0;
				}
#endif
				res = store_xdir(&dj);
			}
			else
#endif
			{	/* On the FAT volume */
				if (fno->fdate) {	/* Change last modified time if needed */
					st_32(dj.dir + DIR_ModTime, (DWORD)fno->fdate << 16 | fno->ftime);
				}
#if FF_FS_CRTIME
				if (fno->crdate) {	/* Change created time if needed */
					st_32(dj.dir + DIR_CrtTime, (DWORD)fno->crdate << 16 | fno->crtime);
					dj.dir[DIR_CrtTime10] = 0;
				}
#endif
				fs->wflag = 1;
			}
			if (res == FR_OK) {
				res = sync_fs(fs);
			}
		}
		FREE_NAMEBUFF();
	}

	LEAVE_FF(fs, res);
}

#endif	/* FF_USE_CHMOD && !FF_FS_READONLY */



#if FF_USE_LABEL
/*-----------------------------------------------------------------------*/
/* API: Get Volume Label                                                 */
/*-----------------------------------------------------------------------*/

FRESULT f_getlabel (
	const TCHAR *path,	/* Logical drive number */
	TCHAR *label,		/* Buffer to store the volume label */
	DWORD *vsn			/* Variable to store the volume serial number */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;
	DIR dj;
	UINT si, di;
	WCHAR wc;


	res = mount_volume(&path, &fs, 0);	/* Get logical drive and mount the volume if needed */

	/* Get volume label */
	if (res == FR_OK && label) {
		dj.obj.fs = fs;
		dj.obj.sclust = 0;	/* Open root directory */
		res = dir_sdi(&dj, 0);
		if (res == FR_OK) {
			res = DIR_READ_LABEL(&dj);		/* Find a volume label entry */
			if (res == FR_OK) {
#if FF_FS_EXFAT
				if (fs->fs_type == FS_EXFAT) {
					WCHAR hs;
					UINT nw;

					for (si = di = hs = 0; si < dj.dir[XDIR_NumLabel]; si++) {	/* Extract volume label from 83 entry */
						wc = ld_16(dj.dir + XDIR_Label + si * 2);
						if (hs == 0 && IsSurrogate(wc)) {	/* Is the code a surrogate? */
							hs = wc;
							continue;
						}
						nw = put_utf((DWORD)hs << 16 | wc, &label[di], 4);	/* Store it in API encoding */
						if (nw == 0) {		/* Encode error? */
							di = 0;
							break;
						}
						di += nw;
						hs = 0;
					}
					if (hs != 0) {
						di = 0;        /* Broken surrogate pair? */
					}
					label[di] = 0;
				}
				else
#endif
				{
					si = di = 0;		/* Extract volume label from AM_VOL entry */
					while (si < 11) {
						wc = dj.dir[si++];
#if FF_USE_LFN && FF_LFN_UNICODE >= 1 	/* Unicode output */
						if (dbc_1st((BYTE)wc) && si < 11) {
							wc = wc << 8 | dj.dir[si++];        /* Is it a DBC? */
						}
						wc = ff_oem2uni(wc, CODEPAGE);		/* Convert it into Unicode */
						if (wc == 0) {		/* Invalid char in current code page? */
							di = 0;
							break;
						}
						di += put_utf(wc, &label[di], 4);	/* Store it in Unicode */
#else									/* ANSI/OEM output */
						label[di++] = (TCHAR)wc;
#endif
					}
					do {				/* Truncate trailing spaces */
						label[di] = 0;
						if (di == 0) {
							break;
						}
					}
					while (label[--di] == ' ');
				}
			}
		}
		if (res == FR_NO_FILE) {	/* No label entry and return nul string */
			label[0] = 0;
			res = FR_OK;
		}
	}

	/* Get volume serial number */
	if (res == FR_OK && vsn) {
		res = move_window(fs, fs->volbase);	/* Load VBR */
		if (res == FR_OK) {
			switch (fs->fs_type) {
				case FS_EXFAT:
					di = BPB_VolIDEx;
					break;

				case FS_FAT32:
					di = BS_VolID32;
					break;

				default:	/* FAT12/16 */
					di = fs->win[BS_BootSig] == 0x29 ? BS_VolID : 0;
			}
			*vsn = di ? ld_32(fs->win + di) : 0;	/* Get VSN in the VBR */
		}
	}

	LEAVE_FF(fs, res);
}



#if !FF_FS_READONLY
/*-----------------------------------------------------------------------*/
/* API: Set Volume Label                                                 */
/*-----------------------------------------------------------------------*/

FRESULT f_setlabel (
	const TCHAR *label	/* Volume label to set with heading logical drive number */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;
	DIR dj;
	BYTE dirvn[22];
	UINT di;
	WCHAR wc;
	static const char badchr[18] = "+.,;=[]" "/*:<>|\\\"\?\x7F";	/* [0..16] for FAT, [7..16] for exFAT */
#if FF_USE_LFN
	DWORD dc;
#endif

	/* Get logical drive and mount the volume if needed */
	res = mount_volume(&label, &fs, FA_WRITE);
	if (res != FR_OK) {
		LEAVE_FF(fs, res);
	}
#if FF_STR_VOLUME_ID == 2
	for ( ; *label == '/'; label++) ;	/* Snip the separators off */
#endif

#if FF_FS_EXFAT
	if (fs->fs_type == FS_EXFAT) {	/* On the exFAT volume */
		memset(dirvn, 0, 22);
		di = 0;
		while ((UINT)*label >= ' ') {	/* Create volume label */
			dc = tchar2uni(&label);	/* Get a Unicode character */
			if (dc >= 0x10000) {
				if (dc == 0xFFFFFFFF || di >= 10) {	/* Wrong surrogate or buffer overflow */
					dc = 0;
				}
				else {
					st_16(dirvn + di * 2, (WCHAR)(dc >> 16)); di++;
					di++;
				}
			}
			if (dc == 0 || strchr(&badchr[7], (int)dc) || di >= 11) {	/* Check validity of the volume label */
				LEAVE_FF(fs, FR_INVALID_NAME);
			}
			st_16(dirvn + di * 2, (WCHAR)dc);
			di++;
		}
	}
	else
#endif
	{	/* On the FAT/FAT32 volume */
		memset(dirvn, ' ', 11);
		di = 0;
		while ((UINT)*label >= ' ') {	/* Create volume label */
#if FF_USE_LFN
			dc = tchar2uni(&label);
			wc = (dc < 0x10000) ? ff_uni2oem(ff_wtoupper(dc), CODEPAGE) : 0;
#else									/* ANSI/OEM input */
			wc = (BYTE) * label++;
			if (dbc_1st((BYTE)wc)) {
				wc = dbc_2nd((BYTE) * label) ? wc << 8 | (BYTE) * label++ : 0;
			}
			if (IsLower(wc)) {
				wc -= 0x20;        /* To upper ASCII characters */
			}
#if FF_CODE_PAGE == 0
			if (ExCvt && wc >= 0x80) {
				wc = ExCvt[wc - 0x80];        /* To upper extended characters (SBCS cfg) */
			}
#elif FF_CODE_PAGE < 900
			if (wc >= 0x80) {
				wc = ExCvt[wc - 0x80];        /* To upper extended characters (SBCS cfg) */
			}
#endif
#endif
			if (wc == 0 || strchr(&badchr[0], (int)wc)
			    || di >= (UINT)((wc >= 0x100) ? 10 : 11)) {	/* Reject invalid characters for volume label */
				LEAVE_FF(fs, FR_INVALID_NAME);
			}
			if (wc >= 0x100) {
				dirvn[di++] = (BYTE)(wc >> 8);
			}
			dirvn[di++] = (BYTE)wc;
		}
		if (dirvn[0] == DDEM) {
			LEAVE_FF(fs, FR_INVALID_NAME);        /* Reject illegal name (heading DDEM) */
		}
		while (di && dirvn[di - 1] == ' ') {
			di--;        /* Snip trailing spaces */
		}
	}

	/* Set volume label */
	dj.obj.fs = fs;
	dj.obj.sclust = 0;	/* Open root directory */
	res = dir_sdi(&dj, 0);
	if (res == FR_OK) {
		res = DIR_READ_LABEL(&dj);	/* Get volume label entry */
		if (res == FR_OK) {
			if (FF_FS_EXFAT && fs->fs_type == FS_EXFAT) {
				dj.dir[XDIR_NumLabel] = (BYTE)di;	/* Change the volume label */
				mem_cpy(dj.dir + XDIR_Label, dirvn, 22);
			}
			else {
				if (di != 0) {
					mem_cpy(dj.dir, dirvn, 11);	/* Change the volume label */
				}
				else {
					dj.dir[DIR_Name] = DDEM;	/* Remove the volume label */
				}
			}
			fs->wflag = 1;
			res = sync_fs(fs);
		}
		else {			/* No volume label entry or an error */
			if (res == FR_NO_FILE) {
				res = FR_OK;
				if (di != 0) {	/* Create a volume label entry */
					res = dir_alloc(&dj, 1);	/* Allocate an entry */
					if (res == FR_OK) {
						memset(dj.dir, 0, SZDIRE);	/* Clean the entry */
						if (FF_FS_EXFAT && fs->fs_type == FS_EXFAT) {
							dj.dir[XDIR_Type] = ET_VLABEL;	/* Create volume label entry */
							dj.dir[XDIR_NumLabel] = (BYTE)di;
							mem_cpy(dj.dir + XDIR_Label, dirvn, 22);
						}
						else {
							dj.dir[DIR_Attr] = AM_VOL;		/* Create volume label entry */
							mem_cpy(dj.dir, dirvn, 11);
						}
						fs->wflag = 1;
						res = sync_fs(fs);
					}
				}
			}
		}
	}

	LEAVE_FF(fs, res);
}

#endif /* !FF_FS_READONLY */
#endif /* FF_USE_LABEL */



#if FF_USE_EXPAND && !FF_FS_READONLY
/*-----------------------------------------------------------------------*/
/* API: Allocate a Contiguous Blocks to the File                         */
/*-----------------------------------------------------------------------*/

FRESULT f_expand (
	FIL *fp,		/* Pointer to the file object */
	FSIZE_t fsz,	/* File size to be expanded to */
	BYTE opt		/* Operation mode 0:Find and prepare or 1:Find and allocate */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;
	DWORD n, clst, stcl, scl, ncl, tcl, lclst;


	res = validate(&fp->obj, &fs);		/* Check validity of the file object */
	if (res != FR_OK || (res = (FRESULT)fp->err) != FR_OK) {
		LEAVE_FF(fs, res);
	}
	if (fsz == 0 || fp->obj.objsize != 0 || !(fp->flag & FA_WRITE)) {
		LEAVE_FF(fs, FR_DENIED);
	}
#if FF_FS_EXFAT
	if (fs->fs_type != FS_EXFAT && fsz >= 0x100000000) {
		LEAVE_FF(fs, FR_DENIED);        /* Check if in size limit */
	}
#endif
	n = (DWORD)fs->csize * SS(fs);	/* Cluster size */
	tcl = (DWORD)(fsz / n) + ((fsz & (n - 1)) ? 1 : 0);	/* Number of clusters required */
	stcl = fs->last_clst;
	lclst = 0;
	if (stcl < 2 || stcl >= fs->n_fatent) {
		stcl = 2;
	}

#if FF_FS_EXFAT
	if (fs->fs_type == FS_EXFAT) {
		scl = find_bitmap(fs, stcl, tcl);			/* Find a contiguous cluster block */
		if (scl == 0) {
			res = FR_DENIED;        /* No contiguous cluster block was found */
		}
		if (scl == 0xFFFFFFFF) {
			res = FR_DISK_ERR;
		}
		if (res == FR_OK) {	/* A contiguous free area is found */
			if (opt) {		/* Allocate it now */
				res = change_bitmap(fs, scl, tcl, 1);	/* Mark the cluster block 'in use' */
				lclst = scl + tcl - 1;
			}
			else {		/* Set it as suggested point for next allocation */
				lclst = scl - 1;
			}
		}
	}
	else
#endif
	{
		scl = clst = stcl;
		ncl = 0;
		for (;;) {	/* Find a contiguous cluster block */
			n = get_fat(&fp->obj, clst);
			if (++clst >= fs->n_fatent) {
				clst = 2;
			}
			if (n == 1) {
				res = FR_INT_ERR;
				break;
			}
			if (n == 0xFFFFFFFF) {
				res = FR_DISK_ERR;
				break;
			}
			if (n == 0) {	/* Is it a free cluster? */
				if (++ncl == tcl) {
					break;        /* Break if a contiguous cluster block is found */
				}
			}
			else {
				scl = clst;
				ncl = 0;		/* Not a free cluster */
			}
			if (clst == stcl) {		/* No contiguous cluster? */
				res = FR_DENIED;
				break;
			}
		}
		if (res == FR_OK) {	/* A contiguous free area is found */
			if (opt) {		/* Allocate it now */
				for (clst = scl, n = tcl; n; clst++, n--) {	/* Create a cluster chain on the FAT */
					res = put_fat(fs, clst, (n == 1) ? 0xFFFFFFFF : clst + 1);
					if (res != FR_OK) {
						break;
					}
					lclst = clst;
				}
			}
			else {		/* Set it as suggested point for next allocation */
				lclst = scl - 1;
			}
		}
	}

	if (res == FR_OK) {
		fs->last_clst = lclst;		/* Set suggested start cluster to start next */
		if (opt) {	/* Is it allocated now? */
			fp->obj.sclust = scl;		/* Update object allocation information */
			fp->obj.objsize = fsz;
			if (FF_FS_EXFAT) {
				fp->obj.stat = 2;        /* Set status 'contiguous chain' */
			}
			fp->flag |= FA_MODIFIED;
			if (fs->free_clst <= fs->n_fatent - 2) {	/* Update FSINFO */
				fs->free_clst -= tcl;
				fs->fsi_flag |= 1;
			}
		}
	}

	LEAVE_FF(fs, res);
}

#endif /* FF_USE_EXPAND && !FF_FS_READONLY */



#if FF_USE_FORWARD
/*-----------------------------------------------------------------------*/
/* API: Forward Data to the Stream Directly                              */
/*-----------------------------------------------------------------------*/

FRESULT f_forward (
	FIL *fp, 						/* Pointer to the file object */
	UINT (*func)(const BYTE *, UINT),	/* Pointer to the streaming function */
	UINT btf,						/* Number of bytes to forward */
	UINT *bf						/* Pointer to number of bytes forwarded */
)
{
	FRESULT res = FR_DISK_ERR;
	FATFS *fs;
	DWORD clst;
	LBA_t sect;
	FSIZE_t remain;
	UINT rcnt, csect;
	BYTE *dbuf;


	*bf = 0;	/* Clear transfer byte counter */
	res = validate(&fp->obj, &fs);		/* Check validity of the file object */
	if (res != FR_OK || (res = (FRESULT)fp->err) != FR_OK) {
		LEAVE_FF(fs, res);
	}
	if (!(fp->flag & FA_READ)) {
		LEAVE_FF(fs, FR_DENIED);        /* Check access mode */
	}

	remain = fp->obj.objsize - fp->fptr;
	if (btf > remain) {
		btf = (UINT)remain;        /* Truncate btf by remaining bytes */
	}

	for ( ; btf > 0
	      && (*func)(0, 0); fp->fptr += rcnt, *bf += rcnt, btf -= rcnt) {	/* Repeat until all data transferred or stream goes busy */
		csect = (UINT)(fp->fptr / SS(fs) & (fs->csize - 1));	/* Sector offset in the cluster */
		if (fp->fptr % SS(fs) == 0) {				/* On the sector boundary? */
			if (csect == 0) {						/* On the cluster boundary? */
				clst = (fp->fptr == 0) ?			/* On the top of the file? */
				       fp->obj.sclust : get_fat(&fp->obj, fp->clust);
				if (clst <= 1) {
					ABORT(fs, FR_INT_ERR);
				}
				if (clst == 0xFFFFFFFF) {
					ABORT(fs, FR_DISK_ERR);
				}
				fp->clust = clst;					/* Update current cluster */
			}
		}
		sect = clst2sect(fs, fp->clust);			/* Get current data sector */
		if (sect == 0) {
			ABORT(fs, FR_INT_ERR);
		}
		sect += csect;
#if FF_FS_TINY
		if (move_window(fs, sect) != FR_OK) {
			ABORT(fs, FR_DISK_ERR);        /* Move sector window to the file data */
		}
		dbuf = fs->win;
#else
		if (fp->sect != sect) {		/* Fill sector cache with file data */
#if !FF_FS_READONLY
			if (fp->flag & FA_DIRTY) {		/* Write-back dirty sector cache */
				if (disk_write(fs->pdrv, fp->buf, fp->sect, 1) != RES_OK) {
					ABORT(fs, FR_DISK_ERR);
				}
				fp->flag &= (BYTE)~FA_DIRTY;
			}
#endif
			if (disk_read(fs->pdrv, fp->buf, sect, 1) != RES_OK) {
				ABORT(fs, FR_DISK_ERR);
			}
		}
		dbuf = fp->buf;
#endif
		fp->sect = sect;
		rcnt = SS(fs) - (UINT)fp->fptr % SS(fs);	/* Number of bytes remains in the sector */
		if (rcnt > btf) {
			rcnt = btf;        /* Clip it by btr if needed */
		}
		rcnt = (*func)(dbuf + ((UINT)fp->fptr % SS(fs)), rcnt);	/* Forward the file data */
		if (rcnt == 0) {
			ABORT(fs, FR_INT_ERR);
		}
	}

	LEAVE_FF(fs, FR_OK);
}
#endif /* FF_USE_FORWARD */



#if !FF_FS_READONLY && FF_USE_MKFS
/*-----------------------------------------------------------------------*/
/* API: Create FAT/exFAT volume (with a sub-function)                    */
/*-----------------------------------------------------------------------*/

#define N_SEC_TRACK 63			/* Sectors per track for determination of drive CHS */
#define	GPT_ALIGN	0x100000	/* Alignment of partitions in GPT [byte] (>=128KB) */
#define GPT_ITEMS	128			/* Number of GPT table items (>=128, sector aligned) */


/* Create partitions on the physical drive in format of MBR or GPT */

static FRESULT create_partition (
	BYTE drv,			/* Physical drive number */
	const LBA_t plst[],	/* Partition list */
	BYTE sys,			/* System ID for each partition (for only MBR) */
	BYTE *buf			/* Working buffer for a sector */
)
{
	UINT i, cy;
	LBA_t sz_drv;
	DWORD sz_drv32, nxt_alloc32, sz_part32;
	BYTE *pte;
	BYTE hd, n_hd, sc, n_sc;

	/* Get physical drive size */
	if (disk_ioctl(drv, GET_SECTOR_COUNT, &sz_drv) != RES_OK) {
		return FR_DISK_ERR;
	}

#if FF_LBA64
	if (sz_drv >= FF_MIN_GPT) {	/* Create partitions in GPT format */
		WORD ss;
		UINT sz_ptbl, pi, si, ofs;
		DWORD bcc, rnd, align;
		QWORD nxt_alloc, sz_part, sz_pool, top_bpt;
		static const BYTE gpt_mbr[16] = {0x00, 0x00, 0x02, 0x00, 0xEE, 0xFE, 0xFF, 0x00, 0x01, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF};

#if FF_MAX_SS != FF_MIN_SS
		if (disk_ioctl(drv, GET_SECTOR_SIZE, &ss) != RES_OK) {
			return FR_DISK_ERR;        /* Get sector size */
		}
		if (ss > FF_MAX_SS || ss < FF_MIN_SS || (ss & (ss - 1))) {
			return FR_DISK_ERR;
		}
#else
		ss = FF_MAX_SS;
#endif
		rnd = (DWORD)sz_drv + GET_FATTIME();	/* Random seed */
		align = GPT_ALIGN / ss;				/* Partition alignment for GPT [sector] */
		sz_ptbl = GPT_ITEMS * SZ_GPTE / ss;	/* Size of partition table [sector] */
		top_bpt = sz_drv - sz_ptbl - 1;		/* Backup partition table start LBA */
		nxt_alloc = 2 + sz_ptbl;			/* First allocatable LBA */
		sz_pool = top_bpt - nxt_alloc;		/* Size of allocatable area [sector] */
		bcc = 0xFFFFFFFF;
		sz_part = 1;
		pi = si = 0;	/* partition table index, map index */
		do {
			if (pi * SZ_GPTE % ss == 0) {
				memset(buf, 0, ss);        /* Clean the buffer if needed */
			}
			if (sz_part != 0) {				/* Is the size table not termintated? */
				nxt_alloc = (nxt_alloc + align - 1) & ((QWORD)0 - align);	/* Align partition start LBA */
				sz_part = plst[si++];		/* Get a partition size */
				if (sz_part <= 100) {		/* Is the size in percentage? */
					sz_part = sz_pool * sz_part / 100;	/* Sectors in percentage */
					sz_part = (sz_part + align - 1) & ((QWORD)0 - align);	/* Align partition end LBA (only if in percentage) */
				}
				if (nxt_alloc + sz_part > top_bpt) {	/* Clip the size at end of the pool */
					sz_part = (nxt_alloc < top_bpt) ? top_bpt - nxt_alloc : 0;
				}
			}
			if (sz_part != 0) {				/* Add a partition? */
				ofs = pi * SZ_GPTE % ss;
				mem_cpy(buf + ofs + GPTE_PtGuid, GUID_MS_Basic, 16);	/* Set partition GUID (Microsoft Basic Data) */
				rnd = make_rand(rnd, buf + ofs + GPTE_UpGuid, 16);	/* Set unique partition GUID */
				st_64(buf + ofs + GPTE_FstLba, nxt_alloc);			/* Set partition start LBA */
				st_64(buf + ofs + GPTE_LstLba, nxt_alloc + sz_part - 1);	/* Set partition end LBA */
				nxt_alloc += sz_part;								/* Next allocatable LBA */
			}
			if ((pi + 1) * SZ_GPTE % ss == 0) {		/* Write the sector buffer if it is filled up */
				for (i = 0; i < ss; bcc = crc32(bcc, buf[i++])) ;	/* Calculate table check sum */
				if (disk_write(drv, buf, 2 + pi * SZ_GPTE / ss, 1) != RES_OK) {
					return FR_DISK_ERR;        /* Write to primary table */
				}
				if (disk_write(drv, buf, top_bpt + pi * SZ_GPTE / ss, 1) != RES_OK) {
					return FR_DISK_ERR;        /* Write to secondary table */
				}
			}
		}
		while (++pi < GPT_ITEMS);

		/* Create primary GPT header */
		memset(buf, 0, ss);
		mem_cpy(buf + GPTH_Sign, "EFI PART" "\0\0\1\0" "\x5C\0\0", 16);	/* Signature, version (1.0) and size (92) */
		st_32(buf + GPTH_PtBcc, ~bcc);			/* Table check sum */
		st_64(buf + GPTH_CurLba, 1);			/* LBA of this header */
		st_64(buf + GPTH_BakLba, sz_drv - 1);	/* LBA of secondary header */
		st_64(buf + GPTH_FstLba, 2 + sz_ptbl);	/* LBA of first allocatable sector */
		st_64(buf + GPTH_LstLba, top_bpt - 1);	/* LBA of last allocatable sector */
		st_32(buf + GPTH_PteSize, SZ_GPTE);		/* Size of a table entry */
		st_32(buf + GPTH_PtNum, GPT_ITEMS);		/* Number of table entries */
		st_32(buf + GPTH_PtOfs, 2);				/* LBA of this table */
		rnd = make_rand(rnd, buf + GPTH_DskGuid, 16);	/* Disk GUID */
		for (i = 0, bcc = 0xFFFFFFFF; i < 92; bcc = crc32(bcc, buf[i++])) ;	/* Calculate header check sum */
		st_32(buf + GPTH_Bcc, ~bcc);			/* Header check sum */
		if (disk_write(drv, buf, 1, 1) != RES_OK) {
			return FR_DISK_ERR;
		}

		/* Create secondary GPT header */
		st_64(buf + GPTH_CurLba, sz_drv - 1);	/* LBA of this header */
		st_64(buf + GPTH_BakLba, 1);		 	/* LBA of primary header */
		st_64(buf + GPTH_PtOfs, top_bpt);		/* LBA of this table */
		st_32(buf + GPTH_Bcc, 0);
		for (i = 0, bcc = 0xFFFFFFFF; i < 92; bcc = crc32(bcc, buf[i++])) ;	/* Calculate header check sum */
		st_32(buf + GPTH_Bcc, ~bcc);			/* Header check sum */
		if (disk_write(drv, buf, sz_drv - 1, 1) != RES_OK) {
			return FR_DISK_ERR;
		}

		/* Create protective MBR */
		memset(buf, 0, ss);
		mem_cpy(buf + MBR_Table, gpt_mbr, 16);	/* Create a GPT partition */
		st_16(buf + BS_55AA, 0xAA55);
		if (disk_write(drv, buf, 0, 1) != RES_OK) {
			return FR_DISK_ERR;
		}

	}
	else
#endif
	{	/* Create partitions in MBR format */
		sz_drv32 = (DWORD)sz_drv;
		n_sc = N_SEC_TRACK;				/* Determine drive CHS without any consideration of the drive geometry */
		for (n_hd = 8; n_hd != 0 && sz_drv32 / n_hd / n_sc > 1024; n_hd *= 2) ;
		if (n_hd == 0) {
			n_hd = 255;        /* Number of heads needs to be <256 */
		}

		memset(buf, 0, FF_MAX_SS);		/* Clear MBR */
		pte = buf + MBR_Table;	/* Partition table in the MBR */
		for (i = 0, nxt_alloc32 = n_sc; i < 4 && nxt_alloc32 != 0 && nxt_alloc32 < sz_drv32; i++, nxt_alloc32 += sz_part32) {
			sz_part32 = (DWORD)plst[i];	/* Get partition size */
			if (sz_part32 <= 100) {
				sz_part32 = (sz_part32 == 100) ? sz_drv32 : sz_drv32 / 100 * sz_part32;        /* Size in percentage? */
			}
			if (nxt_alloc32 + sz_part32 > sz_drv32 || nxt_alloc32 + sz_part32 < nxt_alloc32) {
				sz_part32 = sz_drv32 - nxt_alloc32;        /* Clip at drive size */
			}
			if (sz_part32 == 0) {
				break;        /* End of table or no sector to allocate? */
			}

			st_32(pte + PTE_StLba, nxt_alloc32);	/* Partition start LBA sector */
			st_32(pte + PTE_SizLba, sz_part32);		/* Size of partition [sector] */
			pte[PTE_System] = sys;					/* System type */

			cy = (UINT)(nxt_alloc32 / n_sc / n_hd);	/* Partitio start CHS cylinder */
			hd = (BYTE)(nxt_alloc32 / n_sc % n_hd);	/* Partition start CHS head */
			sc = (BYTE)(nxt_alloc32 % n_sc + 1);	/* Partition start CHS sector */
			pte[PTE_StHead] = hd;
			pte[PTE_StSec] = (BYTE)((cy >> 2 & 0xC0) | sc);
			pte[PTE_StCyl] = (BYTE)cy;

			cy = (UINT)((nxt_alloc32 + sz_part32 - 1) / n_sc / n_hd);	/* Partition end CHS cylinder */
			hd = (BYTE)((nxt_alloc32 + sz_part32 - 1) / n_sc % n_hd);	/* Partition end CHS head */
			sc = (BYTE)((nxt_alloc32 + sz_part32 - 1) % n_sc + 1);		/* Partition end CHS sector */
			pte[PTE_EdHead] = hd;
			pte[PTE_EdSec] = (BYTE)((cy >> 2 & 0xC0) | sc);
			pte[PTE_EdCyl] = (BYTE)cy;

			pte += SZ_PTE;		/* Next entry */
		}

		st_16(buf + BS_55AA, 0xAA55);		/* MBR signature */
		if (disk_write(drv, buf, 0, 1) != RES_OK) {
			return FR_DISK_ERR;        /* Write it to the MBR */
		}
	}

	return FR_OK;
}



FRESULT f_mkfs (
	const TCHAR *path,		/* Logical drive number */
	const MKFS_PARM *opt,	/* Format options */
	void *work,				/* Pointer to working buffer (null: use len bytes of heap memory) */
	UINT len				/* Size of working buffer [byte] */
)
{
	static const WORD cst[] = {1, 4, 16, 64, 256, 512, 0};	/* Cluster size boundary for FAT volume (4K sector unit) */
	static const WORD cst32[] = {1, 2, 4, 8, 16, 32, 0};	/* Cluster size boundary for FAT32 volume (128K sector unit) */
	static const MKFS_PARM defopt = {FM_ANY, 0, 0, 0, 0};	/* Default parameter */
	BYTE fsopt, fsty, sys, pdrv, ipart;
	BYTE *buf;
	BYTE *pte;
	WORD ss;	/* Sector size */
	DWORD sz_buf, sz_blk, n_clst, pau, nsect, n, vsn;
	LBA_t sz_vol, b_vol, b_fat, b_data;		/* Volume size, base LBA of volume, base LBA of FAT and base LBA of data */
	LBA_t sect, lba[2];
	DWORD sz_rsv, sz_fat, sz_dir, sz_au;	/* Size of reserved area, FAT area, directory area, data area and cluster */
	UINT n_fat, n_root, i;					/* Number of FATs, number of roor directory entries and some index */
	int vol;
	DSTATUS ds;
	FRESULT res;


	/* Check mounted drive and clear work area */
	vol = get_ldnumber(&path);					/* Get logical drive number to be formatted */
	if (vol < 0) {
		return FR_INVALID_DRIVE;
	}
	if (FatFs[vol]) {
		FatFs[vol]->fs_type = 0;        /* Clear the fs object if mounted */
	}
	pdrv = LD2PD(vol);		/* Hosting physical drive */
	ipart = LD2PT(vol);		/* Hosting partition (0:create as new, 1..:existing partition) */

	/* Initialize the hosting physical drive */
	ds = disk_initialize(pdrv);
	if (ds & STA_NOINIT) {
		return FR_NOT_READY;
	}
	if (ds & STA_PROTECT) {
		return FR_WRITE_PROTECTED;
	}

	/* Get physical drive parameters (sz_drv, sz_blk and ss) */
	if (!opt) {
		opt = &defopt;        /* Use default parameter if it is not given */
	}
	sz_blk = opt->align;
	if (sz_blk == 0) {
		disk_ioctl(pdrv, GET_BLOCK_SIZE, &sz_blk);        /* Block size from the parameter or lower layer */
	}
	if (sz_blk == 0 || sz_blk > 0x8000 || (sz_blk & (sz_blk - 1))) {
		sz_blk = 1;        /* Use default if the block size is invalid */
	}
#if FF_MAX_SS != FF_MIN_SS
	if (disk_ioctl(pdrv, GET_SECTOR_SIZE, &ss) != RES_OK) {
		return FR_DISK_ERR;
	}
	if (ss > FF_MAX_SS || ss < FF_MIN_SS || (ss & (ss - 1))) {
		return FR_DISK_ERR;
	}
#else
	ss = FF_MAX_SS;
#endif

	/* Options for FAT sub-type and FAT parameters */
	fsopt = opt->fmt & (FM_ANY | FM_SFD);
	n_fat = (opt->n_fat >= 1 && opt->n_fat <= 2) ? opt->n_fat : 1;
	n_root = (opt->n_root >= 1 && opt->n_root <= 32768 && (opt->n_root % (ss / SZDIRE)) == 0) ? opt->n_root : 512;
	sz_au = (opt->au_size <= 0x1000000 && (opt->au_size & (opt->au_size - 1)) == 0) ? opt->au_size : 0;
	sz_au /= ss;	/* Byte --> Sector */

	/* Get working buffer */
	sz_buf = len / ss;		/* Size of working buffer [sector] */
	if (sz_buf == 0) {
		return FR_NOT_ENOUGH_CORE;
	}
	buf = (BYTE *)work;		/* Working buffer */
#if FF_USE_LFN == 3
	if (!buf) {
		buf = ff_memalloc(sz_buf * ss);        /* Use heap memory for working buffer */
	}
#endif
	if (!buf) {
		return FR_NOT_ENOUGH_CORE;
	}

	/* Determine where the volume to be located (b_vol, sz_vol) */
	b_vol = sz_vol = 0;
	if (FF_MULTI_PARTITION && ipart != 0) {	/* Is the volume associated with any specific partition? */
		/* Get partition location from the existing partition table */
		if (disk_read(pdrv, buf, 0, 1) != RES_OK) {
			LEAVE_MKFS(FR_DISK_ERR);        /* Load MBR */
		}
		if (ld_16(buf + BS_55AA) != 0xAA55) {
			LEAVE_MKFS(FR_MKFS_ABORTED);        /* Check if MBR is valid */
		}
#if FF_LBA64
		if (buf[MBR_Table + PTE_System] == 0xEE) {	/* GPT protective MBR? */
			DWORD n_ent, ofs;
			QWORD pt_lba;

			/* Get the partition location from GPT */
			if (disk_read(pdrv, buf, 1, 1) != RES_OK) {
				LEAVE_MKFS(FR_DISK_ERR);        /* Load GPT header sector (next to MBR) */
			}
			if (!test_gpt_header(buf)) {
				LEAVE_MKFS(FR_MKFS_ABORTED);        /* Check if GPT header is valid */
			}
			n_ent = ld_32(buf + GPTH_PtNum);	/* Number of entries */
			pt_lba = ld_64(buf + GPTH_PtOfs);	/* Table start sector */
			ofs = i = 0;
			while (n_ent) {		/* Find MS Basic partition with order of ipart */
				if (ofs == 0 && disk_read(pdrv, buf, pt_lba++, 1) != RES_OK) {
					LEAVE_MKFS(FR_DISK_ERR);        /* Get PT sector */
				}
				if (!memcmp(buf + ofs + GPTE_PtGuid, GUID_MS_Basic, 16) && ++i == ipart) {	/* MS basic data partition? */
					b_vol = ld_64(buf + ofs + GPTE_FstLba);
					sz_vol = ld_64(buf + ofs + GPTE_LstLba) - b_vol + 1;
					break;
				}
				n_ent--;
				ofs = (ofs + SZ_GPTE) % ss;	/* Next entry */
			}
			if (n_ent == 0) {
				LEAVE_MKFS(FR_MKFS_ABORTED);        /* Partition not found */
			}
			fsopt |= 0x80;	/* Partitioning is in GPT */
		}
		else
#endif
		{	/* Get the partition location from MBR partition table */
			pte = buf + (MBR_Table + (ipart - 1) * SZ_PTE);
			if (ipart > 4 || pte[PTE_System] == 0) {
				LEAVE_MKFS(FR_MKFS_ABORTED);        /* No partition? */
			}
			b_vol = ld_32(pte + PTE_StLba);		/* Get volume start sector */
			sz_vol = ld_32(pte + PTE_SizLba);	/* Get volume size */
		}
	}
	else {	/* The volume is associated with a physical drive */
		if (disk_ioctl(pdrv, GET_SECTOR_COUNT, &sz_vol) != RES_OK) {
			LEAVE_MKFS(FR_DISK_ERR);
		}
		if (!(fsopt & FM_SFD)) {	/* To be partitioned? */
			/* Create a single-partition on the drive in this function */
#if FF_LBA64
			if (sz_vol >= FF_MIN_GPT) {	/* Which partition type to create, MBR or GPT? */
				fsopt |= 0x80;		/* Partitioning is in GPT */
				b_vol = GPT_ALIGN / ss;
				sz_vol -= b_vol + GPT_ITEMS * SZ_GPTE / ss + 1;	/* Estimated partition offset and size */
			}
			else
#endif
			{	/* Partitioning is in MBR */
				if (sz_vol > N_SEC_TRACK) {
					b_vol = N_SEC_TRACK;
					sz_vol -= b_vol;	/* Estimated partition offset and size */
				}
			}
		}
	}
	if (sz_vol < 128) {
		LEAVE_MKFS(FR_MKFS_ABORTED);        /* Check if volume size is >=128 sectors */
	}

	/* Now start to create an FAT volume at b_vol and sz_vol */

	do {	/* Pre-determine the FAT type */
		if (FF_FS_EXFAT && (fsopt & FM_EXFAT)) {	/* exFAT possible? */
			if ((fsopt & FM_ANY) == FM_EXFAT || sz_vol >= 0x4000000
			    || sz_au > 128) {	/* exFAT only, vol >= 64M sectors or sz_au > 128 sectors? */
				fsty = FS_EXFAT;
				break;
			}
		}
#if FF_LBA64
		if (sz_vol >= 0x100000000) {
			LEAVE_MKFS(FR_MKFS_ABORTED);        /* Too large volume for FAT/FAT32 */
		}
#endif
		if (sz_au > 128) {
			sz_au = 128;        /* Invalid AU for FAT/FAT32? */
		}
		if (fsopt & FM_FAT32) {	/* FAT32 possible? */
			if (!(fsopt & FM_FAT)) {	/* no-FAT? */
				fsty = FS_FAT32;
				break;
			}
		}
		if (!(fsopt & FM_FAT)) {
			LEAVE_MKFS(FR_INVALID_PARAMETER);        /* no-FAT? */
		}
		fsty = FS_FAT16;
	}
	while (0);

	vsn = (DWORD)sz_vol + GET_FATTIME();	/* VSN generated from current time and partition size */

#if FF_FS_EXFAT
	if (fsty == FS_EXFAT) {	/* Create an exFAT volume */
		DWORD szb_bit, szb_case, sum, nbit, clu, clen[3];
		WCHAR ch, si;
		UINT j, st;

		if (sz_vol < 0x1000) {
			LEAVE_MKFS(FR_MKFS_ABORTED);        /* Too small volume for exFAT? */
		}
#if FF_USE_TRIM
		lba[0] = b_vol;
		lba[1] = b_vol + sz_vol - 1;	/* Inform storage device that the volume area may be erased */
		disk_ioctl(pdrv, CTRL_TRIM, lba);
#endif
		/* Determine FAT location, data location and number of clusters */
		if (sz_au == 0) {	/* AU auto-selection */
			sz_au = 8;
			if (sz_vol >= 0x80000) {
				sz_au = 64;        /* >= 512Ks */
			}
			if (sz_vol >= 0x4000000) {
				sz_au = 256;        /* >= 64Ms */
			}
		}
		b_fat = b_vol + 32;										/* FAT start at offset 32 */
		sz_fat = (DWORD)((sz_vol / sz_au + 2) * 4 + ss - 1) / ss;	/* Number of FAT sectors */
		b_data = (b_fat + sz_fat + sz_blk - 1) & ~((LBA_t)sz_blk - 1);	/* Align data area to the erase block boundary */
		if (b_data - b_vol >= sz_vol / 2) {
			LEAVE_MKFS(FR_MKFS_ABORTED);        /* Too small volume? */
		}
		n_clst = (DWORD)((sz_vol - (b_data - b_vol)) / sz_au);	/* Number of clusters */
		if (n_clst < 16) {
			LEAVE_MKFS(FR_MKFS_ABORTED);        /* Too few clusters? */
		}
		if (n_clst > MAX_EXFAT) {
			LEAVE_MKFS(FR_MKFS_ABORTED);        /* Too many clusters? */
		}

		szb_bit = (n_clst + 7) / 8;								/* Size of allocation bitmap */
		clen[0] = (szb_bit + sz_au * ss - 1) / (sz_au * ss);	/* Number of allocation bitmap clusters */

		/* Create a compressed up-case table */
		sect = b_data + sz_au * clen[0];	/* Table start sector */
		sum = 0;							/* Table checksum to be stored in the 82 entry */
		st = 0;
		si = 0;
		i = 0;
		j = 0;
		szb_case = 0;
		do {
			switch (st) {
				case 0:
					ch = (WCHAR)ff_wtoupper(si);	/* Get an up-case char */
					if (ch != si) {
						si++;
						break;		/* Store the up-case char if exist */
					}
					for (j = 1; (WCHAR)(si + j)
					     && (WCHAR)(si + j) == ff_wtoupper((WCHAR)(si + j)); j++) ;	/* Get run length of no-case block */
					if (j >= 128) {
						ch = 0xFFFF;
						st = 2;
						break;	/* Compress the no-case block if run is >= 128 chars */
					}
					st = 1;			/* Do not compress short run */
					ch = si++;		/* Fill the short run */
					if (--j == 0) {
						st = 0;
					}
					break;
				case 1:
					ch = si++;		/* Fill the short run */
					if (--j == 0) {
						st = 0;
					}
					break;

				default:
					ch = (WCHAR)j;
					si += (WCHAR)j;	/* Number of chars to skip */
					st = 0;
			}
			sum = xsum32(buf[i + 0] = (BYTE)ch, sum);	/* Put it into the write buffer */
			sum = xsum32(buf[i + 1] = (BYTE)(ch >> 8), sum);
			i += 2;
			szb_case += 2;
			if (si == 0 || i == sz_buf * ss) {		/* Write buffered data when buffer full or end of process */
				n = (i + ss - 1) / ss;
				if (disk_write(pdrv, buf, sect, n) != RES_OK) {
					LEAVE_MKFS(FR_DISK_ERR);
				}
				sect += n;
				i = 0;
			}
		}
		while (si);
		clen[1] = (szb_case + sz_au * ss - 1) / (sz_au * ss);	/* Number of up-case table clusters */
		clen[2] = 1;	/* Number of root directory clusters */

		/* Initialize the allocation bitmap */
		sect = b_data;
		nsect = (szb_bit + ss - 1) / ss;	/* Start of bitmap and number of bitmap sectors */
		nbit = clen[0] + clen[1] + clen[2];				/* Number of clusters in-use by system (bitmap, up-case and root-dir) */
		do {
			memset(buf, 0, sz_buf * ss);				/* Initialize bitmap buffer */
			for (i = 0; nbit != 0 && i / 8 < sz_buf * ss; buf[i / 8] |= 1 << (i % 8), i++, nbit--) ;	/* Mark used clusters */
			n = (nsect > sz_buf) ? sz_buf : nsect;		/* Write the buffered data */
			if (disk_write(pdrv, buf, sect, n) != RES_OK) {
				LEAVE_MKFS(FR_DISK_ERR);
			}
			sect += n;
			nsect -= n;
		}
		while (nsect);

		/* Initialize the FAT */
		sect = b_fat;
		nsect = sz_fat;	/* Start of FAT and number of FAT sectors */
		j = nbit = clu = 0;
		do {
			memset(buf, 0, sz_buf * ss);
			i = 0;	/* Clear work area and reset write offset */
			if (clu == 0) {	/* Initialize FAT [0] and FAT[1] */
				st_32(buf + i, 0xFFFFFFF8);
				i += 4;
				clu++;
				st_32(buf + i, 0xFFFFFFFF);
				i += 4;
				clu++;
			}

			do {			/* Create chains of bitmap, up-case and root directory */
				while (nbit != 0 && i < sz_buf * ss) {	/* Create a chain */
					st_32(buf + i, (nbit > 1) ? clu + 1 : 0xFFFFFFFF);
					i += 4;
					clu++;
					nbit--;
				}
				if (nbit == 0 && j < 3) {
					nbit = clen[j++];        /* Get next chain length */
				}
			}
			while (nbit != 0 && i < sz_buf * ss);
			n = (nsect > sz_buf) ? sz_buf : nsect;	/* Write the buffered data */
			if (disk_write(pdrv, buf, sect, n) != RES_OK) {
				LEAVE_MKFS(FR_DISK_ERR);
			}
			sect += n;
			nsect -= n;
		}
		while (nsect);

		/* Initialize the root directory */
		memset(buf, 0, sz_buf * ss);
		buf[SZDIRE * 0 + 0] = ET_VLABEL;			/* Volume label entry (no label) */
		buf[SZDIRE * 1 + 0] = ET_BITMAP;			/* Bitmap entry */
		st_32(buf + SZDIRE * 1 + 20, 2);			/*  cluster */
		st_32(buf + SZDIRE * 1 + 24, szb_bit);		/*  size */
		buf[SZDIRE * 2 + 0] = ET_UPCASE;			/* Up-case table entry */
		st_32(buf + SZDIRE * 2 + 4, sum);			/*  sum */
		st_32(buf + SZDIRE * 2 + 20, 2 + clen[0]);	/*  cluster */
		st_32(buf + SZDIRE * 2 + 24, szb_case);		/*  size */
		sect = b_data + sz_au * (clen[0] + clen[1]);
		nsect = sz_au;	/* Start of the root directory and number of sectors */
		do {	/* Fill root directory sectors */
			n = (nsect > sz_buf) ? sz_buf : nsect;
			if (disk_write(pdrv, buf, sect, n) != RES_OK) {
				LEAVE_MKFS(FR_DISK_ERR);
			}
			memset(buf, 0, ss);	/* Rest of entries are filled with zero */
			sect += n;
			nsect -= n;
		}
		while (nsect);

		/* Create two set of the exFAT VBR blocks */
		sect = b_vol;
		for (n = 0; n < 2; n++) {
			/* Main record (+0) */
			memset(buf, 0, ss);
			mem_cpy(buf + BS_JmpBoot, "\xEB\x76\x90" "EXFAT   ", 11);	/* Boot jump code (x86), OEM name */
			st_64(buf + BPB_VolOfsEx, b_vol);						/* Volume offset in the physical drive [sector] */
			st_64(buf + BPB_TotSecEx, sz_vol);						/* Volume size [sector] */
			st_32(buf + BPB_FatOfsEx, (DWORD)(b_fat - b_vol));		/* FAT offset [sector] */
			st_32(buf + BPB_FatSzEx, sz_fat);						/* FAT size [sector] */
			st_32(buf + BPB_DataOfsEx, (DWORD)(b_data - b_vol));	/* Data offset [sector] */
			st_32(buf + BPB_NumClusEx, n_clst);						/* Number of clusters */
			st_32(buf + BPB_RootClusEx, 2 + clen[0] + clen[1]);		/* Root directory cluster number */
			st_32(buf + BPB_VolIDEx, vsn);							/* VSN */
			st_16(buf + BPB_FSVerEx, 0x100);						/* Filesystem version (1.00) */
			for (buf[BPB_BytsPerSecEx] = 0, i = ss; i >>= 1; buf[BPB_BytsPerSecEx]++) ;		/* Log2 of sector size [byte] */
			for (buf[BPB_SecPerClusEx] = 0, i = sz_au; i >>= 1; buf[BPB_SecPerClusEx]++) ;	/* Log2 of cluster size [sector] */
			buf[BPB_NumFATsEx] = 1;					/* Number of FATs */
			buf[BPB_DrvNumEx] = 0x80;				/* Drive number (for int13) */
			st_16(buf + BS_BootCodeEx, 0xFEEB);	/* Boot code (x86) */
			st_16(buf + BS_55AA, 0xAA55);			/* Signature (placed here regardless of sector size) */
			for (i = sum = 0; i < ss; i++) {		/* VBR checksum */
				if (i != BPB_VolFlagEx && i != BPB_VolFlagEx + 1 && i != BPB_PercInUseEx) {
					sum = xsum32(buf[i], sum);
				}
			}
			if (disk_write(pdrv, buf, sect++, 1) != RES_OK) {
				LEAVE_MKFS(FR_DISK_ERR);
			}
			/* Extended bootstrap record (+1..+8) */
			memset(buf, 0, ss);
			st_16(buf + ss - 2, 0xAA55);	/* Signature (placed at end of sector) */
			for (j = 1; j < 9; j++) {
				for (i = 0; i < ss; sum = xsum32(buf[i++], sum)) ;	/* VBR checksum */
				if (disk_write(pdrv, buf, sect++, 1) != RES_OK) {
					LEAVE_MKFS(FR_DISK_ERR);
				}
			}
			/* OEM/Reserved record (+9..+10) */
			memset(buf, 0, ss);
			for ( ; j < 11; j++) {
				for (i = 0; i < ss; sum = xsum32(buf[i++], sum)) ;	/* VBR checksum */
				if (disk_write(pdrv, buf, sect++, 1) != RES_OK) {
					LEAVE_MKFS(FR_DISK_ERR);
				}
			}
			/* Sum record (+11) */
			for (i = 0; i < ss; i += 4) {
				st_32(buf + i, sum);	/* Fill with checksum value */
			}
			if (disk_write(pdrv, buf, sect++, 1) != RES_OK) {
				LEAVE_MKFS(FR_DISK_ERR);
			}
		}

	}
	else
#endif	/* FF_FS_EXFAT */
	{	/* Create an FAT/FAT32 volume */
		do {
			pau = sz_au;
			/* Pre-determine number of clusters and FAT sub-type */
			if (fsty == FS_FAT32) {	/* FAT32 volume */
				if (pau == 0) {	/* AU auto-selection */
					n = (DWORD)sz_vol / 0x20000;	/* Volume size in unit of 128KS */
					for (i = 0, pau = 1; cst32[i] && cst32[i] <= n; i++, pau <<= 1) ;	/* Get from table */
				}
				n_clst = (DWORD)sz_vol / pau;	/* Number of clusters */
				sz_fat = (n_clst * 4 + 8 + ss - 1) / ss;	/* FAT size [sector] */
				sz_rsv = 32;	/* Number of reserved sectors */
				sz_dir = 0;		/* No static directory */
				if (n_clst <= MAX_FAT16 || n_clst > MAX_FAT32) {
					LEAVE_MKFS(FR_MKFS_ABORTED);
				}
			}
			else {				/* FAT volume */
				if (pau == 0) {	/* au auto-selection */
					n = (DWORD)sz_vol / 0x1000;	/* Volume size in unit of 4KS */
					for (i = 0, pau = 1; cst[i] && cst[i] <= n; i++, pau <<= 1) ;	/* Get from table */
				}
				n_clst = (DWORD)sz_vol / pau;
				if (n_clst > MAX_FAT12) {
					n = n_clst * 2 + 4;		/* FAT size [byte] */
				}
				else {
					fsty = FS_FAT12;
					n = (n_clst * 3 + 1) / 2 + 3;	/* FAT size [byte] */
				}
				sz_fat = (n + ss - 1) / ss;		/* FAT size [sector] */
				sz_rsv = 1;						/* Number of reserved sectors */
				sz_dir = (DWORD)n_root * SZDIRE / ss;	/* Root directory size [sector] */
			}
			b_fat = b_vol + sz_rsv;						/* FAT base */
			b_data = b_fat + sz_fat * n_fat + sz_dir;	/* Data base */

			/* Align data area to erase block boundary (for flash memory media) */
			n = (DWORD)(((b_data + sz_blk - 1) & ~(sz_blk - 1)) - b_data);	/* Sectors to next nearest from current data base */
			if (fsty == FS_FAT32) {		/* FAT32: Move FAT */
				sz_rsv += n;
				b_fat += n;
			}
			else {					/* FAT: Expand FAT */
				if (n % n_fat) {	/* Adjust fractional error if needed */
					n--;
					sz_rsv++;
					b_fat++;
				}
				sz_fat += n / n_fat;
			}

			/* Determine number of clusters and final check of validity of the FAT sub-type */
			if (sz_vol < b_data + pau * 16 - b_vol) {
				LEAVE_MKFS(FR_MKFS_ABORTED);        /* Too small volume? */
			}
			n_clst = ((DWORD)sz_vol - sz_rsv - sz_fat * n_fat - sz_dir) / pau;
			if (fsty == FS_FAT32) {
				if (n_clst <= MAX_FAT16) {	/* Too few clusters for FAT32? */
					if (sz_au == 0 && (sz_au = pau / 2) != 0) {
						continue;        /* Adjust cluster size and retry */
					}
					LEAVE_MKFS(FR_MKFS_ABORTED);
				}
			}
			if (fsty == FS_FAT16) {
				if (n_clst > MAX_FAT16) {	/* Too many clusters for FAT16 */
					if (sz_au == 0 && (pau * 2) <= 64) {
						sz_au = pau * 2;
						continue;	/* Adjust cluster size and retry */
					}
					if ((fsopt & FM_FAT32)) {
						fsty = FS_FAT32;
						continue;	/* Switch type to FAT32 and retry */
					}
					if (sz_au == 0 && (sz_au = pau * 2) <= 128) {
						continue;        /* Adjust cluster size and retry */
					}
					LEAVE_MKFS(FR_MKFS_ABORTED);
				}
				if  (n_clst <= MAX_FAT12) {	/* Too few clusters for FAT16 */
					if (sz_au == 0 && (sz_au = pau * 2) <= 128) {
						continue;        /* Adjust cluster size and retry */
					}
					LEAVE_MKFS(FR_MKFS_ABORTED);
				}
			}
			if (fsty == FS_FAT12 && n_clst > MAX_FAT12) {
				LEAVE_MKFS(FR_MKFS_ABORTED);        /* Too many clusters for FAT12 */
			}

			/* Ok, it is the valid cluster configuration */
			break;
		}
		while (1);

#if FF_USE_TRIM
		lba[0] = b_vol;
		lba[1] = b_vol + sz_vol - 1;	/* Inform storage device that the volume area may be erased */
		disk_ioctl(pdrv, CTRL_TRIM, lba);
#endif
		/* Create FAT VBR */
		memset(buf, 0, ss);
		mem_cpy(buf + BS_JmpBoot, "\xEB\xFE\x90" "MSDOS5.0", 11);	/* Boot jump code (x86), OEM name */
		st_16(buf + BPB_BytsPerSec, ss);				/* Sector size [byte] */
		buf[BPB_SecPerClus] = (BYTE)pau;				/* Cluster size [sector] */
		st_16(buf + BPB_RsvdSecCnt, (WORD)sz_rsv);		/* Size of reserved area */
		buf[BPB_NumFATs] = (BYTE)n_fat;					/* Number of FATs */
		st_16(buf + BPB_RootEntCnt, (WORD)((fsty == FS_FAT32) ? 0 : n_root));	/* Number of root directory entries */
		if (sz_vol < 0x10000) {
			st_16(buf + BPB_TotSec16, (WORD)sz_vol);	/* Volume size in 16-bit LBA */
		}
		else {
			st_32(buf + BPB_TotSec32, (DWORD)sz_vol);	/* Volume size in 32-bit LBA */
		}
		buf[BPB_Media] = 0xF8;							/* Media descriptor byte */
		st_16(buf + BPB_SecPerTrk, 63);					/* Number of sectors per track (for int13) */
		st_16(buf + BPB_NumHeads, 255);					/* Number of heads (for int13) */
		st_32(buf + BPB_HiddSec, (DWORD)b_vol);			/* Volume offset in the physical drive [sector] */
		if (fsty == FS_FAT32) {
			st_32(buf + BS_VolID32, vsn);				/* VSN */
			st_32(buf + BPB_FATSz32, sz_fat);			/* FAT size [sector] */
			st_32(buf + BPB_RootClus32, 2);				/* Root directory cluster # (2) */
			st_16(buf + BPB_FSInfo32, 1);				/* Offset of FSINFO sector (VBR + 1) */
			st_16(buf + BPB_BkBootSec32, 6);			/* Offset of backup VBR (VBR + 6) */
			buf[BS_DrvNum32] = 0x80;					/* Drive number (for int13) */
			buf[BS_BootSig32] = 0x29;					/* Extended boot signature */
			mem_cpy(buf + BS_VolLab32, "NO NAME    " "FAT32   ", 19);	/* Volume label, FAT signature */
		}
		else {
			st_32(buf + BS_VolID, vsn);					/* VSN */
			st_16(buf + BPB_FATSz16, (WORD)sz_fat);		/* FAT size [sector] */
			buf[BS_DrvNum] = 0x80;						/* Drive number (for int13) */
			buf[BS_BootSig] = 0x29;						/* Extended boot signature */
			mem_cpy(buf + BS_VolLab, "NO NAME    " "FAT     ", 19);	/* Volume label, FAT signature */
		}
		st_16(buf + BS_55AA, 0xAA55);					/* Signature (offset is fixed here regardless of sector size) */
		if (disk_write(pdrv, buf, b_vol, 1) != RES_OK) {
			LEAVE_MKFS(FR_DISK_ERR);        /* Write it to the VBR sector */
		}

		/* Create FSINFO record if needed */
		if (fsty == FS_FAT32) {
			(void)disk_write(pdrv, buf, b_vol + 6, 1);		/* Write backup VBR (VBR + 6) */
			memset(buf, 0, ss);
			st_32(buf + FSI_LeadSig, 0x41615252);
			st_32(buf + FSI_StrucSig, 0x61417272);
			st_32(buf + FSI_Free_Count, n_clst - 1);	/* Number of free clusters */
			st_32(buf + FSI_Nxt_Free, 2);				/* Last allocated cluster# */
			st_16(buf + BS_55AA, 0xAA55);
			(void)disk_write(pdrv, buf, b_vol + 7, 1);		/* Write backup FSINFO (VBR + 7) */
			(void)disk_write(pdrv, buf, b_vol + 1, 1);		/* Write original FSINFO (VBR + 1) */
		}

		/* Initialize FAT area */
		memset(buf, 0, sz_buf * ss);
		sect = b_fat;		/* FAT start sector */
		for (i = 0; i < n_fat; i++) {			/* Initialize FATs each */
			if (fsty == FS_FAT32) {
				st_32(buf + 0, 0xFFFFFFF8);	/* FAT[0] */
				st_32(buf + 4, 0xFFFFFFFF);	/* FAT[1] */
				st_32(buf + 8, 0x0FFFFFFF);	/* FAT[2] (root directory at cluster# 2) */
			}
			else {
				st_32(buf + 0, (fsty == FS_FAT12) ? 0xFFFFF8 : 0xFFFFFFF8);	/* FAT[0] and FAT[1] */
			}
			nsect = sz_fat;		/* Number of FAT sectors */
			do {	/* Fill FAT sectors */
				n = (nsect > sz_buf) ? sz_buf : nsect;
				if (disk_write(pdrv, buf, sect, (UINT)n) != RES_OK) {
					LEAVE_MKFS(FR_DISK_ERR);
				}
				memset(buf, 0, ss);	/* Rest of FAT area is initially zero */
				sect += n;
				nsect -= n;
			}
			while (nsect);
		}

		/* Initialize root directory (fill with zero) */
		nsect = (fsty == FS_FAT32) ? pau : sz_dir;	/* Number of root directory sectors */
		do {
			n = (nsect > sz_buf) ? sz_buf : nsect;
			if (disk_write(pdrv, buf, sect, (UINT)n) != RES_OK) {
				LEAVE_MKFS(FR_DISK_ERR);
			}
			sect += n;
			nsect -= n;
		}
		while (nsect);
	}

	/* A FAT volume has been created here */

	/* Determine system ID in the MBR partition table */
	if (FF_FS_EXFAT && fsty == FS_EXFAT) {
		sys = 0x07;		/* exFAT */
	}
	else if (fsty == FS_FAT32) {
		sys = 0x0C;		/* FAT32X */
	}
	else if (sz_vol >= 0x10000) {
		sys = 0x06;		/* FAT12/16 (large) */
	}
	else if (fsty == FS_FAT16) {
		sys = 0x04;		/* FAT16 */
	}
	else {
		sys = 0x01;		/* FAT12 */
	}

	/* Update partition information */
	if (FF_MULTI_PARTITION && ipart != 0) {	/* Volume is in the existing partition */
		if (!FF_LBA64 || !(fsopt & 0x80)) {	/* Is the partition in MBR? */
			/* Update system ID in the partition table */
			if (disk_read(pdrv, buf, 0, 1) != RES_OK) {
				LEAVE_MKFS(FR_DISK_ERR);        /* Read the MBR */
			}
			buf[MBR_Table + (ipart - 1) * SZ_PTE + PTE_System] = sys;			/* Set system ID */
			if (disk_write(pdrv, buf, 0, 1) != RES_OK) {
				LEAVE_MKFS(FR_DISK_ERR);        /* Write it back to the MBR */
			}
		}
	}
	else {								/* Volume as a new single partition */
		if (!(fsopt & FM_SFD)) {			/* Create partition table if not in SFD format */
			lba[0] = sz_vol;
			lba[1] = 0;
			res = create_partition(pdrv, lba, sys, buf);
			if (res != FR_OK) {
				LEAVE_MKFS(res);
			}
		}
	}

	if (disk_ioctl(pdrv, CTRL_SYNC, 0) != RES_OK) {
		LEAVE_MKFS(FR_DISK_ERR);
	}

	LEAVE_MKFS(FR_OK);
}




#if FF_MULTI_PARTITION
/*-----------------------------------------------------------------------*/
/* API: Create Partition Table on the Physical Drive                     */
/*-----------------------------------------------------------------------*/

FRESULT f_fdisk (
	BYTE pdrv,			/* Physical drive number */
	const LBA_t ptbl[],	/* Pointer to the size table for each partitions */
	void *work			/* Pointer to the working buffer (null: use heap memory) */
)
{
	BYTE *buf = (BYTE *)work;
	DSTATUS stat;
	FRESULT res = FR_DISK_ERR;


	/* Initialize the physical drive */
	stat = disk_initialize(pdrv);
	if (stat & STA_NOINIT) {
		return FR_NOT_READY;
	}
	if (stat & STA_PROTECT) {
		return FR_WRITE_PROTECTED;
	}

#if FF_USE_LFN == 3
	if (!buf) {
		buf = ff_memalloc(FF_MAX_SS);        /* Use heap memory for working buffer */
	}
#endif
	if (!buf) {
		return FR_NOT_ENOUGH_CORE;
	}

	res = create_partition(pdrv, ptbl, 0x07,
			       buf);	/* Create partitions (system ID is temporary setting and determined by f_mkfs) */

	LEAVE_MKFS(res);
}

#endif /* FF_MULTI_PARTITION */
#endif /* !FF_FS_READONLY && FF_USE_MKFS */




#if FF_USE_STRFUNC
#if FF_USE_LFN && FF_LFN_UNICODE && (FF_STRF_ENCODE < 0 || FF_STRF_ENCODE > 3)
#error Wrong FF_STRF_ENCODE setting
#endif
/*-----------------------------------------------------------------------*/
/* API: Get a String from the File                                       */
/*-----------------------------------------------------------------------*/

TCHAR *f_gets (
	TCHAR *buff,	/* Pointer to the buffer to store read string */
	int len,		/* Size of string buffer (items) */
	FIL *fp			/* Pointer to the file object */
)
{
	int nc = 0;
	TCHAR *p = buff;
	BYTE s[4];
	UINT rc;
	DWORD dc;
#if FF_USE_LFN && FF_LFN_UNICODE && FF_STRF_ENCODE <= 2
	WCHAR wc;
#endif
#if FF_USE_LFN && FF_LFN_UNICODE && FF_STRF_ENCODE == 3
	UINT ct;
#endif

#if FF_USE_LFN && FF_LFN_UNICODE			/* With code conversion (Unicode API) */
	/* Make a room for the character and terminator  */
	if (FF_LFN_UNICODE == 1) {
		len -= (FF_STRF_ENCODE == 0) ? 1 : 2;
	}
	if (FF_LFN_UNICODE == 2) {
		len -= (FF_STRF_ENCODE == 0) ? 3 : 4;
	}
	if (FF_LFN_UNICODE == 3) {
		len -= 1;
	}
	while (nc < len) {
#if FF_STRF_ENCODE == 0				/* Read a character in ANSI/OEM */
		f_read(fp, s, 1, &rc);		/* Get a code unit */
		if (rc != 1) {
			break;        /* EOF? */
		}
		wc = s[0];
		if (dbc_1st((BYTE)wc)) {	/* DBC 1st byte? */
			f_read(fp, s, 1, &rc);	/* Get 2nd byte */
			if (rc != 1 || !dbc_2nd(s[0])) {
				continue;        /* Wrong code? */
			}
			wc = wc << 8 | s[0];
		}
		dc = ff_oem2uni(wc, CODEPAGE);	/* Convert ANSI/OEM into Unicode */
		if (dc == 0) {
			continue;        /* Conversion error? */
		}
#elif FF_STRF_ENCODE == 1 || FF_STRF_ENCODE == 2 	/* Read a character in UTF-16LE/BE */
		f_read(fp, s, 2, &rc);		/* Get a code unit */
		if (rc != 2) {
			break;        /* EOF? */
		}
		dc = (FF_STRF_ENCODE == 1) ? ld_16(s) : s[0] << 8 | s[1];
		if (IsSurrogateL(dc)) {
			continue;        /* Broken surrogate pair? */
		}
		if (IsSurrogateH(dc)) {		/* High surrogate? */
			f_read(fp, s, 2, &rc);	/* Get low surrogate */
			if (rc != 2) {
				break;        /* EOF? */
			}
			wc = (FF_STRF_ENCODE == 1) ? ld_16(s) : s[0] << 8 | s[1];
			if (!IsSurrogateL(wc)) {
				continue;        /* Broken surrogate pair? */
			}
			dc = ((dc & 0x3FF) + 0x40) << 10 | (wc & 0x3FF);	/* Merge surrogate pair */
		}
#else	/* Read a character in UTF-8 */
		f_read(fp, s, 1, &rc);		/* Get a code unit */
		if (rc != 1) {
			break;        /* EOF? */
		}
		dc = s[0];
		if (dc >= 0x80) {			/* Multi-byte sequence? */
			ct = 0;
			if ((dc & 0xE0) == 0xC0) {	/* 2-byte sequence? */
				dc &= 0x1F;
				ct = 1;
			}
			if ((dc & 0xF0) == 0xE0) {	/* 3-byte sequence? */
				dc &= 0x0F;
				ct = 2;
			}
			if ((dc & 0xF8) == 0xF0) {	/* 4-byte sequence? */
				dc &= 0x07;
				ct = 3;
			}
			if (ct == 0) {
				continue;
			}
			f_read(fp, s, ct, &rc);	/* Get trailing bytes */
			if (rc != ct) {
				break;
			}
			rc = 0;
			do {	/* Merge the byte sequence */
				if ((s[rc] & 0xC0) != 0x80) {
					break;
				}
				dc = dc << 6 | (s[rc] & 0x3F);
			}
			while (++rc < ct);
			if (rc != ct || dc < 0x80 || IsSurrogate(dc) || dc >= 0x110000) {
				continue;        /* Wrong encoding? */
			}
		}
#endif
		/* A code point is available in dc to be output */

		if (FF_USE_STRFUNC == 2 && dc == '\r') {
			continue;        /* Strip \r off if needed */
		}
#if FF_LFN_UNICODE == 1	|| FF_LFN_UNICODE == 3	/* Output it in UTF-16/32 encoding */
		if (FF_LFN_UNICODE == 1 && dc >= 0x10000) {	/* Out of BMP at UTF-16? */
			*p++ = (TCHAR)(0xD800 | ((dc >> 10) - 0x40));
			nc++;	/* Make and output high surrogate */
			dc = 0xDC00 | (dc & 0x3FF);		/* Make low surrogate */
		}
		*p++ = (TCHAR)dc;
		nc++;
		if (dc == '\n') {
			break;        /* End of line? */
		}
#elif FF_LFN_UNICODE == 2		/* Output it in UTF-8 encoding */
		if (dc < 0x80) {	/* Single byte? */
			*p++ = (TCHAR)dc;
			nc++;
			if (dc == '\n') {
				break;        /* End of line? */
			}
		}
		else if (dc < 0x800) {	/* 2-byte sequence? */
			*p++ = (TCHAR)(0xC0 | (dc >> 6 & 0x1F));
			*p++ = (TCHAR)(0x80 | (dc >> 0 & 0x3F));
			nc += 2;
		}
		else if (dc < 0x10000) {	/* 3-byte sequence? */
			*p++ = (TCHAR)(0xE0 | (dc >> 12 & 0x0F));
			*p++ = (TCHAR)(0x80 | (dc >> 6 & 0x3F));
			*p++ = (TCHAR)(0x80 | (dc >> 0 & 0x3F));
			nc += 3;
		}
		else {					/* 4-byte sequence */
			*p++ = (TCHAR)(0xF0 | (dc >> 18 & 0x07));
			*p++ = (TCHAR)(0x80 | (dc >> 12 & 0x3F));
			*p++ = (TCHAR)(0x80 | (dc >> 6 & 0x3F));
			*p++ = (TCHAR)(0x80 | (dc >> 0 & 0x3F));
			nc += 4;
		}
#endif
	}

#else			/* Byte-by-byte read without any conversion (ANSI/OEM API) */
	len -= 1;	/* Make a room for the terminator */
	while (nc < len) {
		f_read(fp, s, 1, &rc);	/* Get a byte */
		if (rc != 1) {
			break;        /* EOF? */
		}
		dc = s[0];
		if (FF_USE_STRFUNC == 2 && dc == '\r') {
			continue;
		}
		*p++ = (TCHAR)dc;
		nc++;
		if (dc == '\n') {
			break;
		}
	}
#endif

	*p = 0;		/* Terminate the string */
	return nc ? buff : 0;	/* When no data read due to EOF or error, return with error. */
}




#if !FF_FS_READONLY
#include <stdarg.h>
#define SZ_PUTC_BUF	64
#define SZ_NUM_BUF	32

/*-----------------------------------------------------------------------*/
/* API: Put a Character to the File (with sub-functions)                 */
/*-----------------------------------------------------------------------*/

/* Output buffer and work area */

typedef struct {
	FIL *fp;		/* Pointer to the writing file */
	int idx, nchr;	/* Write index of buf[] (-1:error), number of encoding units written */
#if FF_USE_LFN && FF_LFN_UNICODE == 1
	WCHAR hs;
#elif FF_USE_LFN && FF_LFN_UNICODE == 2
	BYTE bs[4];
	UINT wi, ct;
#endif
	BYTE buf[SZ_PUTC_BUF];	/* Write buffer */
} putbuff;


/* Buffered file write with code conversion */

static void putc_bfd (putbuff *pb, TCHAR c)
{
	UINT n;
	int i, nc;
#if FF_USE_LFN && FF_LFN_UNICODE
	WCHAR hs, wc;
#if FF_LFN_UNICODE == 2
	DWORD dc;
	const TCHAR *tp;
#endif
#endif

	if (FF_USE_STRFUNC == 2 && c == '\n') {	 /* LF -> CRLF conversion */
		putc_bfd(pb, '\r');
	}

	i = pb->idx;			/* Write index of pb->buf[] */
	if (i < 0) {
		return;        /* In write error? */
	}
	nc = pb->nchr;			/* Write unit counter */

#if FF_USE_LFN && FF_LFN_UNICODE
#if FF_LFN_UNICODE == 1		/* UTF-16 input */
	if (IsSurrogateH(c)) {	/* Is this a high-surrogate? */
		pb->hs = c;
		return;	/* Save it for next */
	}
	hs = pb->hs;
	pb->hs = 0;
	if (hs != 0) {			/* Is there a leading high-surrogate? */
		if (!IsSurrogateL(c)) {
			hs = 0;        /* Discard high-surrogate if a stray high-surrogate */
		}
	}
	else {
		if (IsSurrogateL(c)) {
			return;        /* Discard stray low-surrogate */
		}
	}
	wc = c;
#elif FF_LFN_UNICODE == 2	/* UTF-8 input */
	for (;;) {
		if (pb->ct == 0) {	/* Not in the multi-byte sequence? */
			pb->bs[pb->wi = 0] = (BYTE)c;	/* Save 1st byte */
			if ((BYTE)c < 0x80) {
				break;        /* Single byte code? */
			}
			if (((BYTE)c & 0xE0) == 0xC0) {
				pb->ct = 1;        /* 2-byte sequence? */
			}
			if (((BYTE)c & 0xF0) == 0xE0) {
				pb->ct = 2;        /* 3-byte sequence? */
			}
			if (((BYTE)c & 0xF8) == 0xF0) {
				pb->ct = 3;        /* 4-byte sequence? */
			}
			return;										/* Invalid leading byte (discard it) */
		}
		else {				/* In the multi-byte sequence */
			if (((BYTE)c & 0xC0) != 0x80) {	/* Broken sequence? */
				pb->ct = 0;
				continue;		/* Discard the sequence */
			}
			pb->bs[++pb->wi] = (BYTE)c;	/* Save the trailing byte */
			if (--pb->ct == 0) {
				break;        /* End of the sequence? */
			}
			return;
		}
	}
	tp = (const TCHAR *)pb->bs;
	dc = tchar2uni(&tp);			/* UTF-8 ==> UTF-16 */
	if (dc == 0xFFFFFFFF) {
		return;        /* Wrong code? */
	}
	hs = (WCHAR)(dc >> 16);
	wc = (WCHAR)dc;
#elif FF_LFN_UNICODE == 3	/* UTF-32 input */
	if (IsSurrogate(c) || c >= 0x110000) {
		return;        /* Discard invalid code */
	}
	if (c >= 0x10000) {		/* Out of BMP? */
		hs = (WCHAR)(0xD800 | ((c >> 10) - 0x40)); 	/* Make high surrogate */
		wc = 0xDC00 | (c & 0x3FF);					/* Make low surrogate */
	}
	else {
		hs = 0;
		wc = (WCHAR)c;
	}
#endif
	/* A code point in UTF-16 is available in hs and wc */

#if FF_STRF_ENCODE == 1		/* Write a code point in UTF-16LE */
	if (hs != 0) {	/* Surrogate pair? */
		st_16(&pb->buf[i], hs);
		i += 2;
		nc++;
	}
	st_16(&pb->buf[i], wc);
	i += 2;
#elif FF_STRF_ENCODE == 2	/* Write a code point in UTF-16BE */
	if (hs != 0) {	/* Surrogate pair? */
		pb->buf[i++] = (BYTE)(hs >> 8);
		pb->buf[i++] = (BYTE)hs;
		nc++;
	}
	pb->buf[i++] = (BYTE)(wc >> 8);
	pb->buf[i++] = (BYTE)wc;
#elif FF_STRF_ENCODE == 3	/* Write a code point in UTF-8 */
	if (hs != 0) {	/* 4-byte sequence? */
		nc += 3;
		hs = (hs & 0x3FF) + 0x40;
		pb->buf[i++] = (BYTE)(0xF0 | hs >> 8);
		pb->buf[i++] = (BYTE)(0x80 | (hs >> 2 & 0x3F));
		pb->buf[i++] = (BYTE)(0x80 | (hs & 3) << 4 | (wc >> 6 & 0x0F));
		pb->buf[i++] = (BYTE)(0x80 | (wc & 0x3F));
	}
	else {
		if (wc < 0x80) {	/* Single byte? */
			pb->buf[i++] = (BYTE)wc;
		}
		else {
			if (wc < 0x800) {	/* 2-byte sequence? */
				nc += 1;
				pb->buf[i++] = (BYTE)(0xC0 | wc >> 6);
			}
			else {			/* 3-byte sequence */
				nc += 2;
				pb->buf[i++] = (BYTE)(0xE0 | wc >> 12);
				pb->buf[i++] = (BYTE)(0x80 | (wc >> 6 & 0x3F));
			}
			pb->buf[i++] = (BYTE)(0x80 | (wc & 0x3F));
		}
	}
#else						/* Write a code point in ANSI/OEM */
	if (hs != 0) {
		return;
	}
	wc = ff_uni2oem(wc, CODEPAGE);	/* UTF-16 ==> ANSI/OEM */
	if (wc == 0) {
		return;
	}
	if (wc >= 0x100) {
		pb->buf[i++] = (BYTE)(wc >> 8);
		nc++;
	}
	pb->buf[i++] = (BYTE)wc;
#endif

#else							/* ANSI/OEM input (without re-encoding) */
	pb->buf[i++] = (BYTE)c;
#endif

	if (i >= (int)(sizeof pb->buf) - 4) {	/* Write buffered characters to the file */
		f_write(pb->fp, pb->buf, (UINT)i, &n);
		i = (n == (UINT)i) ? 0 : -1;
	}
	pb->idx = i;
	pb->nchr = nc + 1;
}


/* Flush characters left in the buffer and return number of characters written */

static int putc_flush (putbuff *pb)
{
	UINT nw;

	if (   pb->idx >= 0	/* Flush buffered characters to the file */
	       && f_write(pb->fp, pb->buf, (UINT)pb->idx, &nw) == FR_OK
	       && (UINT)pb->idx == nw) {
		return pb->nchr;
	}
	return -1;
}


/* Initialize write buffer */

static void putc_init (putbuff *pb, FIL *fp)
{
	memset(pb, 0, sizeof (putbuff));
	pb->fp = fp;
}



int f_putc (
	TCHAR c,	/* A character to be output */
	FIL *fp		/* Pointer to the file object */
)
{
	putbuff pb;


	putc_init(&pb, fp);
	putc_bfd(&pb, c);	/* Put the character */
	return putc_flush(&pb);
}




/*-----------------------------------------------------------------------*/
/* API: Put a String to the File                                         */
/*-----------------------------------------------------------------------*/

int f_puts (
	const TCHAR *str,	/* Pointer to the string to be output */
	FIL *fp				/* Pointer to the file object */
)
{
	putbuff pb;


	putc_init(&pb, fp);
	while (*str) {
		putc_bfd(&pb, *str++);        /* Put the string */
	}
	return putc_flush(&pb);
}




/*-----------------------------------------------------------------------*/
/* API: Put a Formatted String to the File (with sub-functions)          */
/*-----------------------------------------------------------------------*/
#if FF_PRINT_FLOAT && FF_INTDEF == 2
#include <math.h>

static int ilog10 (double n)	/* Calculate log10(n) in integer output */
{
	int rv = 0;

	while (n >= 10) {	/* Decimate digit in right shift */
		if (n >= 100000) {
			n /= 100000;
			rv += 5;
		}
		else {
			n /= 10;
			rv++;
		}
	}
	while (n < 1) {		/* Decimate digit in left shift */
		if (n < 0.00001) {
			n *= 100000;
			rv -= 5;
		}
		else {
			n *= 10;
			rv--;
		}
	}
	return rv;
}


static double i10x (int n)	/* Calculate 10^n in integer input */
{
	double rv = 1;

	while (n > 0) {		/* Left shift */
		if (n >= 5) {
			rv *= 100000;
			n -= 5;
		}
		else {
			rv *= 10;
			n--;
		}
	}
	while (n < 0) {		/* Right shift */
		if (n <= -5) {
			rv /= 100000;
			n += 5;
		}
		else {
			rv /= 10;
			n++;
		}
	}
	return rv;
}


static void ftoa (
	char *buf,	/* Buffer to output the floating point string */
	double val,	/* Value to output */
	int prec,	/* Number of fractional digits */
	TCHAR fmt	/* Notation */
)
{
	int digit;
	int exp = 0, mag = 0;
	char sign = 0;
	double w;
	const char *er = 0;
	const char ds = FF_PRINT_FLOAT == 2 ? ',' : '.';


	if (isnan(val)) {			/* Not a number? */
		er = "NaN";
	}
	else {
		if (prec < 0) {
			prec = 6;        /* Default precision? (6 fractional digits) */
		}
		if (val < 0) {			/* Negative? */
			val = 0 - val;
			sign = '-';
		}
		else {
			sign = '+';
		}
		if (isinf(val)) {		/* Infinite? */
			er = "INF";
		}
		else {
			if (fmt == 'f') {	/* Decimal notation? */
				val += i10x(0 - prec) / 2;	/* Round (nearest) */
				mag = ilog10(val);
				if (mag < 0) {
					mag = 0;
				}
				if (mag + prec + 3 >= SZ_NUM_BUF) {
					er = "OV";        /* Buffer overflow? */
				}
			}
			else {			/* E notation */
				if (val != 0) {		/* Not a true zero? */
					val += i10x(ilog10(val) - prec) / 2;	/* Round (nearest) */
					exp = ilog10(val);
					if (exp > 99 || prec + 7 >= SZ_NUM_BUF) {	/* Buffer overflow or E > +99? */
						er = "OV";
					}
					else {
						if (exp < -99) {
							exp = -99;
						}
						val /= i10x(exp);	/* Normalize */
					}
				}
			}
		}
		if (!er) {	/* Not error condition */
			if (sign == '-') {
				*buf++ = sign;        /* Add a - if negative value */
			}
			do {				/* Put decimal number */
				if (mag == -1) {
					*buf++ = ds;        /* Insert a decimal separator when get into fractional part */
				}
				w = i10x(mag);				/* Snip the highest digit d */
				digit = (int)(val / w);
				val -= digit * w;
				*buf++ = (char)('0' + digit);	/* Put the digit */
			}
			while (--mag >= -prec);		/* Output all digits specified by prec */
			if (fmt != 'f') {	/* Put exponent if needed */
				*buf++ = (char)fmt;
				if (exp < 0) {
					exp = 0 - exp;
					*buf++ = '-';
				}
				else {
					*buf++ = '+';
				}
				*buf++ = (char)('0' + exp / 10);
				*buf++ = (char)('0' + exp % 10);
			}
		}
	}
	if (er) {	/* Error condition */
		if (sign) {
			*buf++ = sign;        /* Add sign if needed */
		}
		do {		/* Put error symbol */
			*buf++ = *er++;
		}
		while (*er);
	}
	*buf = 0;	/* Term */
}
#endif	/* FF_PRINT_FLOAT && FF_INTDEF == 2 */



int f_printf (
	FIL *fp,			/* Pointer to the file object */
	const TCHAR *fmt,	/* Pointer to the format string */
	...					/* Optional arguments... */
)
{
	va_list arp;
	putbuff pb;
	UINT i, j, width, flag, radix;
	int prec;
#if FF_PRINT_LLI && FF_INTDEF == 2
	QWORD val;
#else
	DWORD val;
#endif
	TCHAR *tp;
	TCHAR chr, pad;
	TCHAR nul = 0;
	char digit, str[SZ_NUM_BUF];


	putc_init(&pb, fp);

	va_start(arp, fmt);

	for (;;) {
		chr = *fmt++;
		if (chr == 0) break;		/* End of format string */
		if (chr != '%') {			/* Not an escape character (pass-through) */
			putc_bfd(&pb, chr);
			continue;
		}
		flag = width = 0; pad = ' '; prec = -1;	/* Initialize the parameters */
		chr = *fmt++;
		if (chr == '0') {			/* Flag: '0' padded */
			pad = '0'; chr = *fmt++;
		} else if (chr == '-') {	/* Flag: Left aligned */
			flag = 2; chr = *fmt++;
		}
		if (chr == '*') {			/* Minimum width from an argument */
			width = (UINT)va_arg(arp, int);
			chr = *fmt++;
		} else {
			while (IsDigit(chr)) {	/* Minimum width */
				width = width * 10 + chr - '0';
				chr = *fmt++;
			}
		}
		if (chr == '.') {			/* Precision */
			chr = *fmt++;
			if (chr == '*') {		/* Precision from an argument */
				prec = va_arg(arp, int);
				chr = *fmt++;
			} else {
				prec = 0;
				while (IsDigit(chr)) {	/* Precision */
					prec = prec * 10 + chr - '0';
					chr = *fmt++;
				}
			}
		}
		if (chr == 'l') {			/* Size: long int */
			flag |= 4; chr = *fmt++;
#if FF_PRINT_LLI && FF_INTDEF == 2
			if (chr == 'l') {		/* Size: long long int */
				flag |= 8; chr = *fmt++;
			}
#endif
		}
		if (chr == 0) break;		/* End of format string */
		switch (chr) {				/* Atgument type is... */
		case 'b':					/* Unsigned binary */
			radix = 2; break;

		case 'o':					/* Unsigned octal */
			radix = 8; break;

		case 'd':					/* Signed decimal */
		case 'u': 					/* Unsigned decimal */
			radix = 10; break;

		case 'x':					/* Unsigned hexadecimal (lower case) */
		case 'X': 					/* Unsigned hexadecimal (upper case) */
			radix = 16; break;

		case 'c':					/* Character */
			putc_bfd(&pb, (TCHAR)va_arg(arp, int));
			continue;

		case 's':					/* String */
			tp = va_arg(arp, TCHAR*);	/* Get a pointer argument */
			if (!tp) tp = &nul;			/* Null pointer generates a null string */
			for (j = 0; tp[j]; j++) ;	/* j = tcslen(tp) */
			if (prec >= 0 && j > (UINT)prec) j = (UINT)prec;	/* Limited length of string body */
			for ( ; !(flag & 2) && j < width; j++) putc_bfd(&pb, pad);	/* Left padding */
			while (*tp && prec--) putc_bfd(&pb, *tp++);		/* Body */
			while (j++ < width) putc_bfd(&pb, ' ');			/* Right padding */
			continue;
#if FF_PRINT_FLOAT && FF_INTDEF == 2
		case 'f':					/* Floating point (decimal) */
		case 'e':					/* Floating point (e) */
		case 'E':					/* Floating point (E) */
			ftoa(str, va_arg(arp, double), prec, chr);		/* Make a floating point string */
			for (j = strlen(str); !(flag & 2) && j < width; j++) putc_bfd(&pb, pad);	/* Leading pads */
			for (i = 0; str[i]; putc_bfd(&pb, str[i++])) ;	/* Body */
			while (j++ < width) putc_bfd(&pb, ' ');			/* Trailing pads */
			continue;
#endif
		default:					/* Unknown type (pass-through) */
			putc_bfd(&pb, chr);
			continue;
		}

		/* Get an integer argument and put it in numeral */
#if FF_PRINT_LLI && FF_INTDEF == 2
		if (flag & 8) {		/* long long argument? */
			val = (QWORD)va_arg(arp, long long);
		} else if (flag & 4) {	/* long argument? */
			val = (chr == 'd') ? (QWORD)(long long)va_arg(arp, long) : (QWORD)va_arg(arp, unsigned long);
		} else {			/* int/short/char argument */
			val = (chr == 'd') ? (QWORD)(long long)va_arg(arp, int) : (QWORD)va_arg(arp, unsigned int);
		}
		if (chr == 'd' && (val & 0x8000000000000000)) {	/* Negative value? */
			val = 0 - val; flag |= 1;
		}
#else
		if (flag & 4) {	/* long argument? */
			val = (DWORD)va_arg(arp, long);
		} else {		/* int/short/char argument */
			val = (chr == 'd') ? (DWORD)(long)va_arg(arp, int) : (DWORD)va_arg(arp, unsigned int);
		}
		if (chr == 'd' && (val & 0x80000000)) {	/* Negative value? */
			val = 0 - val; flag |= 1;
		}
#endif
		i = 0;
		do {	/* Make an integer number string */
			digit = (char)(val % radix) + '0'; val /= radix;
			if (digit > '9') digit += (chr == 'x') ? 0x27 : 0x07;
			str[i++] = digit;
		} while (val && i < SZ_NUM_BUF);
		if (flag & 1) str[i++] = '-';	/* Sign */
		/* Write it */
		for (j = i; !(flag & 2) && j < width; j++) {	/* Leading pads */
			putc_bfd(&pb, pad);
		}
		do {					/* Body */
			putc_bfd(&pb, (TCHAR)str[--i]);
		} while (i);
		while (j++ < width) {	/* Trailing pads */
			putc_bfd(&pb, ' ');
		}
	}

	va_end(arp);

	return putc_flush(&pb);
}

#endif /* !FF_FS_READONLY */
#endif /* FF_USE_STRFUNC */



#if FF_CODE_PAGE == 0
/*-----------------------------------------------------------------------*/
/* API: Set Active Codepage for the Path Name                            */
/*-----------------------------------------------------------------------*/

FRESULT f_setcp (
	WORD cp		/* Value to be set as active code page */
)
{
	static const WORD       validcp[22] = {  437,   720,   737,   771,   775,   850,   852,   855,   857,   860,   861,   862,   863,   864,   865,   866,   869,   932,   936,   949,   950, 0};
	static const BYTE *const tables[22] = {Ct437, Ct720, Ct737, Ct771, Ct775, Ct850, Ct852, Ct855, Ct857, Ct860, Ct861, Ct862, Ct863, Ct864, Ct865, Ct866, Ct869, Dc932, Dc936, Dc949, Dc950, 0};
	UINT i;


	for (i = 0; validcp[i] != 0 && validcp[i] != cp; i++) ;	/* Find the code page */
	if (validcp[i] != cp) {
		return FR_INVALID_PARAMETER;        /* Not found? */
	}

	CodePage = cp;
	if (cp >= 900) {	/* DBCS */
		ExCvt = 0;
		DbcTbl = tables[i];
	}
	else {			/* SBCS */
		ExCvt = tables[i];
		DbcTbl = 0;
	}
	return FR_OK;
}
#endif	/* FF_CODE_PAGE == 0 */

#ifdef XPAR_XUFSPSXC_NUM_INSTANCES
/*---------------------------------------------------------------------------*/
/* Interface used to access the Low level device for specific functionality  */
/*---------------------------------------------------------------------------*/
FRESULT f_ioctl (const TCHAR *path, BYTE Cmd, void *buff)
{
	int vol;
	FATFS *fs;

	vol = get_ldnumber(&path);					/* Get target logical drive */
	if (vol < 0) {
		return FR_INVALID_DRIVE;
	}

	fs = FatFs[vol];

	if (disk_ioctl(fs->pdrv, Cmd, buff) != RES_OK) {
		return FR_DISK_ERR;
	}

	return FR_OK;
}
#endif

#endif /* (defined FILE_SYSTEM_INTERFACE_SD) || (defined FILE_SYSTEM_INTERFACE_RAM) */
